@@ -6,14 +6,23 @@
 # Author: Docbroke
 # License: GPL3
 
+# Dependencies: fzf, find
+
+if [ "$1" = "-h" ] || [ "$1" = "--help" ]; then
+	name="${CLIFM_PLUGIN_NAME:-$(basename "$0")}"
+	printf "Find and change the current working directory via FZF\n"
+	printf "\n\x1b[1mUSAGE\x1b[0m\n  %s\n" "$name"
+	exit 0
+fi
+
 if ! type fzf > /dev/null 2>&1; then
-	printf "%s" "CliFM: fzf: Command not found\n" >&2
-	exit 1
+	printf "%s" "clifm: fzf: Command not found\n" >&2
+	exit 127
 fi
 
 # Source our plugins helper
 if [ -z "$CLIFM_PLUGINS_HELPER" ] || ! [ -f "$CLIFM_PLUGINS_HELPER" ]; then
-	printf "CliFM: Unable to find plugins-helper file\n" >&2
+	printf "clifm: Unable to find plugins-helper file\n" >&2
 	exit 1
 fi
 # shellcheck source=/dev/null
@@ -22,7 +31,7 @@ fi
 # shellcheck disable=SC2154
 DIR="$(find / -type d -print0 2> /dev/null | \
 fzf --read0 --prompt "$fzf_prompt" \
---reverse --height 15 --header "Fuzzy directory changer" \
+--reverse --height "$fzf_height" --header "Fuzzy directory changer" \
 --bind "tab:accept" --info=inline \
 --color="$(get_fzf_colors)")"
 
