@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -30,20 +30,29 @@
 #include <string.h>
 #include <readline/readline.h>
 
+#if defined(_NO_MAGIC) && !defined(_BE_POSIX)
+# include <paths.h>
+# ifndef _PATH_DEVNULL
+#  define _PATH_DEVNULL "/dev/null"
+# endif /* _PATH_DEVNULL */
+#else
+# define _PATH_DEVNULL "/dev/null"
+#endif /* _NO_MAGIC && !_BE_POSIX */
+
 #include "aux.h"
 #include "checks.h"
-#include "exec.h"
 #include "history.h"
 #include "jump.h"
 #include "listing.h"
 #include "misc.h"
 #include "navigation.h"
 #include "readline.h"
+#include "spawn.h"
 
 #ifndef _NO_MAGIC
 # include "mime.h"
 #else
-# include <unistd.h>
+# include <unistd.h> /* dup(), dup2(), unlink(), STDOUT_FILENO */
 #endif /* !_NO_MAGIC */
 
 #define OP_ISO    1
@@ -52,20 +61,29 @@
 static char *
 ask_user_for_path(void)
 {
-	char *ext_path = (char *)NULL;
+	const char *m = "Extraction path ('q' to quit): ";
 
-	while (!ext_path) {
-		int bk = conf.warning_prompt;
-		conf.warning_prompt = 0;
-		ext_path = readline(_("Extraction path ('q' to quit): "));
-		conf.warning_prompt = bk;
-		if (!ext_path)
-			continue;
-		if (!*ext_path) {
-			free(ext_path);
-			ext_path = (char *)NULL;
-			continue;
-		}
+	const int poffset_bk = prompt_offset;
+	prompt_offset = (int)strlen(m) + 1;
+	xrename = rl_nohist = 1;
+
+	char *ext_path = secondary_prompt(m, NULL);
+
+	xrename = rl_nohist = 0;
+	prompt_offset = poffset_bk;
+
+	if (!ext_path)
+		return (char *)NULL;
+
+	char *p = (char *)NULL;
+	if ((*ext_path == '"' || *ext_path == '\'')
+	&& (p = remove_quotes(ext_path)))
+		memmove(ext_path, p, strlen(p) + 1);
+
+	char *unesc_path = unescape_str(ext_path, 0);
+	if (unesc_path) {
+		free(ext_path);
+		ext_path = unesc_path;
 	}
 
 	return ext_path;
@@ -143,7 +161,7 @@ get_operation(const int mode)
 		case 'q': /* fallthrough */
 		case 'Q':
 			free(op);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 
 		default:
 			free(op);
@@ -161,17 +179,17 @@ get_operation(const int mode)
 static int
 extract_iso(char *file)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	/* 7z x -oDIR FILE (use FILE as DIR) */
-	size_t flen = strlen(file);
-	char *o_option = (char *)xnmalloc(flen + 7, sizeof(char));
+	const size_t flen = strlen(file);
+	char *o_option = xnmalloc(flen + 7, sizeof(char));
 	snprintf(o_option, flen + 7, "-o%s.dir", file); /* NOLINT */
 
 	/* Construct and execute cmd */
 	char *cmd[] = {"7z", "x", o_option, file, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	free(o_option);
 	return exit_status;
@@ -181,21 +199,21 @@ static int
 extract_iso_to_dir(char *file)
 {
 	/* 7z x -oDIR FILE (ask for DIR) */
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	char *ext_path = get_extraction_path();
 	if (!ext_path)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	size_t len = strlen(ext_path);
-	char *o_option = (char *)xnmalloc(len + 3, sizeof(char));
+	const size_t len = strlen(ext_path);
+	char *o_option = xnmalloc(len + 3, sizeof(char));
 	snprintf(o_option, len + 3, "-o%s", ext_path); /* NOLINT */
 	free(ext_path);
 
 	/* Construct and execute cmd */
 	char *cmd[] = {"7z", "x", o_option, file, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	free(o_option);
 
@@ -205,12 +223,12 @@ extract_iso_to_dir(char *file)
 static int
 list_iso_contents(char *file)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	/* 7z l FILE */
 	char *cmd[] = {"7z", "l", file, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	return exit_status;
 }
@@ -218,18 +236,18 @@ list_iso_contents(char *file)
 static int
 test_iso(char *file)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	/* 7z t FILE */
 	char *cmd[] = {"7z", "t", file, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	return exit_status;
 }
 
 /* Create mountpoint for file. Returns the path to the mountpoint in case
- * of success, or NULL in case of error */
+ * of success, or NULL in case of error. */
 static char *
 create_mountpoint(char *file)
 {
@@ -238,17 +256,17 @@ create_mountpoint(char *file)
 	char *tfile = (p && *(++p)) ? p : file;
 
 	if (xargs.stealth_mode == 1) {
-		size_t len = strlen(tfile) + P_tmpdir_len + 15;
-		mountpoint = (char *)xnmalloc(len, sizeof(char));
+		const size_t len = strlen(tfile) + P_tmpdir_len + 15;
+		mountpoint = xnmalloc(len, sizeof(char));
 		snprintf(mountpoint, len, "%s/clifm-mounts/%s", P_tmpdir, tfile);
 	} else {
-		size_t len = config_dir_len + strlen(tfile) + 9;
-		mountpoint = (char *)xnmalloc(len, sizeof(char));
+		const size_t len = config_dir_len + strlen(tfile) + 9;
+		mountpoint = xnmalloc(len, sizeof(char));
 		snprintf(mountpoint, len, "%s/mounts/%s", config_dir, tfile); /* NOLINT */
 	}
 
 	char *dir_cmd[] = {"mkdir", "-pm700", mountpoint, NULL};
-	if (launch_execv(dir_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+	if (launch_execv(dir_cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS) {
 		free(mountpoint);
 		mountpoint = (char *)NULL;
 	}
@@ -261,20 +279,20 @@ static int
 cd_to_mountpoint(char *file, char *mountpoint)
 {
 	if (xchdir(mountpoint, SET_TITLE) == -1) {
-		xerror("archiver: %s: %s\n", mountpoint, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("archiver: '%s': %s\n", mountpoint, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(mountpoint, strlen(mountpoint));
 	add_to_jumpdb(workspaces[cur_ws].path);
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	if (conf.autols == 1) {
 		reload_dirlist();
 		add_to_dirhist(workspaces[cur_ws].path);
 	} else {
-		printf("%s: Successfully mounted on %s\n", file, mountpoint);
+		printf("'%s': Successfully mounted on '%s'\n", file, mountpoint);
 	}
 
 	return exit_status;
@@ -287,30 +305,30 @@ mount_iso(char *file)
 #if !defined(__linux__)
 	UNUSED(file);
 	xerror("%s\n", _("mount: This feature is for Linux only"));
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 #else
 	char *mountpoint = create_mountpoint(file);
 	if (!mountpoint)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Construct and execute the cmd */
 	char *sudo = get_sudo_path();
 	if (!sudo) {
 		free(mountpoint);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	char *cmd[] = {sudo, "mount", "-o", "loop", file, mountpoint, NULL};
 	if (confirm_sudo_cmd(cmd) == 0) {
 		free(mountpoint);
 		free(sudo);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS) {
 		free(mountpoint);
 		free(sudo);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 	free(sudo);
 
@@ -335,7 +353,7 @@ handle_iso(char *file)
 	char sel_op = get_operation(OP_ISO);
 
 	if (strchr(file, '\\')) {
-		char *deq_file = dequote_str(file, 0);
+		char *deq_file = unescape_str(file, 0);
 		if (deq_file) {
 			xstrsncpy(file, deq_file, strlen(deq_file) + 1);
 			free(deq_file);
@@ -348,7 +366,7 @@ handle_iso(char *file)
 	case 'l': return list_iso_contents(file);
 	case 'm': return mount_iso(file);
 	case 't': return test_iso(file);
-	default: return EXIT_SUCCESS;
+	default: return FUNC_SUCCESS;
 	}
 }
 
@@ -356,27 +374,27 @@ static int
 create_iso_from_block_dev(char *in_file, char *out_file)
 {
 	size_t len = strlen(in_file) + 4;
-	char *if_option = (char *)xnmalloc(len, sizeof(char));
+	char *if_option = xnmalloc(len, sizeof(char));
 	snprintf(if_option, len, "if=%s", in_file); /* NOLINT */
 
 	len = strlen(out_file) + 4;
-	char *of_option = (char *)xnmalloc(len, sizeof(char));
+	char *of_option = xnmalloc(len, sizeof(char));
 	snprintf(of_option, len, "of=%s", out_file); /* NOLINT */
 
 	char *sudo = get_sudo_path();
 	if (!sudo) {
 		free(if_option);
 		free(of_option);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	char *cmd[] = {sudo, "dd", if_option, of_option, "bs=64k",
 	    "conv=noerror,sync", "status=progress", NULL};
 
 	if (confirm_sudo_cmd(cmd) == 1) {
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	}
 
 	free(sudo);
@@ -391,16 +409,16 @@ create_iso(char *in_file, char *out_file)
 {
 	struct stat attr;
 	if (lstat(in_file, &attr) == -1) {
-		xerror("archiver: %s: %s\n", in_file, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("archiver: '%s': %s\n", in_file, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	/* If IN_FILE is a directory */
 	if (S_ISDIR(attr.st_mode)) {
 		char *cmd[] = {"mkisofs", "-R", "-o", out_file, in_file, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			return EXIT_FAILURE;
-		return EXIT_SUCCESS;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			return FUNC_FAILURE;
+		return FUNC_SUCCESS;
 	}
 
 	/* If IN_FILE is a block device */
@@ -408,9 +426,9 @@ create_iso(char *in_file, char *out_file)
 		return create_iso_from_block_dev(in_file, out_file);
 
 	/* If any other file format */
-	xerror(_("archiver: %s: Invalid file format. File should be either "
+	xerror(_("archiver: '%s': Invalid file format. File should be either "
 		"a directory or a block device\n"), in_file);
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Check the MIME type of the file named FILE and look for "ISO 9660" in its
@@ -436,14 +454,14 @@ check_iso(char *file)
 		is_iso = 1;
 
 	free(t);
-	return (is_iso == 1 ? EXIT_SUCCESS : EXIT_FAILURE);
+	return (is_iso == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
 
 #else
-	char *rand_ext = gen_rand_str(10);
+	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
 	if (!rand_ext)
 		return (-1);
 
-	char tmp_file[PATH_MAX];
+	char tmp_file[PATH_MAX + 1];
 	snprintf(tmp_file, sizeof(tmp_file), "%s/.temp%s",
 		(xargs.stealth_mode == 1) ? P_tmpdir : tmp_dir, rand_ext);
 	free(rand_ext);
@@ -451,13 +469,13 @@ check_iso(char *file)
 	int fd;
 	FILE *fp_out = open_fwrite(tmp_file, &fd);
 	if (!fp_out) {
-		xerror("archiver: %s: %s\n", tmp_file, strerror(errno));
+		xerror("archiver: '%s': %s\n", tmp_file, strerror(errno));
 		return (-1);
 	}
 
-	FILE *fp_err = fopen("/dev/null", "w");
+	FILE *fp_err = fopen(_PATH_DEVNULL, "w");
 	if (!fp_err) {
-		xerror("archiver: /dev/null: %s\n", strerror(errno));
+		xerror("archiver: '%s': %s\n", _PATH_DEVNULL, strerror(errno));
 		fclose(fp_out);
 		unlink(tmp_file);
 		return (-1);
@@ -488,18 +506,18 @@ check_iso(char *file)
 	close(stdout_bk);
 	close(stderr_bk);
 
-	if (retval != EXIT_SUCCESS) {
+	if (retval != FUNC_SUCCESS) {
 		unlink(tmp_file);
 		return (-1);
 	}
 
 	fp_out = open_fread(tmp_file, &fd);
 	if (fp_out) {
-		char line[NAME_MAX] = "";
+		char line[NAME_MAX]; *line = '\0';
 		if (fgets(line, (int)sizeof(line), fp_out) == NULL) {
 			fclose(fp_out);
 			unlink(tmp_file);
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 		}
 
 		if (strstr(line, "ISO 9660"))
@@ -509,7 +527,7 @@ check_iso(char *file)
 
 	unlink(tmp_file);
 
-	return (is_iso == 1 ? EXIT_SUCCESS : EXIT_FAILURE);
+	return (is_iso == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
 
 ERROR:
 	xerror("archiver: %s\n", strerror(errno));
@@ -529,9 +547,9 @@ check_compressed(const char *line, const int test_iso)
 	|| (test_iso && strstr(line, "ISO 9660")));
 }
 
-/* Run the 'file' command on FILE and look for "archive" and
- * "compressed" strings in its output. Returns zero if compressed,
- * one if not, and -1 in case of error.
+/* Check the MIME type of the file names FILE and look for "archive" and
+ * "compressed" strings. Returns zero if compressed, one if not, and -1
+ * in case of error.
  * test_iso is used to determine if ISO files should be checked as
  * well: this is the case when called from open_function() or
  * mime_open(), since both need to check compressed and ISOs as
@@ -552,13 +570,13 @@ is_compressed(char *file, const int test_iso)
 		return (-1);
 	}
 
-	int compressed = check_compressed(t, test_iso);
+	const int compressed = check_compressed(t, test_iso);
 	free(t);
 
-	return (compressed == 1 ? EXIT_SUCCESS : EXIT_FAILURE);
+	return (compressed == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
 
 #else
-	char *rand_ext = gen_rand_str(10);
+	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
 	if (!rand_ext)
 		return (-1);
 
@@ -570,13 +588,13 @@ is_compressed(char *file, const int test_iso)
 	int fd;
 	FILE *fp_out = open_fwrite(tmp_file, &fd);
 	if (!fp_out) {
-		xerror("archiver: %s: %s\n", tmp_file, strerror(errno));
+		xerror("archiver: '%s': %s\n", tmp_file, strerror(errno));
 		return (-1);
 	}
 
 	FILE *fp_err = fopen("/dev/null", "w");
 	if (!fp_err) {
-		xerror("archiver: /dev/null: %s\n", strerror(errno));
+		xerror("archiver: '/dev/null': %s\n", strerror(errno));
 		unlink(tmp_file);
 		fclose(fp_out);
 		return (-1);
@@ -607,7 +625,7 @@ is_compressed(char *file, const int test_iso)
 	close(stdout_bk);
 	close(stderr_bk);
 
-	if (retval != EXIT_SUCCESS) {
+	if (retval != FUNC_SUCCESS) {
 		unlink(tmp_file);
 		return (-1);
 	}
@@ -615,11 +633,11 @@ is_compressed(char *file, const int test_iso)
 	int compressed = 0;
 	fp_out = open_fread(tmp_file, &fd);
 	if (fp_out) {
-		char line[255];
+		char line[NAME_MAX + 1]; *line = '\0';
 		if (fgets(line, (int)sizeof(line), fp_out) == NULL) {
 			fclose(fp_out);
 			unlink(tmp_file);
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 		}
 
 		compressed = check_compressed(line, test_iso);
@@ -628,7 +646,7 @@ is_compressed(char *file, const int test_iso)
 
 	unlink(tmp_file);
 
-	return (compressed == 1 ? EXIT_SUCCESS : EXIT_FAILURE);
+	return (compressed == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
 
 ERROR:
 	xerror("archiver: %s\n", strerror(errno));
@@ -644,10 +662,10 @@ ERROR:
 static char *
 add_default_extension(char *name)
 {
-	size_t name_len = strlen(name);
+	const size_t name_len = strlen(name);
 
 	char *t = savestring(name, name_len);
-	name = (char *)xrealloc(name, (name_len + 8) * sizeof(char));
+	name = xnrealloc(name, name_len + 8, sizeof(char));
 	snprintf(name, name_len + 8, "%s.tar.gz", t); /* NOLINT */
 	free(t);
 
@@ -665,9 +683,8 @@ get_archive_filename(void)
 		flags |= NO_FIX_RL_POINT;
 		name = rl_no_hist(_("File name ('q' to quit): "));
 		flags &= ~NO_FIX_RL_POINT;
-		if (!name)
-			continue;
-		if (!*name) {
+
+		if (!name || !*name) {
 			free(name);
 			name = (char *)NULL;
 			continue;
@@ -697,27 +714,27 @@ get_archive_filename(void)
  * file named OUT_FILE. If MODE is 'd', extract, test or get
  * information about IN_FILE. OP is used only for the 'd' mode: it
  * tells if we have one or multiple file. Returns zero on success and
- * one on error */
+ * one on error. */
 static int
 zstandard(char *in_file, char *out_file, const char mode, const char op)
 {
-	int exit_status = EXIT_SUCCESS;
-	char *deq_file = dequote_str(in_file, 0);
+	int exit_status = FUNC_SUCCESS;
+	char *deq_file = unescape_str(in_file, 0);
 	if (!deq_file) {
-		xerror(_("archiver: %s: Error dequoting file name\n"), in_file);
-		return EXIT_FAILURE;
+		xerror(_("archiver: '%s': Error unescaping file name\n"), in_file);
+		return FUNC_FAILURE;
 	}
 
 	if (mode == 'c') {
 		if (out_file) {
 			char *cmd[] = {"zstd", "-zo", out_file, deq_file, NULL};
-			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-				exit_status = EXIT_FAILURE;
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+				exit_status = FUNC_FAILURE;
 		} else {
 			char *cmd[] = {"zstd", "-z", deq_file, NULL};
 
-			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-				exit_status = EXIT_FAILURE;
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+				exit_status = FUNC_FAILURE;
 		}
 
 		free(deq_file);
@@ -727,7 +744,7 @@ zstandard(char *in_file, char *out_file, const char mode, const char op)
 	/* mode == 'd' */
 
 	/* op is non-zero when multiple files, including at least one
-	 * zst file, are passed to the archiver function */
+	 * zst file, are passed to the archiver function. */
 	if (op != 0) {
 		char option[3] = "";
 
@@ -742,10 +759,10 @@ zstandard(char *in_file, char *out_file, const char mode, const char op)
 		exit_status = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 		free(deq_file);
 
-		if (exit_status != EXIT_SUCCESS)
-			return EXIT_FAILURE;
+		if (exit_status != FUNC_SUCCESS)
+			return FUNC_FAILURE;
 
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	printf(_("%s[e]%sxtract %s[t]%sest %s[i]%snfo %s[q]%suit\n"),
@@ -765,26 +782,26 @@ zstandard(char *in_file, char *out_file, const char mode, const char op)
 		switch (*operation) {
 		case 'e': {
 			char *cmd[] = {"zstd", "-d", deq_file, NULL};
-			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-				exit_status = EXIT_FAILURE;
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+				exit_status = FUNC_FAILURE;
 		} break;
 
 		case 't': {
 			char *cmd[] = {"zstd", "-t", deq_file, NULL};
-			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-				exit_status = EXIT_FAILURE;
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+				exit_status = FUNC_FAILURE;
 		} break;
 
 		case 'i': {
 			char *cmd[] = {"zstd", "-l", deq_file, NULL};
-			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-				exit_status = EXIT_FAILURE;
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+				exit_status = FUNC_FAILURE;
 		} break;
 
 		case 'q':
 			free(operation);
 			free(deq_file);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 
 		default:
 			free(operation);
@@ -804,18 +821,18 @@ compress_zstandard(char *name, char **args)
 	if (!args[2]) /* Only one file */
 		return zstandard(args[1], name, 'c', 0);
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	/* Multiple files */
 	printf(_("\n%sNOTE%s: Zstandard does not support compression of "
 		 "multiple files into one single compressed file. Files will "
 		 "be compressed rather into multiple compressed files using the "
-		 "original file names\n"), BOLD, df_c);
+		 "original file names.\n"), BOLD, df_c);
 
 	size_t i;
 	for (i = 1; args[i]; i++) {
-		if (zstandard(args[i], NULL, 'c', 0) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (zstandard(args[i], NULL, 'c', 0) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	}
 
 	return exit_status;
@@ -828,16 +845,16 @@ compress_others(char **args, char *name)
 	for (i = 1; args[i]; i++);
 
 	char *ext_ok = strrchr(name, '.');
-	char **tcmd = (char **)xnmalloc(3 + i + 1, sizeof(char *));
+	char **tcmd = xnmalloc(3 + i + 1, sizeof(char *));
 	tcmd[0] = savestring("atool", 5);
 	tcmd[1] = savestring("-a", 2);
-	size_t len = strlen(name) + (!ext_ok ? 7 : 0) + 1;
-	tcmd[2] = (char *)xnmalloc(len, sizeof(char *));
+	const size_t len = strlen(name) + (!ext_ok ? 7 : 0) + 1;
+	tcmd[2] = xnmalloc(len, sizeof(char *));
 	snprintf(tcmd[2], len, "%s%s", name, !ext_ok ? ".tar.gz" : ""); /* NOLINT */
 	n += 3;
 
 	for (i = 1; args[i]; i++) {
-		char *p = dequote_str(args[i], 0);
+		char *p = unescape_str(args[i], 0);
 		if (!p)
 			continue;
 		tcmd[n] = savestring(p, strlen(p));
@@ -846,7 +863,7 @@ compress_others(char **args, char *name)
 	}
 	tcmd[n] = (char *)NULL;
 
-	int ret = launch_execv(tcmd, FOREGROUND, E_NOFLAG);
+	const int ret = launch_execv(tcmd, FOREGROUND, E_NOFLAG);
 
 	for (i = 0; tcmd[i]; i++)
 		free(tcmd[i]);
@@ -858,29 +875,29 @@ compress_others(char **args, char *name)
 static int
 compress_files(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	char *name = get_archive_filename();
 	if (!name)
 		return exit_status;
 
-	/* # ZSTANDARD # */
 	char *ret = strrchr(name, '.');
-	if (strcmp(ret, ".zst") == 0) {
+
+	/* # ZSTANDARD # */
+	if (ret && strcmp(ret, ".zst") == 0) {
 		exit_status = compress_zstandard(name, args);
 		free(name);
 		return exit_status;
 	}
 
 	/* # ISO 9660 # */
-	if (strcmp(ret, ".iso") == 0) {
+	if (ret && strcmp(ret, ".iso") == 0) {
 		exit_status = create_iso(args[1], name);
 		free(name);
 		return exit_status;
 	}
 
 	/* # OTHER FORMATS # */
-	/* Construct the command */
 	exit_status = compress_others(args, name);
 	free(name);
 
@@ -895,13 +912,13 @@ check_not_compressed(char **args)
 	for (i = 1; args[i]; i++) {
 		char *deq = (char *)NULL;
 		if (strchr(args[i], '\\')) {
-			deq = dequote_str(args[i], 0);
+			deq = unescape_str(args[i], 0);
 			xstrsncpy(args[i], deq, strlen(deq) + 1);
 			free(deq);
 		}
 
 		if (is_compressed(args[i], 1) != 0) {
-			xerror(_("archiver: %s: Not an archive/compressed file\n"), args[i]);
+			xerror(_("archiver: '%s': Not an archive/compressed file\n"), args[i]);
 			return 1;
 		}
 	}
@@ -978,26 +995,26 @@ get_zstandard_operation(void)
 static int
 decompress_zstandard(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	size_t files_num = 0, i;
 	for (i = 1; args[i]; i++)
 		files_num++;
 
 	if (files_num == 1) {
-		if (zstandard(args[1], NULL, 'd', 0) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (zstandard(args[1], NULL, 'd', 0) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 		return exit_status;
 	}
 
 	/* Multiple files */
 	char sel_op = get_zstandard_operation();
 	if (sel_op == 0) /* quit */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	for (i = 1; args[i]; i++) {
-		if (zstandard(args[i], NULL, 'd', sel_op) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (zstandard(args[i], NULL, 'd', sel_op) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	}
 
 	return exit_status;
@@ -1006,15 +1023,15 @@ decompress_zstandard(char **args)
 static int
 list_others(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	size_t i;
 	for (i = 1; args[i]; i++) {
 		printf(_("%s%sFile%s: %s\n"), (i > 1) ? "\n" : "", BOLD, df_c, args[i]);
 
 		char *cmd[] = {"atool", "-l", args[i], NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	}
 
 	return exit_status;
@@ -1023,7 +1040,7 @@ list_others(char **args)
 static int
 extract_to_dir_others(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	size_t i;
 	for (i = 1; args[i]; i++) {
@@ -1036,11 +1053,10 @@ extract_to_dir_others(char **args)
 
 		/* Construct and execute cmd */
 		char *cmd[] = {"atool", "-X", ext_path, args[i], NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 
 		free(ext_path);
-		ext_path = (char *)NULL;
 	}
 
 	return exit_status;
@@ -1055,7 +1071,7 @@ extract_others(char **args)
 	for (i = 1; args[i]; i++);
 
 	/* Construct the cmd */
-	tcmd = (char **)xnmalloc(3 + i + 1, sizeof(char *));
+	tcmd = xnmalloc(3 + i + 1, sizeof(char *));
 	tcmd[0] = savestring("atool", 5);
 	tcmd[1] = savestring("-x", 2);
 	tcmd[2] = savestring("-e", 2);
@@ -1067,9 +1083,9 @@ extract_others(char **args)
 	tcmd[n] = (char *)NULL;
 
 	/* Launch it */
-	int exit_status = EXIT_SUCCESS;
-	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	int exit_status = FUNC_SUCCESS;
+	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	for (i = 0; tcmd[i]; i++)
 		free(tcmd[i]);
@@ -1090,7 +1106,7 @@ get_repack_format(void)
 			continue;
 
 		/* Do not allow any of these characters (mitigate command injection) */
-		char invalid_c[] = " \t\n\"\\/'`@$><=;|&{([*?!%";
+		const char invalid_c[] = " \t\n\"\\/'`@$><=;|&{([*?!%";
 		if (!*format || (*format != '.' && *format != 'q')
 		|| strpbrk(format, invalid_c)) {
 			free(format);
@@ -1113,13 +1129,13 @@ repack_others(char **args)
 	/* Ask for new archive/compression format */
 	char *format = get_repack_format();
 	if (!format) /* quit */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	/* Construct and execute the cmd */
 	size_t n = 0, i;
 	for (i = 1; args[i]; i++);
 
-	char **tcmd = (char **)xnmalloc(4 + i + 1, sizeof(char *));
+	char **tcmd = xnmalloc(4 + i + 1, sizeof(char *));
 	tcmd[0] = savestring("arepack", 7);
 	tcmd[1] = savestring("-F", 2);
 	tcmd[2] = savestring(format, strlen(format));
@@ -1131,9 +1147,9 @@ repack_others(char **args)
 	}
 	tcmd[n] = (char *)NULL;
 
-	int exit_status = EXIT_SUCCESS;
-	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	int exit_status = FUNC_SUCCESS;
+	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
 	for (i = 0; tcmd[i]; i++)
 		free(tcmd[i]);
@@ -1147,19 +1163,17 @@ static int
 list_mounted_files(char *mountpoint)
 {
 	if (xchdir(mountpoint, SET_TITLE) == -1) {
-		xerror("archiver: %s: %s\n", mountpoint, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("archiver: '%s': %s\n", mountpoint, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(mountpoint, strlen(mountpoint)); /* NOLINT */
 	add_to_jumpdb(workspaces[cur_ws].path);
 
-	int exit_status = EXIT_SUCCESS;
-	if (conf.autols) {
-		free_dirlist();
-		if (list_dir() != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+	int exit_status = FUNC_SUCCESS;
+	if (conf.autols == 1) {
+		reload_dirlist();
 		add_to_dirhist(workspaces[cur_ws].path);
 	}
 
@@ -1169,7 +1183,7 @@ list_mounted_files(char *mountpoint)
 static int
 mount_others(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	size_t files_num = 0, i;
 	for (i = 1; args[i]; i++)
 		files_num++;
@@ -1181,7 +1195,7 @@ mount_others(char **args)
 
 		/* Construct and execute cmd */
 		char *cmd[] = {"archivemount", args[i], mountpoint, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS) {
 			free(mountpoint);
 			continue;
 		}
@@ -1194,8 +1208,8 @@ mount_others(char **args)
 			continue;
 		}
 
-		if (list_mounted_files(mountpoint) == EXIT_FAILURE)
-			exit_status = EXIT_FAILURE;
+		if (list_mounted_files(mountpoint) == FUNC_FAILURE)
+			exit_status = FUNC_FAILURE;
 		free(mountpoint);
 	}
 
@@ -1220,22 +1234,22 @@ decompress_others(char **args)
 	default: break;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 decompress_files(char **args)
 {
 	if (check_not_compressed(args) == 1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* # ISO 9660 # */
-	char *ret = strrchr(args[1], '.');
+	const char *ret = strrchr(args[1], '.');
 	if ((ret && strcmp(ret, ".iso") == 0) || check_iso(args[1]) == 0)
 		return handle_iso(args[1]);
 
 	/* Check if we have at least one Zstandard file */
-	size_t zst = check_zstandard(args);
+	const size_t zst = check_zstandard(args);
 
 	/* # ZSTANDARD # */
 	if (zst == 1)
@@ -1255,7 +1269,7 @@ int
 archiver(char **args, const char mode)
 {
 	if (!args[1])
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (mode == 'c')
 		return compress_files(args);
