@@ -2,7 +2,7 @@
  *
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -26,8 +26,8 @@
 
 __BEGIN_DECLS
 
-int is_compressed(char *, const int);
-int archiver(char **, char);
+int is_compressed(char *file, const int test_iso);
+int archiver(char **args, char mode);
 
 __END_DECLS
 
