@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -28,7 +28,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
-#include <limits.h> /* INT_MAX */
 #include <pwd.h> /* getpwuid() */
 #include <errno.h>
 #include <readline/tilde.h> /* tilde_expand() */
@@ -36,7 +35,7 @@
 
 #include "aux.h"
 #include "checks.h"
-#include "exec.h"
+#include "config.h" /* set_time_style */
 #include "file_operations.h"
 #include "init.h"
 #include "mime.h"
@@ -44,10 +43,11 @@
 #include "navigation.h"
 #include "profiles.h"
 #include "sanitize.h"
+#include "spawn.h"
 #include "strings.h"
 
 #if defined(_NO_PROFILES) || defined(_NO_FZF) || defined(_NO_ICONS) \
-|| defined(_NO_TRASH) || defined(_BE_POSIX)
+|| defined(_NO_TRASH) || defined(_BE_POSIX) || defined(_NO_LIRA)
 # include "messages.h"
 #endif /* _NO_PROFILES || _NO_FZF || _NO_ICONS || _NO_TRASH */
 
@@ -63,12 +63,12 @@
 #endif /* CLIFM_DATADIR */
 
 #ifdef _BE_POSIX
-# define OPTSTRING ":aAb:B:c:CdDeEfFgGhHij:J:k:lLmMnNo:O:p:P:qQrRsSt:TuvV:w:WxXyYz:Z"
+# define OPTSTRING ":aAb:B:c:CdDeEfFgGhHiI:j:J:k:lLmMnNo:O:p:P:qQrRsSt:TuvV:w:WxXyYz:Z"
 #else
 # ifdef RUN_CMD
-#  define OPTSTRING "+:aAb:c:C:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
+#  define OPTSTRING "+:aAb:c:C:D:eEfFgGhHiIk:lLmoOp:P:rsStT:vw:xyz:"
 # else
-#  define OPTSTRING "+:aAb:c:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
+#  define OPTSTRING "+:aAb:c:D:eEfFgGhHiIk:lLmoOp:P:rsStT:vw:xyz:"
 # endif /* RUN_CMD */
 #endif /* _BE_POSIX */
 
@@ -149,11 +149,17 @@
 #define LOPT_STAT                   271
 #define LOPT_STAT_FULL              272
 #define LOPT_READONLY               273
+#define LOPT_LSCOLORS               274
+#define LOPT_PROP_FIELDS            275
+#define LOPT_TIME_STYLE             276
+#define LOPT_PTIME_STYLE            277
+#define LOPT_COLOR_LNK_AS_TARGET    278
+#define LOPT_PAGER_VIEW             279
 
 /* Link long (--option) and short options (-o) for the getopt_long function. */
-static const struct option longopts[] = {
-	{"no-hidden", no_argument, 0, 'a'},
-	{"show-hidden", no_argument, 0, 'A'},
+static struct option const longopts[] = {
+	{"show-hidden", no_argument, 0, 'a'},
+	{"no-hidden", no_argument, 0, 'A'},
 	{"bookmarks-file", required_argument, 0, 'b'},
 	{"config-file", required_argument, 0, 'c'},
 
@@ -164,8 +170,8 @@ static const struct option longopts[] = {
 	{"config-dir", required_argument, 0, 'D'},
 	{"no-eln", no_argument, 0, 'e'},
 	{"eln-use-workspace-color", no_argument, 0, 'E'},
-	{"no-dirs-first", no_argument, 0, 'f'},
-	{"dirs-first", no_argument, 0, 'F'},
+	{"dirs-first", no_argument, 0, 'f'},
+	{"no-dirs-first", no_argument, 0, 'F'},
 	{"pager", no_argument, 0, 'g'},
 	{"no-pager", no_argument, 0, 'G'},
 	{"help", no_argument, 0, 'h'},
@@ -173,17 +179,18 @@ static const struct option longopts[] = {
 	{"no-case-sensitive", no_argument, 0, 'i'},
 	{"case-sensitive", no_argument, 0, 'I'},
 	{"keybindings-file", required_argument, 0, 'k'},
-	{"no-long-view", no_argument, 0, 'l'},
-	{"long-view", no_argument, 0, 'L'},
+	{"long-view", no_argument, 0, 'l'},
+	{"follow-symlinks-long", no_argument, 0, 'L'},
 	{"dirhist-map", no_argument, 0, 'm'},
-	{"no-autols", no_argument, 0, 'o'},
-	{"autols", no_argument, 0, 'O'},
+	{"autols", no_argument, 0, 'o'},
+	{"no-autols", no_argument, 0, 'O'},
 	{"path", required_argument, 0, 'p'},
 	{"profile", required_argument, 0, 'P'},
 	{"no-refresh-on-emtpy-line", no_argument, 0, 'r'},
 	{"splash", no_argument, 0, 's'},
 	{"stealth-mode", no_argument, 0, 'S'},
 	{"disk-usage-analyzer", no_argument, 0, 't'},
+	{"trash-dir", required_argument, 0, 'T'},
 	{"version", no_argument, 0, 'v'},
 	{"workspace", required_argument, 0, 'w'},
 	{"no-ext-cmds", no_argument, 0, 'x'},
@@ -191,17 +198,16 @@ static const struct option longopts[] = {
 	{"sort", required_argument, 0, 'z'},
 
 	/* Only-long options */
-//		{"autojump", no_argument, 0, LOPT_NO_AUTOJUMP},
 	{"bell", required_argument, 0, LOPT_BELL},
 	{"case-sens-dirjump", no_argument, 0, LOPT_CASE_SENS_DIRJUMP},
 	{"case-sens-path-comp", no_argument, 0, LOPT_CASE_SENS_PATH_COMP},
 	{"cd-on-quit", no_argument, 0, LOPT_CD_ON_QUIT},
 	{"color-scheme", required_argument, 0, LOPT_COLOR_SCHEME},
+	{"color-links-as-target", no_argument, 0, LOPT_COLOR_LNK_AS_TARGET},
 	{"cwd-in-title", no_argument, 0, LOPT_CWD_IN_TITLE},
 	{"data-dir", required_argument, 0, LOPT_DATA_DIR},
 	{"desktop-notifications", no_argument, 0, LOPT_DESKTOP_NOTIFICATIONS},
 	{"disk-usage", no_argument, 0, LOPT_DISK_USAGE},
-//		{"enable-logs", no_argument, 0, LOPT_ENABLE_LOGS},
 	{"fnftab", no_argument, 0, LOPT_FNFTAB},
 	{"full-dir-size", no_argument, 0, LOPT_FULL_DIR_SIZE},
 	{"fuzzy-matching", no_argument, 0, LOPT_FUZZY_MATCHING},
@@ -214,6 +220,8 @@ static const struct option longopts[] = {
 	{"icons-use-file-color", no_argument, 0, LOPT_ICONS_USE_FILE_COLOR},
 	{"int-vars", no_argument, 0, LOPT_INT_VARS},
 	{"list-and-quit", no_argument, 0, LOPT_LIST_AND_QUIT},
+	{"ls", no_argument, 0, LOPT_LIST_AND_QUIT},
+	{"lscolors", no_argument, 0, LOPT_LSCOLORS},
 	{"max-dirhist", required_argument, 0, LOPT_MAX_DIRHIST},
 	{"max-files", required_argument, 0, LOPT_MAX_FILES},
 	{"max-path", required_argument, 0, LOPT_MAX_PATH},
@@ -244,8 +252,11 @@ static const struct option longopts[] = {
 	{"only-dirs", no_argument, 0, LOPT_ONLY_DIRS},
 	{"open", required_argument, 0, LOPT_OPEN},
 	{"opener", required_argument, 0, LOPT_OPENER},
+	{"pager-view", required_argument, 0, LOPT_PAGER_VIEW},
+	{"ptime-style", required_argument, 0, LOPT_PTIME_STYLE},
 	{"preview", required_argument, 0, LOPT_PREVIEW},
 	{"print-sel", no_argument, 0, LOPT_PRINT_SEL},
+	{"prop-fields", required_argument, 0, LOPT_PROP_FIELDS},
 	{"readonly", no_argument, 0, LOPT_READONLY},
 	{"rl-vi-mode", no_argument, 0, LOPT_RL_VI_MODE},
 	{"share-selbox", no_argument, 0, LOPT_SHARE_SELBOX},
@@ -261,6 +272,7 @@ static const struct option longopts[] = {
 	{"stat", required_argument, 0, LOPT_STAT},
 	{"stat-full", required_argument, 0, LOPT_STAT_FULL},
 	{"stdtab", no_argument, 0, LOPT_STDTAB},
+	{"time-style", required_argument, 0, LOPT_TIME_STYLE},
 	{"virtual-dir", required_argument, 0, LOPT_VIRTUAL_DIR},
 	{"virtual-dir-full-paths", no_argument, 0, LOPT_VIRTUAL_DIR_FULL_PATHS},
 	{"vt100", no_argument, 0, LOPT_VT100},
@@ -306,7 +318,7 @@ set_cur_workspace(void)
 	if (workspaces[cur_ws].path)
 		return;
 
-	char p[PATH_MAX] = "";
+	char p[PATH_MAX + 1] = "";
 	char *cwd = get_cwd(p, sizeof(p), 0);
 
 	if (cwd && *cwd) {
@@ -324,7 +336,7 @@ set_cur_workspace(void)
 		return;
 	}
 
-	xerror("%s: /: %s\n", PROGRAM_NAME, strerror(errno));
+	xerror("%s: '/': %s\n", PROGRAM_NAME, strerror(errno));
 	exit(EXIT_FAILURE);
 }
 
@@ -340,7 +352,7 @@ set_start_path(void)
 
 	if (cur_ws > MAX_WS - 1) {
 		cur_ws = DEF_CUR_WS;
-		err('w', PRINT_PROMPT, _("%s: %zu: Invalid workspace."
+		err('w', PRINT_PROMPT, _("%s: '%zu': Invalid workspace."
 			"\nFalling back to workspace %zu\n"),
 			PROGRAM_NAME, cur_ws, cur_ws + 1);
 	}
@@ -349,9 +361,9 @@ set_start_path(void)
 	set_cur_workspace();
 
 	/* Make path the CWD. */
-	int ret = xchdir(workspaces[cur_ws].path, NO_TITLE);
+	const int ret = xchdir(workspaces[cur_ws].path, NO_TITLE);
 
-	char tmp[PATH_MAX] = "";
+	char tmp[PATH_MAX + 1] = "";
 	char *pwd = get_cwd(tmp, sizeof(tmp), 0);
 
 	/* If chdir() fails, set path to PWD, list files and print the
@@ -376,13 +388,13 @@ set_start_path(void)
 		setenv("OLDPWD", pwd, 1);
 
 	dir_changed = 1;
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 try_standard_data_dirs(void)
 {
-	char home_local[PATH_MAX];
+	char home_local[PATH_MAX + 1];
 	*home_local = '\0';
 	if (user.home && *user.home)
 		snprintf(home_local, sizeof(home_local), "%s/.local/share", user.home);
@@ -412,10 +424,10 @@ try_standard_data_dirs(void)
 			continue;
 
 		data_dir = savestring(data_dirs[i], strlen(data_dirs[i]));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 static char *
@@ -431,13 +443,13 @@ try_datadir(const char *dir)
 
 	struct stat a;
 
-	char p[PATH_MAX];
+	char p[PATH_MAX + 1];
 
 	/* Try DIR/share/clifm/clifmrc */
 	snprintf(p, sizeof(p), "%s/share/%s/%src", dir, PROGRAM_NAME, PROGRAM_NAME);
 	if (stat(p, &a) != -1 && S_ISREG(a.st_mode)) {
-		size_t len = strlen(dir) + 7;
-		char *q = (char *)xnmalloc(len, sizeof(char));
+		const size_t len = strlen(dir) + 7;
+		char *q = xnmalloc(len, sizeof(char));
 		snprintf(q, len, "%s/share", dir);
 		return q;
 	}
@@ -456,7 +468,9 @@ resolve_absolute_path(const char *s)
 	if (!s || !*s)
 		return (char *)NULL;
 
-	char *p = strrchr(s, '/'), *t = (char *)NULL;
+	char *p = strrchr(s, '/');
+	char *t = (char *)NULL;
+
 	if (p && p != s)
 		*p = '\0';
 	else
@@ -511,7 +525,7 @@ static int
 get_data_dir_from_path(char *arg)
 {
 	if (!arg || !*arg)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	char *datadir = (char *)NULL;
 	char *name = *arg == '~' ? tilde_expand(arg) : arg;
@@ -529,10 +543,10 @@ END:
 		free(name);
 
 	if (!datadir)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	data_dir = datadir;
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Get the system data directory (usually /usr/local/share),
@@ -546,7 +560,7 @@ get_data_dir(void)
 
 #ifdef CLIFM_DATADIR
 	struct stat a;
-	char p[PATH_MAX];
+	char p[PATH_MAX + 1];
 	snprintf(p, sizeof(p), "%s/%s/%src", STRINGIZE(CLIFM_DATADIR),
 		PROGRAM_NAME, PROGRAM_NAME);
 
@@ -557,10 +571,10 @@ get_data_dir(void)
 	}
 #endif /* CLIFM_DATADIR */
 
-	if (try_standard_data_dirs() == EXIT_SUCCESS)
+	if (try_standard_data_dirs() == FUNC_SUCCESS)
 		return;
 
-	if (get_data_dir_from_path(argv_bk[0]) == EXIT_SUCCESS)
+	if (get_data_dir_from_path(argv_bk[0]) == FUNC_SUCCESS)
 		return;
 
 	err('w', PRINT_PROMPT, _("%s: No data directory found. Data files, "
@@ -592,7 +606,7 @@ open_reg_exit(char *filename, const int url, const int preview)
 	char *homedir = (xargs.secure_env == 1 || xargs.secure_env_full == 1)
 		? get_home_sec_env() : getenv("HOME");
 	if (!homedir) {
-		xerror(_("%s: Could not retrieve the home directory\n"), PROGRAM_NAME);
+		xerror(_("%s: Cannot retrieve the home directory\n"), PROGRAM_NAME);
 		exit(EXIT_FAILURE);
 	}
 
@@ -601,10 +615,10 @@ open_reg_exit(char *filename, const int url, const int preview)
 	if (alt_preview_file && preview == 1) {
 		mime_file = savestring(alt_preview_file, strlen(alt_preview_file));
 	} else {
-		size_t mime_file_len = strlen(homedir)
+		const size_t mime_file_len = strlen(homedir)
 			+ (alt_profile ? strlen(alt_profile) : 7) + 40;
 
-		mime_file = (char *)xnmalloc(mime_file_len, sizeof(char));
+		mime_file = xnmalloc(mime_file_len, sizeof(char));
 		snprintf(mime_file, mime_file_len,
 			"%s/.config/clifm/profiles/%s/%s.clifm",
 			homedir, alt_profile ? alt_profile : "default",
@@ -612,10 +626,10 @@ open_reg_exit(char *filename, const int url, const int preview)
 	}
 
 	if (path_n == 0)
-		path_n = get_path_env();
+		path_n = get_path_env(0);
 
 #ifndef _NO_LIRA
-	if (url == 1 && mime_open_url(filename) == EXIT_SUCCESS)
+	if (url == 1 && mime_open_url(filename) == FUNC_SUCCESS)
 		exit(EXIT_SUCCESS);
 #else
 	UNUSED(url);
@@ -625,7 +639,7 @@ open_reg_exit(char *filename, const int url, const int preview)
 	if (*filename == '~')
 		p = tilde_expand(filename);
 
-	int ret = open_file(p ? p : filename);
+	const int ret = open_file(p ? p : filename);
 	free(p);
 	exit(ret);
 }
@@ -640,7 +654,11 @@ set_sort_by_name(const char *name)
 			return sort_methods[i].num;
 	}
 
-	return SNAME;
+	fprintf(stderr, _("%s: --sort: '%s': Invalid value\n"
+		"Valid values: atime, btime, ctime, mtime, extension, group, "
+		"inode, name,\n              none, owner, size, version, "
+		"blocks, links.\n"), PROGRAM_NAME, name);
+	exit(EXIT_FAILURE);
 }
 
 static void
@@ -654,7 +672,7 @@ set_sort(const char *arg)
 		n = atoi(arg);
 
 	if (n < 0 || n > SORT_TYPES) {
-		fprintf(stderr, _("%s: -z: %s: Valid values are 0-%d\n"),
+		fprintf(stderr, _("%s: --sort: '%s': Valid values are 0-%d\n"),
 			PROGRAM_NAME, arg, SORT_TYPES);
 		exit(EXIT_FAILURE);
 	}
@@ -675,23 +693,24 @@ open_preview_file(char *file, const int mode)
 	}
 
 	char *fpath = file;
-	int url = 1, preview = mode == PREVIEW_FILE ? 1 : 0;
+	int url = 1;
+	const int preview = mode == PREVIEW_FILE ? 1 : 0;
 
 	struct stat attr;
 	if (IS_FILE_URI(fpath)) {
 		fpath = file + 7;
 		if (stat(fpath, &attr) == -1) {
-			xerror("%s: %s: %s\n", PROGRAM_NAME, file, strerror(errno));
+			xerror("%s: '%s': %s\n", PROGRAM_NAME, file, strerror(errno));
 			exit(errno);
 		}
 		url = 0;
 		goto RUN;
 	}
 
-	if (is_url(fpath) == EXIT_FAILURE) {
+	if (is_url(fpath) == FUNC_FAILURE) {
 		url = 0;
 		if (*fpath != '~' && stat(fpath, &attr) == -1) {
-			xerror("%s: %s: %s\n", PROGRAM_NAME, fpath, strerror(errno));
+			xerror("%s: '%s': %s\n", PROGRAM_NAME, fpath, strerror(errno));
 			exit(errno);
 		}
 	}
@@ -709,6 +728,9 @@ RUN:
 static void
 set_alt_bm_file(char *file)
 {
+	if (!file || !*file || *file == '-')
+		err_arg_required("-b"); /* noreturn */
+
 	char *p = (char *)NULL;
 
 	if (*file == '~') {
@@ -716,18 +738,7 @@ set_alt_bm_file(char *file)
 		file = p;
 	}
 
-	if (access(file, R_OK) == -1) {
-		err('e', PRINT_PROMPT, _("%s: %s: %s\n"
-			"Falling back to the default bookmarks file\n"),
-		    PROGRAM_NAME, file, strerror(errno));
-		free(p);
-		return;
-	}
-
 	alt_bm_file = savestring(file, strlen(file));
-	err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
-		"bookmarks file\n"), PROGRAM_NAME);
-
 	free(p);
 }
 
@@ -780,7 +791,7 @@ set_fuzzy_algo(const char *opt)
 	int a = opt ? atoi(opt) : -1;
 
 	if (a < 1 || a > FUZZY_ALGO_MAX) {
-		fprintf(stderr, _("%s: %s: Invalid fuzzy algorithm. Valid "
+		fprintf(stderr, _("%s: '%s': Invalid fuzzy algorithm. Valid "
 			"values are either 1 or 2.\n"), PROGRAM_NAME, opt ? opt : "NULL");
 		exit(EXIT_FAILURE);
 	}
@@ -794,7 +805,7 @@ set_bell_style(const char *opt)
 	int a = atoi(opt);
 
 	if (!is_number(opt) || a < 0 || a > 3) {
-		fprintf(stderr, _("%s: %s: Invalid bell style. Valid values "
+		fprintf(stderr, _("%s: '%s': Invalid bell style. Valid values "
 			"are 0:none, 1:audible, 2:visible (requires readline >= 8.1), "
 			"3:flash. Defaults to 'visible', and, if not possible, 'none'.\n"),
 			PROGRAM_NAME, opt);
@@ -807,6 +818,9 @@ set_bell_style(const char *opt)
 static void
 set_alt_config_dir(char *dir)
 {
+	if (!dir || !*dir || *dir == '-')
+		err_arg_required("--config-dir"); /* noreturn */
+
 	char *dir_exp = (char *)NULL;
 
 	if (*dir == '~') {
@@ -818,11 +832,11 @@ set_alt_config_dir(char *dir)
 	struct stat attr;
 
 	if (stat(dir, &attr) == -1) {
-		char *tmp_cmd[] = {"mkdir", "-p", dir, NULL};
-		int ret = launch_execv(tmp_cmd, FOREGROUND, E_NOSTDERR);
-		if (ret != EXIT_SUCCESS) {
-			err('e', PRINT_PROMPT, _("%s: %s: Cannot create directory "
-				"(error %d)\nFalling back to default configuration "
+		char *tmp_cmd[] = {"mkdir", "-p", "--", dir, NULL};
+		const int ret = launch_execv(tmp_cmd, FOREGROUND, E_NOSTDERR);
+		if (ret != FUNC_SUCCESS) {
+			err('e', PRINT_PROMPT, _("%s: Cannot create directory '%s' "
+				"(error %d)\nFalling back to the default configuration "
 				"directory\n"), PROGRAM_NAME, dir, ret);
 			dir_ok = 0;
 		}
@@ -830,13 +844,13 @@ set_alt_config_dir(char *dir)
 
 	if (access(dir, W_OK) == -1) {
 		if (dir_ok == 1) {
-			err('e', PRINT_PROMPT, _("%s: %s: %s\n"
-				"Falling back to default configuration directory\n"),
+			err('e', PRINT_PROMPT, _("%s: '%s': %s\n"
+				"Falling back to the default configuration directory\n"),
 				PROGRAM_NAME, dir, strerror(errno));
 		}
 	} else {
 		alt_config_dir = savestring(dir, strlen(dir));
-		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: %s: Using alternative "
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: '%s': Using alternative "
 			"configuration directory\n"), PROGRAM_NAME, alt_config_dir);
 	}
 
@@ -854,7 +868,7 @@ set_custom_selfile(char *file)
 		return;
 	}
 
-	err('e', PRINT_PROMPT, _("%s: %s: Error setting custom "
+	err('e', PRINT_PROMPT, _("%s: '%s': Error setting custom "
 		"selections file\n"), PROGRAM_NAME, file);
 }
 
@@ -868,7 +882,7 @@ stat_file(char *file)
 	char *p = (char *)NULL;
 	if (*file == '~') {
 		if (!(p = tilde_expand(file))) {
-			xerror(_("%s: %s: Error expanding tilde\n"), PROGRAM_NAME, file);
+			xerror(_("%s: '%s': Error expanding tilde\n"), PROGRAM_NAME, file);
 			exit(EXIT_FAILURE);
 		}
 	} else {
@@ -877,7 +891,7 @@ stat_file(char *file)
 
 	struct stat a;
 	if (stat(p, &a) == -1) {
-		xerror("%s: %s: %s\n", PROGRAM_NAME, p, strerror(errno));
+		xerror("%s: '%s': %s\n", PROGRAM_NAME, p, strerror(errno));
 		if (p != file)
 			free(p);
 		exit(errno);
@@ -897,31 +911,50 @@ set_shotgun_file(char *opt)
 #endif /* !_NO_LIRA */
 #endif /* !_BE_POSIX */
 
+static void
+set_alt_trash_dir(char *file)
+{
+	if (!file || !*file || *file == '-')
+#ifndef _BE_POSIX
+		err_arg_required("-T"); /* noreturn */
+#else
+		err_arg_required("-i"); /* noreturn */
+#endif
+
+	char *trash_exp = (char *)NULL;
+
+	if (*file == '~') {
+		trash_exp = tilde_expand(file);
+		file = trash_exp;
+	}
+
+	alt_trash_dir = savestring(file, strlen(file));
+	free(trash_exp);
+}
+
 static void
 set_alt_kbinds_file(char *file)
 {
+	if (!file || !*file || *file == '-')
+		err_arg_required("-k"); /* noreturn */
+
 	char *kbinds_exp = (char *)NULL;
+
 	if (*file == '~') {
 		kbinds_exp = tilde_expand(file);
 		file = kbinds_exp;
 	}
 
-	if (access(file, R_OK) == -1) {
-		err('e', PRINT_PROMPT, _("%s: %s: %s\n"
-			"Falling back to the default keybindings file\n"),
-		    PROGRAM_NAME, file, strerror(errno));
-	} else {
-		alt_kbinds_file = savestring(file, strlen(file));
-		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
-			"keybindings file\n"), PROGRAM_NAME);
-	}
-
+	alt_kbinds_file = savestring(file, strlen(file));
 	free(kbinds_exp);
 }
 
 static void
 set_alt_config_file(char *file)
 {
+	if (!file || !*file || *file == '-')
+		err_arg_required("-c"); /* noreturn */
+
 	char *config_exp = (char *)NULL;
 
 	if (*file == '~') {
@@ -929,16 +962,7 @@ set_alt_config_file(char *file)
 		file = config_exp;
 	}
 
-	if (access(file, R_OK) == -1) {
-		err('e', PRINT_PROMPT, _("%s: %s: %s\nFalling back to default\n"),
-			PROGRAM_NAME, file, strerror(errno));
-		xargs.config = -1;
-	} else {
-		alt_config_file = savestring(file, strlen(file));
-		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
-			"configuration file\n"), PROGRAM_NAME);
-	}
-
+	alt_config_file = savestring(file, strlen(file));
 	free(config_exp);
 }
 
@@ -950,14 +974,14 @@ resolve_path(char *file)
 	if (*file == '.') {
 		_path = normalize_path(file, strlen(file));
 		if (!_path) {
-			fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, file, strerror(errno));
+			fprintf(stderr, "%s: '%s': %s\n", PROGRAM_NAME, file, strerror(errno));
 			exit(errno);
 		}
 
 	} else if (*file == '~') {
 		_path = tilde_expand(file);
 		if (!_path) {
-			fprintf(stderr, _("%s: %s: Error expanding tilde\n"),
+			fprintf(stderr, _("%s: '%s': Error expanding tilde\n"),
 				PROGRAM_NAME, file);
 			exit(EXIT_FAILURE);
 		}
@@ -966,16 +990,16 @@ resolve_path(char *file)
 		_path = savestring(file, strlen(file));
 
 	} else {
-		char tmp[PATH_MAX] = "";
+		char tmp[PATH_MAX + 1] = "";
 		char *cwd = get_cwd(tmp, sizeof(tmp), 0);
 
 		if (!cwd || !*cwd) {
-			fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, file, strerror(errno));
+			fprintf(stderr, "%s: '%s': %s\n", PROGRAM_NAME, file, strerror(errno));
 			exit(errno);
 		}
 
-		size_t len = strlen(cwd) + strlen(file) + 2;
-		_path = (char *)xnmalloc(len, sizeof(char));
+		const size_t len = strlen(cwd) + strlen(file) + 2;
+		_path = xnmalloc(len, sizeof(char));
 		snprintf(_path, len, "%s/%s", cwd, file);
 	}
 
@@ -989,7 +1013,7 @@ resolve_starting_path(char *file)
 
 	if (IS_FILE_URI(file)) {
 		_path = savestring(file + 7, strlen(file + 7));
-	} else if (is_url(file) == EXIT_SUCCESS) {
+	} else if (is_url(file) == FUNC_SUCCESS) {
 		open_reg_exit(file, 1, 0); /* noreturn */
 	} else {
 		_path = resolve_path(file);
@@ -997,13 +1021,13 @@ resolve_starting_path(char *file)
 
 	struct stat a;
 	if (stat(_path, &a) == -1) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, file, strerror(errno));
+		fprintf(stderr, "%s: '%s': %s\n", PROGRAM_NAME, file, strerror(errno));
 		free(_path);
 		exit(errno);
 	}
 
 	if (!S_ISDIR(a.st_mode)) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, file, strerror(ENOTDIR));
+		fprintf(stderr, "%s: '%s': %s\n", PROGRAM_NAME, file, strerror(ENOTDIR));
 		free(_path);
 		exit(ENOTDIR);
 	}
@@ -1024,11 +1048,11 @@ set_starting_path(char *_path)
 
 	} else { /* Error changing directory */
 		if (xargs.list_and_quit == 1) {
-			xerror("%s: %s: %s\n", PROGRAM_NAME, _path, strerror(errno));
+			xerror("%s: '%s': %s\n", PROGRAM_NAME, _path, strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 
-		err('w', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
+		err('w', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME,
 			_path, strerror(errno));
 	}
 }
@@ -1060,12 +1084,12 @@ set_alt_profile(const char *name)
 #ifndef _NO_PROFILES
 	free(alt_profile);
 
-	if (validate_profile_name(name) == EXIT_SUCCESS) {
+	if (validate_profile_name(name) == FUNC_SUCCESS) {
 		alt_profile = savestring(name, strlen(name));
 		return;
 	}
 
-	fprintf(stderr, _("%s: %s: Invalid profile name\n"),
+	fprintf(stderr, _("%s: '%s': Invalid profile name\n"),
 		PROGRAM_NAME, name);
 	exit(EXIT_FAILURE);
 #else
@@ -1095,7 +1119,7 @@ set_max_value(const char *opt, int *xval, int *intval)
 		return;
 
 	int opt_int = atoi(opt);
-	if (opt_int >= 0 && opt_int <= INT_MAX)
+	if (opt_int >= 0)
 		*xval = *intval = opt_int;
 }
 
@@ -1112,7 +1136,7 @@ set_workspace(const char *opt)
 	}
 
 ERROR:
-	fprintf(stderr, _("%s: %s: Invalid workspace. Valid "
+	fprintf(stderr, _("%s: '%s': Invalid workspace. Valid "
 		"values are 1-8.\n"), PROGRAM_NAME, opt);
 	exit(EXIT_FAILURE);
 }
@@ -1214,9 +1238,45 @@ set_stat(const int optc, const char *optval)
 #endif /* !_BE_POSIX */
 
 	xargs.restore_last_path = conf.restore_last_path = 0;
-	stat_filename = savestring(optval, strlen(optval));
 }
 
+#ifndef _BE_POSIX
+static void
+xset_time_style(char *optval, const int ptime)
+{
+	if (!optval || !*optval || *optval == '-')
+		err_arg_required(ptime == 0 ? "--time-style" : "--ptime-style");
+
+	if (ptime == 1) {
+		xargs.ptime_style = 1;
+		set_time_style(optval, &conf.ptime_str, 1);
+	} else {
+		xargs.time_style = 1;
+		set_time_style(optval, &conf.time_str, 0);
+	}
+}
+
+static void
+xset_prop_fields(const char *optval)
+{
+	/* --prop-fields accepts a single dash (-) as argument. */
+	if (!optval || !*optval || (*optval == '-' && optval[1]))
+		err_arg_required("--prop-fields");
+
+	xargs.prop_fields_str = 1;
+	xstrsncpy(prop_fields_str, optval, sizeof(prop_fields_str));
+	set_prop_fields(prop_fields_str);
+
+#ifndef ST_BTIME
+	if (prop_fields.time == PROP_TIME_BIRTH) {
+		fprintf(stderr, _("%s: --prop-fields: 'b': Birth time is not "
+			"available on this platform\n"), PROGRAM_NAME);
+		exit(EXIT_FAILURE);
+	}
+#endif /* !ST_BTIME*/
+}
+#endif /* !_BE_POSIX */
+
 __attribute__ ((noreturn))
 static void
 print_version(void)
@@ -1237,7 +1297,7 @@ set_tab_mode(const char *opt)
 	if (!opt || !*opt || opt[1])
 		return;
 
-	int n = *opt - '0';
+	const int n = *opt - '0';
 
 	switch (n) {
 	case 0:	set_stdtab(); break;
@@ -1245,12 +1305,32 @@ set_tab_mode(const char *opt)
 	case 2: set_smenutab(); break;
 	case 3: set_fnftab(); break;
 	default:
-		fprintf(stderr, _("%s: %s: Valid values are 0-3\n"), PROGRAM_NAME, opt);
+		fprintf(stderr, _("%s: '%s': Valid values are 0-3\n"), PROGRAM_NAME, opt);
 		exit(EXIT_FAILURE);
 	}
 }
 #endif /* _BE_POSIX */
 
+static void
+xset_pager_view(char *arg)
+{
+	if (!arg || !*arg || *arg == '-')
+		err_arg_required("--pager-view");
+
+	if (*arg == 'a' && strcmp(arg, "auto") == 0) {
+		xargs.pager_view = conf.pager_view = PAGER_AUTO;
+	} else if (*arg == 'l' && strcmp(arg, "long") == 0) {
+		xargs.pager_view = conf.pager_view = PAGER_LONG;
+	} else if (*arg == 's' && strcmp(arg, "short") == 0) {
+		xargs.pager_view = conf.pager_view = PAGER_SHORT;
+	} else {
+		fprintf(stderr, _("%s: --pager-view: '%s': Invalid value.\n"
+			"Valid values are 'auto', 'long', and 'short'\n"),
+			PROGRAM_NAME, arg);
+		exit(EXIT_FAILURE);
+	}
+}
+
 /* Evaluate command line arguments, if any, and change initial variables to
  * their corresponding values. */
 #ifdef _BE_POSIX
@@ -1268,8 +1348,8 @@ parse_cmdline_args(const int argc, char **argv)
 
 	while ((optc = getopt(argc, argv, OPTSTRING)) != EOF) {
 		switch (optc) {
-		case 'a': conf.show_hidden = xargs.hidden = 0; break;
-		case 'A': conf.show_hidden = xargs.hidden = 1; break;
+		case 'a': conf.show_hidden = xargs.hidden = 1; break;
+		case 'A': conf.show_hidden = xargs.hidden = 0; break;
 		case 'b': xargs.bm_file = 1; set_alt_bm_file(optarg); break;
 		case 'B': set_tab_mode(optarg); break;
 		case 'c': xargs.config = 1; set_alt_config_file(optarg); break;
@@ -1295,11 +1375,13 @@ parse_cmdline_args(const int argc, char **argv)
 			fprintf(stderr, "%s: icons: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
 			exit(EXIT_FAILURE);
 #endif /* !_NO_ICONS */
+		case 'I': set_alt_trash_dir(optarg); break;
 		case 'j': set_stat(optc, optarg); break;
 		case 'J': set_stat(optc, optarg); break;
 		case 'k': set_alt_kbinds_file(optarg); break;
 		case 'l': conf.long_view = xargs.longview = 1; break;
-		case 'L': xargs.follow_symlinks = follow_symlinks = 0; break;
+		case 'L':
+			xargs.follow_symlinks_long = conf.follow_symlinks_long = 1; break;
 		case 'm': xargs.fuzzy_match = conf.fuzzy_match = 1; break;
 		case 'M': set_no_colors(); break;
 		case 'n': xargs.history = 0; break;
@@ -1412,8 +1494,8 @@ parse_cmdline_args(const int argc, char **argv)
 
 		switch (optc) {
 		/* Short options */
-		case 'a': conf.show_hidden = xargs.hidden = 0; break;
-		case 'A': conf.show_hidden = xargs.hidden = 1; break;
+		case 'a': conf.show_hidden = xargs.hidden = 1; break;
+		case 'A': conf.show_hidden = xargs.hidden = 0; break;
 		case 'b': xargs.bm_file = 1; set_alt_bm_file(optarg); break;
 		case 'c': xargs.config = 1; set_alt_config_file(optarg); break;
 
@@ -1429,8 +1511,8 @@ parse_cmdline_args(const int argc, char **argv)
 		case 'D': set_alt_config_dir(optarg); break;
 		case 'e': xargs.noeln = conf.no_eln = 1; break;
 		case 'E': xargs.eln_use_workspace_color = 1; break;
-		case 'f': conf.list_dirs_first = xargs.dirs_first = 0; break;
-		case 'F': conf.list_dirs_first = xargs.dirs_first = 1; break;
+		case 'f': conf.list_dirs_first = xargs.dirs_first = 1; break;
+		case 'F': conf.list_dirs_first = xargs.dirs_first = 0; break;
 		case 'g': conf.pager = xargs.pager = 1; break;
 		case 'G': conf.pager = xargs.pager = 0; break;
 		case 'h': help_function(); break; /* noreturn */
@@ -1438,17 +1520,19 @@ parse_cmdline_args(const int argc, char **argv)
 		case 'i': conf.case_sens_list = xargs.case_sens_list = 0; break;
 		case 'I': conf.case_sens_list = xargs.case_sens_list = 1; break;
 		case 'k': set_alt_kbinds_file(optarg); break;
-		case 'l': conf.long_view = xargs.longview = 0; break;
-		case 'L': conf.long_view = xargs.longview = 1; break;
+		case 'l': conf.long_view = xargs.longview = 1; break;
+		case 'L':
+			xargs.follow_symlinks_long = conf.follow_symlinks_long = 1; break;
 		case 'm': conf.dirhist_map = xargs.dirmap = 1; break;
-		case 'o': conf.autols = xargs.autols = 0; break;
-		case 'O': conf.autols = xargs.autols = 1; break;
+		case 'o': conf.autols = xargs.autols = 1; break;
+		case 'O': conf.autols = xargs.autols = 0; break;
 		case 'p': path_value = optarg; xargs.path = 1; break;
 		case 'P': set_alt_profile(optarg); break;
 		case 'r': xargs.refresh_on_empty_line = 0; break;
 		case 's': conf.splash_screen = xargs.splash = 1; break;
 		case 'S': xargs.stealth_mode = 1; break;
 		case 't': xargs.disk_usage_analyzer = 1; break;
+		case 'T': set_alt_trash_dir(optarg); break;
 		case 'v': print_version(); /* noreturn */
 		case 'w': set_workspace(optarg); break;
 		case 'x': conf.ext_cmd_ok = xargs.ext = 0; break;
@@ -1466,6 +1550,8 @@ parse_cmdline_args(const int argc, char **argv)
 			xargs.cd_on_quit = conf.cd_on_quit = 1; break;
 		case LOPT_COLOR_SCHEME:
 			set_color_scheme(optarg, "--color-scheme"); break;
+		case LOPT_COLOR_LNK_AS_TARGET:
+			xargs.color_lnk_as_target = conf.color_lnk_as_target = 1; break;
 		case LOPT_CWD_IN_TITLE:
 			xargs.cwd_in_title = 1; break;
 		case LOPT_DATA_DIR:
@@ -1511,6 +1597,7 @@ parse_cmdline_args(const int argc, char **argv)
 			xargs.no_dirjump = 1;
 			xargs.restore_last_path = conf.restore_last_path = 0;
 			break;
+		case LOPT_LSCOLORS: xargs.lscolors = 1; break;
 		case LOPT_MAX_DIRHIST:
 			set_max_value(optarg, &xargs.max_dirhist, &conf.max_dirhist);
 			break;
@@ -1593,8 +1680,10 @@ parse_cmdline_args(const int argc, char **argv)
 
 		case LOPT_OPENER:
 			set_opener(optarg); break;
+		case LOPT_PAGER_VIEW: xset_pager_view(optarg); break;
 		case LOPT_PRINT_SEL:
 			xargs.printsel = conf.print_selfiles = 1; break;
+		case LOPT_PROP_FIELDS: xset_prop_fields(optarg); break;
 		case LOPT_READONLY: xargs.readonly = conf.readonly = 1; break;
 		case LOPT_RL_VI_MODE:
 			xargs.rl_vi_mode = 1; break;
@@ -1636,6 +1725,8 @@ parse_cmdline_args(const int argc, char **argv)
 			set_stat(optc, optarg); break;
 		case LOPT_STDTAB:
 			set_stdtab(); break;
+		case LOPT_PTIME_STYLE: xset_time_style(optarg, 1); break;
+		case LOPT_TIME_STYLE: xset_time_style(optarg, 0); break;
 		case LOPT_TRASH_AS_RM:
 			set_trash_as_rm(); break;
 		case LOPT_VIRTUAL_DIR:
@@ -1661,7 +1752,7 @@ parse_cmdline_args(const int argc, char **argv)
 #endif /* !_NO_LIRA */
 
 	char *spath = (char *)NULL;
-	if (argv[optind]) { /* Starting path passed as positional parameter */
+	if (xargs.stat == 0 && argv[optind]) { /* Starting path passed as positional parameter */
 		spath = resolve_starting_path(argv[optind]);
 	} else {
 		if (path_value) /* Starting path passed via -p */
