@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -30,22 +30,38 @@
 
 #include "helpers.h"
 
+#if defined(__sun) && defined(ST_BTIME)
+# include <attr.h> /* getattrat, nvlist_lookup_uint64_array, nvlist_free */
+#endif /* __sun && ST_BTIME */
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
 #include <time.h>
-#include <limits.h>
 #include <readline/readline.h>
 
 #include "aux.h"
-#include "exec.h"
 #include "misc.h"
 #include "checks.h"
 #ifndef _NO_HIGHLIGHT
 # include "highlight.h"
 #endif /* !_NO_HIGHLIGHT */
+#include "spawn.h"
+
+void
+press_any_key_to_continue(const int init_newline)
+{
+	const int saved_errno = errno;
+	HIDE_CURSOR;
+	fprintf(stderr, _("%sPress any key to continue... "),
+		init_newline == 1 ? "\n" : "");
+	fflush(stderr);
+	xgetchar();
+	putchar('\n');
+	UNHIDE_CURSOR;
+	errno = saved_errno;
+}
 
 /* Print the regex error ERRCODE (returned by either regcomp(3) or regexec(3)),
  * whose compiled regular expression is REGEXP and original pattern PATTERN,
@@ -55,7 +71,7 @@ xregerror(const char *cmd_name, const char *pattern, const int errcode,
 	const regex_t regexp, const int prompt_err)
 {
 	const size_t err_len = regerror(errcode, &regexp, NULL, 0);
-	char *buf = (char *)xnmalloc(err_len + 1, sizeof(char));
+	char *buf = xnmalloc(err_len + 1, sizeof(char));
 	regerror(errcode, &regexp, buf, err_len);
 
 	if (prompt_err == 1)
@@ -66,17 +82,16 @@ xregerror(const char *cmd_name, const char *pattern, const int errcode,
 	free(buf);
 }
 
-#ifndef _NO_ICONS
 /* Generate a hash of the string STR (case sensitively if CASE_SENTITIVE is
  * set to 1).
  * Based on the sdbm algorithm (see http://www.cse.yorku.ca/~oz/hash.html),
  * released under the public-domain. */
 size_t
-hashme(const char *str, const int case_sensitive) {
+hashme(const char *restrict str, const int case_sensitive) {
 	size_t hash = 0;
 
 	/* Two while loops, so that we don't need to check CASE_SENSITIVE for
-	 * each character in STR */
+	 * each character in STR. */
 	if (case_sensitive == 1) {
 		while (*str) {
 			hash = (size_t)*str	+ (hash << 6) + (hash << 16) - hash;
@@ -91,14 +106,39 @@ hashme(const char *str, const int case_sensitive) {
 
 	return hash;
 }
-#endif /* !_NO_ICONS */
+
+#if defined(__sun) && defined(ST_BTIME)
+struct timespec
+get_birthtime(const char *filename)
+{
+	struct timespec ts;
+	ts.tv_sec = ts.tv_nsec = (time_t)-1;
+	nvlist_t *response;
+
+	if (getattrat(XAT_FDCWD, XATTR_VIEW_READWRITE, filename, &response) != 0)
+		return ts;
+
+	uint64_t *val;
+	uint_t n;
+
+	if (nvlist_lookup_uint64_array(response, A_CRTIME, &val, &n) == 0
+	&& n >= 2 && val[0] <= LONG_MAX && val[1] < 1000000000 * 2 /* for leap seconds */) {
+		ts.tv_sec = (time_t)val[0];
+		ts.tv_nsec = (time_t)val[1];
+	}
+
+	nvlist_free(response);
+
+	return ts;
+}
+#endif /* __sun && ST_BTIME */
 
 void
-gen_time_str(char *buf, const size_t size, const time_t _time)
+gen_time_str(char *buf, const size_t size, const time_t curtime)
 {
 	struct tm t;
 
-	if (_time >= 0 && localtime_r(&_time, &t)) {
+	if (curtime >= 0 && localtime_r(&curtime, &t)) {
 		*buf = '\0';
 		strftime(buf, size, DEF_TIME_STYLE_LONG, &t);
 		return;
@@ -226,7 +266,8 @@ int
 should_expand_eln(const char *text)
 {
 	char *l = rl_line_buffer;
-	if (!l || !*l || !is_number(text))
+	             /* Do not expand numbers starting with zero */
+	if (!l || !*l || *text == '0' || !is_number(text))
 		return 0;
 
 	/* Exclude 'ws' and 'st/sort' commands. */
@@ -321,7 +362,7 @@ abbreviate_file_name(char *str)
 	&& strncmp(str, workspaces[cur_ws].path, wlen) == 0
 	&& *(str + wlen) == '/') {
 		const size_t name_len = strlen(str + wlen + 1) + 3;
-		name = (char *)xnmalloc(name_len, sizeof(char));
+		name = xnmalloc(name_len, sizeof(char));
 		snprintf(name, name_len, "./%s", str + wlen + 1);
 		return name;
 	}
@@ -381,7 +422,7 @@ xmemrchr(const void *s, const int c, size_t n)
 }
 
 /* Canonicalize/normalize the path SRC without resolving symlinks.
- * SRC is deescaped if necessary.
+ * SRC is unescaped if necessary.
  * ~/./.. are resolved.
  * The resolved path is returned and must be free'd by the caller. */
 char *
@@ -395,9 +436,9 @@ normalize_path(char *src, const size_t src_len)
 	const int is_escaped = *src == '\\';
 
 	if (strchr(src, '\\')) {
-		tmp = dequote_str(src, 0);
+		tmp = unescape_str(src, 0);
 		if (!tmp) {
-			xerror(_("%s: %s: Error deescaping string\n"), PROGRAM_NAME, src);
+			xerror(_("%s: '%s': Error unescaping string\n"), PROGRAM_NAME, src);
 			return (char *)NULL;
 		}
 
@@ -414,7 +455,7 @@ normalize_path(char *src, const size_t src_len)
 	if (is_escaped == 0 && *src == '~') {
 		tmp = tilde_expand(src);
 		if (!tmp) {
-			xerror(_("%s: %s: Error expanding tilde\n"), PROGRAM_NAME, src);
+			xerror(_("%s: '%s': Error expanding tilde\n"), PROGRAM_NAME, src);
 			return (char *)NULL;
 		}
 		const size_t tlen = strlen(tmp);
@@ -434,7 +475,7 @@ normalize_path(char *src, const size_t src_len)
 
 	if (l == 0 || *s != '/') {
 		/* Relative path */
-		char p[PATH_MAX]; *p = '\0';
+		char p[PATH_MAX + 1]; *p = '\0';
 		char *cwd = get_cwd(p, sizeof(p), 1);
 		if (!cwd || !*cwd) {
 			xerror(_("%s: Error getting current directory\n"), PROGRAM_NAME);
@@ -446,15 +487,15 @@ normalize_path(char *src, const size_t src_len)
 		if (pwd_len == 1 && *cwd == '/') {
 			/* If CWD is root (/) do not copy anything. Just create a buffer
 			 * big enough to hold "/dir", which will be appended next */
-			res = (char *)xnmalloc(l + 2, sizeof(char));
+			res = xnmalloc(l + 2, sizeof(char));
 			res_len = 0;
 		} else {
-			res = (char *)xnmalloc(pwd_len + 1 + l + 1, sizeof(char));
+			res = xnmalloc(pwd_len + 1 + l + 1, sizeof(char));
 			memcpy(res, cwd, pwd_len);
 			res_len = pwd_len;
 		}
 	} else {
-		res = (char *)xnmalloc(l + 1, sizeof(char));
+		res = xnmalloc(l + 1, sizeof(char));
 		res_len = 0;
 	}
 
@@ -568,37 +609,84 @@ rl_ring_bell(void)
 char *
 gen_date_suffix(const struct tm tm)
 {
-	char *suffix = (char *)xnmalloc(68, sizeof(char));
+	char *suffix = xnmalloc(68, sizeof(char));
 	snprintf(suffix, 67, "%04d%02d%02d%02d%02d%02d", tm.tm_year + 1900,
 	    tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
 
 	return suffix;
 }
 
-/* Create directory DIR with permissions set to MODE (ignoring the current
- * umask value). */
+/* Create directory DIR with permissions set to MODE (this latter modified
+ * by a restrictive umask value: 077). */
 int
 xmkdir(char *dir, const mode_t mode)
 {
-	mode_t old_mask = umask(0);
+	mode_t old_mask = umask(0077);
 	int ret = mkdirat(XAT_FDCWD, dir, mode);
 	umask(old_mask);
 
 	if (ret == -1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
+}
+
+/* Same as readlinkat(3), but resolves relative symbolic links and NUL
+ * terminates the returned string (BUF). */
+ssize_t
+xreadlink(const int fd, char *restrict path, char *restrict buf,
+	const size_t bufsize)
+{
+	buf[0] = '\0';
+	ssize_t buf_len = readlinkat(fd, path, buf, bufsize - 1);
+	if (buf_len == -1)
+		return (-1);
+
+	buf[buf_len] = '\0';
+
+	if (buf_len > 1 && buf[buf_len - 1] == '/')
+		buf[buf_len - 1] = '\0';
+
+	int rem_slash = 0;
+	size_t path_len = strlen(path);
+	if (path_len > 1 && path[path_len - 1] == '/') {
+		path[path_len - 1] = '\0';
+		rem_slash = 1;
+	}
+
+	char *p = (char *)NULL;
+	if (*buf != '/' && (p = strrchr(path, '/'))) { /* Relative link */
+		*p = '\0';
+
+		char *temp = savestring(buf, strlen(buf));
+
+		if (p == path) /* Root */
+			snprintf(buf, bufsize, "/%s", temp);
+		else
+			snprintf(buf, bufsize, "%s/%s", path, temp);
+
+		*p = '/';
+		free(temp);
+	}
+
+	if (rem_slash == 1)
+		path[path_len - 1] = '/';
+
+	return buf_len;
 }
 
 /* Open a file for read only. Return a file stream associated to the file
  * named NAME and updates FD to hold the corresponding file descriptor.
- * NOTE: As stated here, file streams should be preferred over file descriptors:
+ * NOTE: As stated here, file streams are to be preferred over file descriptors:
  * https://www.gnu.org/software/libc/manual/html_node/Streams-and-File-Descriptors.html */
 FILE *
-open_fread(char *name, int *fd)
+open_fread(const char *name, int *fd)
 {
-	if (!name || !*name)
+	errno = 0;
+	if (!name || !*name) {
+		errno = EINVAL;
 		return (FILE *)NULL;
+	}
 
 	*fd = open(name, O_RDONLY);
 	if (*fd == -1)
@@ -616,14 +704,15 @@ open_fread(char *name, int *fd)
 /* Create a file for writing (truncating it to zero length if it already exists,
  * and with permissions 600).
  * Return a file stream associated to the file named NAME and update FD to hold
- * the corresponding file descriptor.
- * NOTE: As stated here, file streams should be preferred over file descriptors:
- * https://www.gnu.org/software/libc/manual/html_node/Streams-and-File-Descriptors.html*/
+ * the corresponding file descriptor. */
 FILE *
-open_fwrite(char *name, int *fd)
+open_fwrite(const char *name, int *fd)
 {
-	if (!name || !*name)
+	errno = 0;
+	if (!name || !*name) {
+		errno = EINVAL;
 		return (FILE *)NULL;
+	}
 
 	*fd = open(name, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
 	if (*fd == -1)
@@ -638,6 +727,31 @@ open_fwrite(char *name, int *fd)
 	return fp;
 }
 
+/* Open a file for appending (using permissions 600).
+ * Return a file stream associated to the file named NAME or NULL in case of
+ * error. */
+FILE *
+open_fappend(const char *name)
+{
+	errno = 0;
+	if (!name || !*name) {
+		errno = EINVAL;
+		return (FILE *)NULL;
+	}
+
+	int fd = open(name, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
+	if (fd == -1)
+		return (FILE *)NULL;
+
+	FILE *fp = fdopen(fd, "a");
+	if (!fp) {
+		close(fd);
+		return (FILE *)NULL;
+	}
+
+	return fp;
+}
+
 /* Transform S_IFXXX (MODE) into the corresponding DT_XXX constant */
 inline mode_t
 get_dt(const mode_t mode)
@@ -837,16 +951,19 @@ get_cmd_path(const char *cmd)
 		return cmd_path;
 	}
 
-	cmd_path = (char *)xnmalloc(PATH_MAX + 1, sizeof(char));
+	cmd_path = xnmalloc(PATH_MAX + 2, sizeof(char));
 
 	size_t i;
 	for (i = 0; i < path_n; i++) { /* Check each path in PATH */
-		/* Skip '.' (CWD) if running with secure environment */
-		if ((xargs.secure_env == 1 || xargs.secure_env_full == 1)
-		&& paths[i].path && *paths[i].path == '.' && !paths[i].path[1])
+		if (!paths[i].path || !*paths[i].path)
 			continue;
 
-		snprintf(cmd_path, PATH_MAX, "%s/%s", paths[i].path, cmd); /* NOLINT */
+		/* Skip '.' (CWD) if running with secure environment */
+		if ((xargs.secure_env == 1 || xargs.secure_env_full == 1)
+		&& *paths[i].path == '.' && !paths[i].path[1])
+			continue;
+
+		snprintf(cmd_path, PATH_MAX + 1, "%s/%s", paths[i].path, cmd); /* NOLINT */
 		if (access(cmd_path, X_OK) == 0)
 			return cmd_path;
 	}
@@ -856,37 +973,6 @@ get_cmd_path(const char *cmd)
 	return (char *)NULL;
 }
 
-/*
-char *
-construct_human_size(const off_t size)
-{
-	static char str[MAX_UNIT_SIZE];
-
-	const off_t base = xargs.si == 1 ? 1000 : 1024;
-	off_t s = size;
-//	uint16_t rem = 0; // much faster than off_t/long, but fails
-	off_t rem = 0;
-	uint8_t n = 0;
-
-	while (s >= base) {
-		rem = s; // reminder (integer form) of the last division
-		s = 0; // quotient part of the last division
-		while (rem >= base) {rem -= base; ++s;}
-		++n; // number of performed divisions
-	}
-
-	float float_rem = rem == 0 ? 0.00f : (float)rem / (float)base;
-
-	static const char *const u = "BKMGTPEZYRQ";
-	snprintf(str, MAX_UNIT_SIZE, "%.*f%c%c",
-		rem == 0 ? 0 : 2,
-		(float)s + float_rem,
-		u[n],
-		(u[n] != 'B' && xargs.si == 1) ? 'B' : 0);
-
-	return str;
-} */
-
 /* Convert SIZE to human readable form (at most 2 decimal places).
  * Returns a pointer to a string of at most MAX_UNIT_SIZE.
  * We follow here the du(1) notation: K, M, G... for powers of 1024
@@ -894,8 +980,7 @@ construct_human_size(const off_t size)
 char *
 construct_human_size(const off_t size)
 {
-	/* MAX_UNIT_SIZE == 10 == "1023.99YB\0" */
-	static char str[MAX_UNIT_SIZE];
+	static char str[MAX_HUMAN_SIZE + 2];
 
 	const float base = xargs.si == 1 ? 1000 : 1024;
 	static float mult_factor = 0;
@@ -910,18 +995,18 @@ construct_human_size(const off_t size)
 		++n;
 	}
 
-	int x = (int)s;
-	/* If (s == 0 || s - (float)x) == 0, then S has no reminder (zero)
+	const int x = (int)s;
+	/* If (s == 0 || s - (float)x == 0), then S has no reminder (zero)
 	 * We don't want to print the reminder when it is zero.
 	 *
 	 * R: Ronnabyte, Q: Quettabyte. It's highly unlikely to have files of
 	 * such huge sizes (and even less) in the near future, but anyway... */
 	static const char *const u = "BKMGTPEZYRQ";
-	snprintf(str, MAX_UNIT_SIZE, "%.*f%c%c",
+	snprintf(str, sizeof(str), "%.*f %c%s",
 		(s == 0.00f || s - (float)x == 0.00f) ? 0 : 2,
 		(double)s,
 		u[n],
-		(u[n] != 'B' && xargs.si == 1) ? 'B' : 0);
+		u[n] != 'B' ? (xargs.si == 1 ? "B" : "iB") : "");
 
 	return str;
 }
@@ -953,8 +1038,8 @@ check_xdu_hardlinks(const dev_t dev, const ino_t ino)
 static inline void
 add_xdu_hardlink(const dev_t dev, const ino_t ino)
 {
-	xdu_hardlinks = (struct hlink_t *)xrealloc(xdu_hardlinks,
-		(xdu_hardlink_n + 1) * sizeof(struct hlink_t));
+	xdu_hardlinks = xnrealloc(xdu_hardlinks, xdu_hardlink_n + 1,
+		sizeof(struct hlink_t));
 
 	xdu_hardlinks[xdu_hardlink_n].dev = dev;
 	xdu_hardlinks[xdu_hardlink_n].ino = ino;
@@ -1097,14 +1182,9 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 	if (!dir || !*dir)
 		return (-1);
 
-	char file[PATH_MAX];
-#if !defined(__OpenBSD__)
-	snprintf(file, sizeof(file), "%s/.duXXXXXX", xargs.stealth_mode == 1
-		? P_tmpdir : tmp_dir); /* NOLINT */
-#else
-	snprintf(file, sizeof(file), "%s/.duXXXXXXXXXX", xargs.stealth_mode == 1
-		? P_tmpdir : tmp_dir); /* NOLINT */
-#endif /* !__OpenBSD__ */
+	char file[PATH_MAX + 1];
+	snprintf(file, sizeof(file), "%s/%s", xargs.stealth_mode == 1
+		? P_tmpdir : tmp_dir, TMP_FILENAME); /* NOLINT */
 
 	int fd = mkstemp(file);
 	if (fd == -1)
@@ -1112,8 +1192,8 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 
 	int stdout_bk = dup(STDOUT_FILENO); /* Save original stdout */
 	if (stdout_bk == -1) {
+		unlinkat(fd, file, 0);
 		close(fd);
-		unlink(file);
 		return (-1);
 	}
 
@@ -1160,7 +1240,7 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 	/* We only need here the first field of the line, which is a file
 	 * size and usually takes only a few digits: since a yottabyte takes 26
 	 * digits, 32 is more than enough. */
-	char line[32];
+	char line[32]; *line = '\0';
 	if (fgets(line, (int)sizeof(line), fp) == NULL)
 		goto END;
 
@@ -1171,11 +1251,11 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 	}
 
 END:
+	unlinkat(fd, file, 0);
 	fclose(fp);
-	unlink(file);
 	return retval;
 }
-#endif // USE_XDU
+#endif /* USE_XDU */
 
 /* Return the file type of the file pointed to by LINK, or -1 in case of
  * error. Possible return values:
@@ -1190,8 +1270,11 @@ S_IFIFO: 10000 / 4096
 int
 get_link_ref(const char *link)
 {
-	if (!link || !*link)
+	errno = 0;
+	if (!link || !*link) {
+		errno = EINVAL;
 		return (-1);
+	}
 
 	struct stat a;
 	if (stat(link, &a) == -1)
@@ -1201,24 +1284,25 @@ get_link_ref(const char *link)
 }
 
 /* Transform an integer (N) into a string of chars.
- * This exists because some operating systems do not support itoa(3). */
+ * This exists because most operating systems do not provide itoa(3).
+ * It handles integer numbers between 0 and LLONG_MAX. */
 char *
 xitoa(long long n)
 {
-	if (!n)
+	if (n == 0)
 		return "0";
 
 	static char buf[32] = {0};
 	long long i = 30;
 
-	while (n && i) {
+	while (n > 0 && i > 0) {
 		long long rem = n / 10;
 		buf[i] = (char)('0' + (n - (rem * 10)));
 		n = rem;
 		--i;
 	}
 
-	return &buf[++i];
+	return &buf[i + 1];
 }
 
 /* Convert the string S into a number in the range of valid ELN's
@@ -1263,51 +1347,8 @@ xatoi(const char *s)
 	return (int)ret;
 }
 
-/* Some memory wrapper functions */
-void *
-xrealloc(void *ptr, const size_t size)
-{
-	void *p = realloc(ptr, size);
-
-	if (!p) {
-		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
-			PROGRAM_NAME, __func__, size);
-		exit(ENOMEM);
-	}
-
-	return p;
-}
-
-void *
-xcalloc(const size_t nmemb, const size_t size)
-{
-	void *p = calloc(nmemb, size);
-
-	if (!p) {
-		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
-			PROGRAM_NAME, __func__, nmemb * size);
-		exit(ENOMEM);
-	}
-
-	return p;
-}
-
-void *
-xnmalloc(const size_t nmemb, const size_t size)
-{
-	void *p = malloc(nmemb * size);
-
-	if (!p) {
-		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
-			PROGRAM_NAME, __func__, nmemb * size);
-		exit(ENOMEM);
-	}
-
-	return p;
-}
-
 /* Unlike getchar(3) this function does not wait for newline ('\n').
-https://stackoverflow.com/questions/12710582/how-can-i-capture-a-key-stroke-immediately-in-linux 
+https://stackoverflow.com/questions/12710582/how-can-i-capture-a-key-stroke-immediately-in-linux
 */
 char
 xgetchar(void)
@@ -1319,6 +1360,7 @@ xgetchar(void)
 		xerror("%s: tcgetattr: %s\n", PROGRAM_NAME, strerror(errno));
 		return 0;
 	}
+
 	newt = oldt;
 	newt.c_lflag &= (tcflag_t)~(ICANON | ECHO);
 	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
@@ -1350,7 +1392,7 @@ url_encode(char *str)
 	if (!str || !*str)
 		return (char *)NULL;
 
-	char *buf = (char *)xnmalloc((strlen(str) * 3) + 1, sizeof(char));
+	char *buf = xnmalloc((strlen(str) * 3) + 1, sizeof(char));
 	/* The max lenght of our buffer is 3 times the length of STR plus
 	 * 1 extra byte for the null byte terminator: each char in STR will
 	 * be, if encoded, %XX (3 chars) */
@@ -1393,7 +1435,7 @@ url_decode(char *str)
 	if (!str || !*str)
 		return (char *)NULL;
 
-	char *buf = (char *)xnmalloc(strlen(str) + 1, sizeof(char));
+	char *buf = xnmalloc(strlen(str) + 1, sizeof(char));
 	/* The decoded string will be at most as long as the encoded string */
 
 	char *pstr = str, *pbuf = buf;
