@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -33,27 +33,13 @@
 #include "aux.h" /* xatoi(), normalize_path(), open_fread(), open_fwrite() */
 #include "bookmarks.h" /* bookmarks_function() */
 #include "checks.h" /* is_number() */
-#include "exec.h" /* launch_execv() */
 #include "file_operations.h" /* open_function() */
 #include "init.h" /* load_bookmarks() */
-#include "readline.h" /* rl_no_hist() */
 #include "messages.h" /* STEALTH_DISABLED */
-#include "listing.h" /* reload_dirlist() */
 #include "misc.h" /* err(), xerror() */
-
-#define NO_BOOKMARKS "bookmarks: No bookmarks\nUse 'bm add dir/ name' \
-to create a bookmark\nTry 'bm --help' for more information"
-
-#define BM_ADD_NO_PARAM "bookmarks: A file and a name are required\n\
-Example: 'bm add dir/ name'\nTry 'bm --help' for more information"
-
-#define BM_DEL_NO_PARAM "bookmarks: A name is required\n\
-Example: 'bm del name'\nTry 'bm --help' for more information"
-
-#define PRINT_BM_HEADER 1
-#define NO_BM_HEADER    0
-#define BM_SCREEN       1 /* The edit function is called from the bookmarks screen */
-#define NO_BM_SCREEN    0
+#include "listing.h" /* reload_dirlist() */
+#include "readline.h" /* rl_no_hist() */
+#include "spawn.h" /* launch_execv() */
 
 void
 free_bookmarks(void)
@@ -96,10 +82,22 @@ bm_prompt(const int print_header)
 	while (!bm)
 		bm = rl_no_hist(bm_str);
 
-	flags |= IN_BOOKMARKS_SCREEN;
-	char **cmd = split_str(bm, NO_UPDATE_ARGS);
-	flags &= ~IN_BOOKMARKS_SCREEN;
-	free(bm);
+	char **cmd = (char **)NULL;
+	/* "e/edit" is the only command that needs to split the input string:
+	 * the second field may be an opening application. */
+	if (*bm == 'e' && (bm[1] == ' ' || strncmp(bm, "edit ", 5) == 0)) {
+		flags |= IN_BOOKMARKS_SCREEN;
+		cmd = split_str(bm, NO_UPDATE_ARGS);
+		flags &= ~IN_BOOKMARKS_SCREEN;
+		free(bm);
+	} else {
+		char *tmp = strchr(bm, '\\') ? unescape_str(bm, 0) : (char *)NULL;
+		cmd = xnmalloc(2, sizeof(char *));
+		cmd[0] = tmp ? tmp : bm;
+		cmd[1] = (char *)NULL;
+		if (tmp)
+			free(bm);
+	}
 
 	return cmd;
 }
@@ -109,12 +107,12 @@ edit_bookmarks(char *cmd, const int flag)
 {
 	struct stat a;
 	if (stat(bm_file, &a) == -1) {
-		xerror("bookmarks: %s: %s\n", bm_file, strerror(errno));
+		xerror("bookmarks: '%s': %s\n", bm_file, strerror(errno));
 		return errno;
 	}
 
-	time_t prev = a.st_mtime;
-	int ret = EXIT_SUCCESS;
+	const time_t prev = a.st_mtime;
+	int ret = FUNC_SUCCESS;
 
 	if (!cmd) {
 		open_in_foreground = 1;
@@ -125,13 +123,17 @@ edit_bookmarks(char *cmd, const int flag)
 		ret = launch_execv(tmp_cmd, FOREGROUND, E_NOFLAG);
 	}
 
-	if (ret != EXIT_SUCCESS) {
+	if (ret != FUNC_SUCCESS) {
 		if (!cmd)
-			xerror("%s\n", _("bookmarks: Error opening the bookmarks file"));
+			xerror("%s\n", _("bookmarks: Cannot open the bookmarks file"));
 		return ret;
 	}
 
-	stat(bm_file, &a);
+	if (stat(bm_file, &a) == -1) {
+		xerror("bookmarks: '%s': %s\n", bm_file, strerror(errno));
+		return errno;
+	}
+
 	if (prev != a.st_mtime) {
 		reload_bookmarks();
 
@@ -141,7 +143,7 @@ edit_bookmarks(char *cmd, const int flag)
 		}
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static size_t
@@ -169,11 +171,13 @@ get_largest_shortcut(void)
 static void
 print_bookmarks(void)
 {
+	HIDE_CURSOR;
 	printf(_("%sBookmarks Manager%s\n\n"), BOLD, df_c);
 
 	struct stat attr;
-	int eln_pad = DIGINUM(bm_n);
-	size_t i, ls = get_largest_shortcut();
+	const int eln_pad = DIGINUM(bm_n);
+	const size_t ls = get_largest_shortcut();
+	size_t i;
 
 	/* Print bookmarks, taking into account shortcut, name, and path */
 	for (i = 0; i < bm_n; i++) {
@@ -215,8 +219,11 @@ print_bookmarks(void)
 		    ? uf_c : "\x1b[0m\x1b[4m")
 		    : (is_dir == 0 ? fi_c : (name_ok == 1 ? bm_c : di_c)),
 		    name_ok == 1 ? bookmarks[i].name
-		    : bookmarks[i].path, df_c); /* Path */
+		    : bookmarks[i].path, conf.colorize == 1 /* Path */
+		    ? df_c : "\x1b[0m");
 	}
+
+	UNHIDE_CURSOR;
 }
 
 static int
@@ -231,7 +238,7 @@ edit_bookmarks_func(char **arg)
 
 	char *tmp_cmd[] = {"bm", NULL};
 	bookmarks_function(tmp_cmd);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Return a pointer to the bookmark path (in the bookmarks array)
@@ -242,7 +249,7 @@ get_bm_path(char *arg)
 {
 	/* If an ELN */
 	if (is_number(arg)) {
-		int num = atoi(arg);
+		const int num = atoi(arg);
 		if (num <= 0 || (size_t)num > bm_n) {
 			xerror(_("%s: No such ELN\n"), arg);
 			return (char *)NULL;
@@ -250,22 +257,24 @@ get_bm_path(char *arg)
 		return bookmarks[num - 1].path;
 	}
 
+	char *name = remove_quotes(arg);
+
 	/* If string, check shortcuts and names */
 	size_t i;
 	for (i = 0; i < bm_n; i++) {
-		if ((bookmarks[i].shortcut && *arg == *bookmarks[i].shortcut
-		&& strcmp(arg, bookmarks[i].shortcut) == 0)
-		|| (bookmarks[i].name && *arg == *bookmarks[i].name
-		&& strcmp(arg, bookmarks[i].name) == 0)) {
+		if ((bookmarks[i].shortcut && *name == *bookmarks[i].shortcut
+		&& strcmp(name, bookmarks[i].shortcut) == 0)
+		|| (bookmarks[i].name && *name == *bookmarks[i].name
+		&& strcmp(name, bookmarks[i].name) == 0)) {
 			if (bookmarks[i].path)
 				return bookmarks[i].path;
 
-			xerror(_("%s: Invalid bookmark\n"), arg);
+			xerror(_("'%s': Invalid bookmark\n"), name);
 			return (char *)NULL;
 		}
 	}
 
-	xerror(_("%s: No such bookmark\n"), arg);
+	xerror(_("'%s': No such bookmark\n"), name);
 	return (char *)NULL;
 }
 
@@ -288,13 +297,13 @@ open_bookmark(void)
 {
 	if (!bookmarks || bm_n == 0) {
 		puts(_(NO_BOOKMARKS));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (conf.clear_screen == 1)
 		CLEAR;
 
-	int exit_status = EXIT_SUCCESS, header_printed = 0,	is_dir = 0;
+	int exit_status = FUNC_SUCCESS, header_printed = 0,	is_dir = 0;
 
 	print_bookmarks();
 
@@ -324,7 +333,7 @@ open_bookmark(void)
 		char *tmp_cmd[] = {"o", tmp_path, arg[1], NULL};
 		exit_status = open_function(tmp_cmd);
 
-		if (exit_status != EXIT_SUCCESS) {
+		if (exit_status != FUNC_SUCCESS) {
 			free_bm_input(&arg);
 			continue;
 		}
@@ -337,15 +346,15 @@ open_bookmark(void)
 	if (conf.autols == 1 && is_dir == 0)
 		reload_dirlist();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Open a bookmark by either shortcut or name. */
 static int
 bm_open(char **cmd)
 {
-	int exit_status = EXIT_FAILURE;
-	char *p = dequote_str(cmd[1], 0);
+	int exit_status = FUNC_FAILURE;
+	char *p = unescape_str(cmd[1], 0);
 	if (!p)
 		p = cmd[1];
 
@@ -359,7 +368,7 @@ bm_open(char **cmd)
 			continue;
 
 		if (!bookmarks[i].path) {
-			xerror(_("%s: Invalid bookmark\n"), p);
+			xerror(_("'%s': Invalid bookmark\n"), p);
 			goto END;
 		}
 
@@ -368,7 +377,7 @@ bm_open(char **cmd)
 		goto END;
 	}
 
-	xerror(_("%s: No such bookmark\n"), p);
+	xerror(_("'%s': No such bookmark\n"), p);
 
 END:
 	if (p != cmd[1])
@@ -393,7 +402,7 @@ check_bm_path(char *file)
 		if (!bookmarks[i].path || strcmp(new_path, bookmarks[i].path) != 0)
 			continue;
 
-		xerror(_("bookmarks: %s: Already bookmarked as '%s'\n"),
+		xerror(_("bookmarks: '%s': Already bookmarked as '%s'\n"),
 			new_path, bookmarks[i].name ? bookmarks[i].name
 			: (bookmarks[i].shortcut ? bookmarks[i].shortcut : _("unnamed")) );
 		free(p);
@@ -410,13 +419,20 @@ name_is_reserved_keyword(const char *name)
 	if (!name || !*name)
 		return 0;
 
-	if ( ((*name == 'e' || *name == 'd' || *name == 'a') && !*(name + 1))
+	if ( ((*name == 'e' || *name == 'd' || *name == 'a' || *name == 'q')
+	&& !*(name + 1)) || strcmp(name, "quit") == 0
 	|| strcmp(name, "edit") == 0 || strcmp(name, "del") == 0
-	|| strcmp(name, "add") == 0) {
+	|| strcmp(name, "add") == 0 || strcmp(name, "reload") == 0) {
 		xerror(_("bookmarks: '%s': Reserved bookmark keyword\n"), name);
 		return 1;
 	}
 
+	if (strchr(name, ':') || strchr(name, '[') || strchr(name, ']')) {
+		xerror(_("bookmarks: '%s': Colons and square brackets "
+			"are not allowed\n"), name);
+		return 1;
+	}
+
 	return 0;
 }
 
@@ -429,7 +445,7 @@ static int
 check_bm_name(const char *name, const int add)
 {
 	if (bm_n == 0 || !bookmarks)
-		return 1;
+		return (-1);
 
 	if (add == 1 && name_is_reserved_keyword(name) == 1)
 		return 0;
@@ -441,7 +457,7 @@ check_bm_name(const char *name, const int add)
 			continue;
 
 		if (add == 1)
-			xerror(_("bookmarks: %s: Name already in use\n"), name);
+			xerror(_("bookmarks: '%s': Name already in use\n"), name);
 		return i;
 	}
 
@@ -453,7 +469,7 @@ static int
 check_bm_shortcut(const char *shortcut, const int add)
 {
 	if (bm_n == 0 || !bookmarks)
-		return 1;
+		return (-1);
 
 	if (add == 1 && name_is_reserved_keyword(shortcut) == 1)
 		return 0;
@@ -465,7 +481,7 @@ check_bm_shortcut(const char *shortcut, const int add)
 			continue;
 
 		if (add == 1)
-			xerror(_("bookmarks: %s: Shortcut already in use\n"), shortcut);
+			xerror(_("bookmarks: '%s': Shortcut already in use\n"), shortcut);
 		return i;
 	}
 
@@ -479,11 +495,11 @@ bookmark_add(char *file, char *name, char *shortcut)
 	/* FILE and NAME are guarranteed to be non-NULL.
 	 * FILE is already dequoted. */
 
-	if (check_bm_path(file) == 1)
-		return EXIT_FAILURE;
+	if (check_bm_path(file) == 1 && rl_get_y_or_n("Continue? [y/n] ") == 0)
+		return FUNC_SUCCESS;
 
-	int exit_status = EXIT_FAILURE;
-	char *p = dequote_str(name, 0);
+	int exit_status = FUNC_FAILURE;
+	char *p = unescape_str(name, 0);
 	char *n = p ? p : name;
 	char *q = (char *)NULL;
 	char *s = (char *)NULL;
@@ -492,7 +508,7 @@ bookmark_add(char *file, char *name, char *shortcut)
 		goto ERROR;
 
 	if (shortcut) {
-		q = dequote_str(shortcut, 0);
+		q = unescape_str(shortcut, 0);
 		s = q ? q : shortcut;
 
 		if (check_bm_shortcut(s, 1) != -1)
@@ -501,16 +517,16 @@ bookmark_add(char *file, char *name, char *shortcut)
 
 	/* Once we have path, name and (optionally) shortcut, write it to the
 	 * bookmarks file. */
-	FILE *bm_fp = fopen(bm_file, "a+");
+	FILE *bm_fp = open_fappend(bm_file);
 	if (!bm_fp) {
 		exit_status = errno;
-		xerror("bookmarks: fopen: %s: %s\n", bm_file, strerror(errno));
+		xerror("bookmarks: '%s': %s\n", bm_file, strerror(errno));
 		goto ERROR;
 	}
 
 	if (fseek(bm_fp, 0L, SEEK_END) == -1) {
 		exit_status = errno;
-		xerror("bookmarks: fseek: %s: %s\n", bm_file, strerror(errno));
+		xerror("bookmarks: '%s': %s\n", bm_file, strerror(errno));
 		fclose(bm_fp);
 		goto ERROR;
 	}
@@ -537,7 +553,7 @@ bookmark_add(char *file, char *name, char *shortcut)
 
 	reload_bookmarks(); /* Update bookmarks for TAB completion */
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 
 ERROR:
 	free(p);
@@ -552,19 +568,24 @@ add_bookmark(char **cmd)
 {
 	if (!cmd || !cmd[0] || !cmd[1]) {
 		puts(BM_ADD_NO_PARAM);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	char *p = dequote_str(cmd[0], 0);
+	if (bm_n == MAX_BOOKMARKS) {
+		xerror("%s\n", _("bookmarks: Cannot add any more bookmarks"));
+		return FUNC_FAILURE;
+	}
+
+	char *p = unescape_str(cmd[0], 0);
 	if (!p) {
-		xerror(_("bookmarks: %s: Error dequoting file name\n"), cmd[0]);
-		return EXIT_FAILURE;
+		xerror(_("bookmarks: '%s': Error unescaping file name\n"), cmd[0]);
+		return FUNC_FAILURE;
 	}
 
 	if (access(p, F_OK) != 0) {
-		xerror("bookmarks: %s: %s\n", p, strerror(errno));
+		xerror("bookmarks: '%s': %s\n", p, strerror(errno));
 		free(p);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	int ret = bookmark_add(p, cmd[1], cmd[2]);
@@ -578,11 +599,11 @@ add_bookmark(char **cmd)
 static size_t
 mark_bookmarks_for_deletion(char **args, int *exit_status)
 {
-	*exit_status = EXIT_SUCCESS;
+	*exit_status = FUNC_SUCCESS;
 	size_t i, counter = 0;
 
 	for (i = 0; args[i]; i++) {
-		char *p = dequote_str(args[i], 0);
+		char *p = unescape_str(args[i], 0);
 		char *name = p ? p : args[i];
 		int index = -1;
 
@@ -590,11 +611,11 @@ mark_bookmarks_for_deletion(char **args, int *exit_status)
 		|| (index = check_bm_shortcut(name, 0)) != -1)
 		&& *bookmarks[index].path) {
 			*bookmarks[index].path = '\0';
-			printf("%s: Bookmark removed\n", name);
+			printf("'%s'\n", name);
 			counter++;
 		} else {
-			xerror("%s: No such bookmark\n", name);
-			*exit_status = EXIT_FAILURE;
+			xerror(_("'%s': No such bookmark\n"), name);
+			*exit_status = FUNC_FAILURE;
 		}
 
 		free(p);
@@ -671,16 +692,16 @@ del_bookmarks(char **args)
 {
 	if (!bookmarks || bm_n == 0) {
 		puts(NO_BOOKMARKS);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (!args || !args[0]) {
 		xerror("%s\n", BM_DEL_NO_PARAM);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	char *rstr = gen_rand_str(10);
-	char tmp_file[PATH_MAX + 12];
+	char *rstr = gen_rand_str(RAND_SUFFIX_LEN);
+	char tmp_file[PATH_MAX + 2 + RAND_SUFFIX_LEN];
 	snprintf(tmp_file, sizeof(tmp_file), "%s.%s", bm_file,
 		rstr ? rstr : "gX6&55#0fa");
 	free(rstr);
@@ -688,22 +709,23 @@ del_bookmarks(char **args)
 	int fd = 0;
 	FILE *fp = open_fread(bm_file, &fd);
 	if (!fp) {
-		xerror("%s: %s\nbookmarks: Error reading the bookmarks file\n",
+		xerror(_("'%s': %s\nbookmarks: Error reading the bookmarks file\n"),
 			bm_file, strerror(errno));
-		return EXIT_FAILURE;
+		return errno;
 	}
 
 	int tmp_fd = 0;
 	FILE *tmp_fp = open_fwrite(tmp_file, &tmp_fd);
 	if (!tmp_fp) {
-		xerror("%s: %s\nbookmarks: Error creating temporary file\n",
+		const int saved_errno = errno;
+		xerror(_("'%s': %s\nbookmarks: Error creating temporary file\n"),
 			tmp_file, strerror(errno));
 		fclose(fp);
-		return EXIT_FAILURE;
+		return saved_errno;
 	}
 
-	int exit_status = EXIT_SUCCESS;
-	size_t n = mark_bookmarks_for_deletion(args, &exit_status);
+	int exit_status = FUNC_SUCCESS;
+	const size_t n = mark_bookmarks_for_deletion(args, &exit_status);
 
 	if (n == 0) {
 		fclose(fp);
@@ -729,9 +751,13 @@ del_bookmarks(char **args)
 	fclose(tmp_fp);
 
 	if (removed > 0) {
-		unlink(bm_file);
-		rename(tmp_file, bm_file);
-		print_reload_msg("Removed %zu bookmark(s)\n", removed);
+		if (rename(tmp_file, bm_file) == 0) {
+			print_reload_msg(_("Removed %zu bookmark(s)\n"), removed);
+		} else {
+			xerror(_("bookmarks: Error updating bookmarks\nCannot rename "
+				"temporary file '%s': %s\n"), tmp_file, strerror(errno));
+			return errno;
+		}
 	} else {
 		unlink(tmp_file);
 	}
@@ -746,12 +772,12 @@ bookmarks_function(char **cmd)
 {
 	if (xargs.stealth_mode == 1) {
 		printf("%s: bookamrks: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (config_ok == 0) {
 		xerror(_("%s: Bookmarks function disabled\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (!cmd[1]) /* No arguments: load the bookmarks screen. */
@@ -768,7 +794,7 @@ bookmarks_function(char **cmd)
 
 	if (*cmd[1] == 'r' && (!cmd[1][1] || strcmp(cmd[1], "reload") == 0)) {
 		reload_bookmarks();
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	/* Shortcut or bm name: open it. */
