@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -30,8 +30,6 @@
 #include <unistd.h>
 
 #if defined(HAVE_FILE_ATTRS)
-# include <linux/fs.h> /* FS_IMMUTABLE_FL */
-# include <sys/ioctl.h> /* ioctl(3) */
 # ifdef __TINYC__
 #  undef SYNC_FILE_RANGE_WRITE_AND_WAIT /* Silence redefinition error */
 # endif
@@ -75,28 +73,26 @@ is_file_in_cwd(char *name)
 	if (!name || !*name || !workspaces[cur_ws].path)
 		return 0;
 
-	char *s = strchr(name, '/');
+	const char *s = strchr(name, '/');
 	if (!s || !*(s + 1)) /* 'name' or 'name/' */
 		return 1;
 
-	char rpath[PATH_MAX];
-	*rpath = '\0';
-	char *ret = realpath(name, rpath);
-	if (!ret || !*rpath)
+	char *rpath = normalize_path(name, strlen(name));
+	if (!rpath)
 		return 0;
 
-	char *cwd = workspaces[cur_ws].path;
-	size_t cwd_len = strlen(cwd);
-	size_t rpath_len = strlen(rpath);
-	if (rpath_len < cwd_len)
-		return 0;
+	const char *cwd = workspaces[cur_ws].path;
+	const size_t cwd_len = strlen(cwd);
+	const size_t rpath_len = strlen(rpath);
 
-	if (strncmp(rpath, cwd, cwd_len) != 0)
-		return 0;
-
-	if (strchr(rpath + cwd_len + 1, '/'))
+	if (rpath_len < cwd_len
+	|| strncmp(rpath, cwd, cwd_len) != 0
+	|| (rpath_len > cwd_len && strchr(rpath + cwd_len + 1, '/'))) {
+		free(rpath);
 		return 0;
+	}
 
+	free(rpath);
 	return 1;
 }
 
@@ -105,8 +101,8 @@ is_url(const char *url)
 {
 	if ((*url == 'w' && url[1] == 'w' && url[2] == 'w' && url[3] == '.'
 	&& url[4]) || strstr(url, "://") != NULL)
-		return EXIT_SUCCESS;
-	return EXIT_FAILURE;
+		return FUNC_SUCCESS;
+	return FUNC_FAILURE;
 }
 
 /* See https://github.com/termstandard/colors#truecolor-detection */
@@ -129,6 +125,8 @@ set_term_caps(const int i)
 
 	if (i == -1) { /* TERM not found in our terminfo database */
 		term_caps.color = true_color == 1 ? TRUE_COLOR : 0;
+		if (term_caps.color <= 8)
+			memset(dim_c, '\0', sizeof(dim_c));
 		term_caps.suggestions = 0;
 		term_caps.pager = 0;
 		term_caps.hide_cursor = 0;
@@ -146,6 +144,8 @@ set_term_caps(const int i)
 
 	term_caps.color = true_color == 1 ? TRUE_COLOR
 		: (TERM_INFO[i].color > 0 ? TERM_INFO[i].color : 0);
+	if (term_caps.color <= 8)
+		memset(dim_c, '\0', sizeof(dim_c));
 
 	term_caps.suggestions = (TERM_INFO[i].cub == 1 && TERM_INFO[i].ed == 1
 		&& TERM_INFO[i].el == 1) ? 1 : 0;
@@ -157,23 +157,24 @@ set_term_caps(const int i)
  * cursor position (needed to print suggestions). If not, disable the
  * feature accordingly. */
 static void
-check_term_support(const char *_term)
+check_term_support(const char *env_term)
 {
-	if (!_term || !*_term) {
+	if (!env_term || !*env_term) {
 		set_term_caps(-1);
 		return;
 	}
 
-	size_t i, len = strlen(_term);
-	/* Color and cursor position request support */
+	size_t i;
+	const size_t len = strlen(env_term);
+	/* Color and cursor position request support. */
 	int index = -1;
 
-	if (*_term == 'x' && strcmp(_term, "xterm-kitty") == 0)
+	if (*env_term == 'x' && strcmp(env_term, "xterm-kitty") == 0)
 		flags |= KITTY_TERM;
 
 	for (i = 0; TERM_INFO[i].name; i++) {
-		if (*_term != *TERM_INFO[i].name || len != TERM_INFO[i].len
-		|| strcmp(_term, TERM_INFO[i].name) != 0)
+		if (*env_term != *TERM_INFO[i].name || len != TERM_INFO[i].len
+		|| strcmp(env_term, TERM_INFO[i].name) != 0)
 			continue;
 
 		index = (int)i;
@@ -182,7 +183,7 @@ check_term_support(const char *_term)
 
 	if (index == -1) {
 		err('w', PRINT_PROMPT, _("%s: '%s': Terminal type not supported. "
-			"Limited functionality is expected.\n"), PROGRAM_NAME, _term);
+			"Limited functionality is expected.\n"), PROGRAM_NAME, env_term);
 	}
 
 	set_term_caps(index);
@@ -205,8 +206,8 @@ static void
 set_mount_cmd(const int udisks2ok, const int udevilok)
 {
 	if (xargs.mount_cmd == MNT_UDISKS2 && !udisks2ok && udevilok) {
-		err('w', PRINT_PROMPT, _("%s: udisks2 not found. Falling back to "
-			"udevil\n"), PROGRAM_NAME);
+		err('w', PRINT_PROMPT, _("%s: udisks2: Command not found. Falling "
+			"back to udevil\n"), PROGRAM_NAME);
 		xargs.mount_cmd = MNT_UDEVIL;
 		return;
 	}
@@ -394,7 +395,7 @@ check_third_party_cmds(void)
 }
 
 /* Return 1 if at least one of the user's groups match the file gid GID.
- * Otherwise, return 0 */
+ * Otherwise, return 0. */
 static int
 check_user_groups(const gid_t gid)
 {
@@ -418,7 +419,7 @@ check_file_access(const mode_t mode, const uid_t uid, const gid_t gid)
 
 	int f = 0; /* Hold file ownership flags */
 
-	mode_t val = (mode & (mode_t)~S_IFMT);
+	const mode_t val = (mode & (mode_t)~S_IFMT);
 	if (val & S_IRUSR) f |= R_USR;
 	if (val & S_IXUSR) f |= X_USR;
 
@@ -461,7 +462,7 @@ get_sudo_path(void)
 	int ret = errno;
 
 	if (!sudo_path) {
-		xerror("%s: %s: %s\n", PROGRAM_NAME, sudo_cmd, strerror(ENOENT));
+		xerror("%s: '%s': %s\n", PROGRAM_NAME, sudo_cmd, strerror(ENOENT));
 		errno = ret;
 		return (char *)NULL;
 	}
@@ -469,38 +470,6 @@ get_sudo_path(void)
 	return sudo_path;
 }
 
-/* Check a file's immutable bit. Returns 1 if true, zero if false, and
- * -1 in case of error */
-int
-check_immutable_bit(char *file)
-{
-	if (!file || !*file)
-		return (-1);
-
-#if !defined(FS_IOC_GETFLAGS) || !defined(FS_IMMUTABLE_FL)
-	return 0;
-#else
-	int attr = 0, fd;
-
-	fd = open(file, O_RDONLY);
-	if (fd == -1) {
-		xerror("open: %s: %s\n", file, strerror(errno));
-		return (-1);
-	}
-
-	int ret = ioctl(fd, FS_IOC_GETFLAGS, &attr);
-	int saved_errno = errno;
-	close(fd);
-
-	if (ret == -1) {
-		xerror("ioctl: %s: %s\n", file, strerror(saved_errno));
-		return (-1);
-	}
-
-	return (attr & FS_IMMUTABLE_FL) ? 1 : 0;
-#endif /* !FS_IOC_GETFLAGS || !FS_IMMUTABLE_FL */
-}
-
 /* Check whether a given string contains only digits. Returns 1 if true
  * and 0 if false. It does not work with negative numbers. */
 int
@@ -535,7 +504,7 @@ contains_digit(const char *str)
 	return (-1);
 }
 
-/* Return 1 if CMD is found in CMDS_LIST and zero otherwise. */
+/* Return 1 if CMD is found in CMDS_LIST or zero otherwise. */
 static int
 find_cmd(const struct cmdslist_t *cmds_list, const size_t list_size, char *cmd)
 {
@@ -547,7 +516,7 @@ find_cmd(const struct cmdslist_t *cmds_list, const size_t list_size, char *cmd)
 		cmd[d] = '\0';
 	}
 
-	size_t cmd_len = strlen(cmd);
+	const size_t cmd_len = strlen(cmd);
 	while (--i >= 0) {
 		if (*cmd == *cmds_list[i].name && cmd_len == cmds_list[i].len
 		&& strcmp(cmd, cmds_list[i].name) == 0) {
@@ -562,21 +531,6 @@ find_cmd(const struct cmdslist_t *cmds_list, const size_t list_size, char *cmd)
 	return found;
 }
 
-int
-is_internal_c(char *restrict cmd)
-{
-	if (find_cmd(internal_cmds, internal_cmds_n, cmd))
-		return 1;
-
-	/* Check for the search and history functions as well */
-	if ((*cmd == '/' && access(cmd, F_OK) != 0) || (*cmd == '!'
-	&& (IS_DIGIT(cmd[1]) || (cmd[1] == '-' && IS_DIGIT(cmd[2]))
-	|| cmd[1] == '!')))
-		return 1;
-
-	return 0;
-}
-
 /* Check whether S is an action name.
  * Returns 1 if true or 0 otherwise. */
 int
@@ -594,6 +548,22 @@ is_action_name(const char *s)
 	return 0;
 }
 
+/* Return 1 if CMD is an internal command, or zero otherwise. */
+int
+is_internal_c(char *restrict cmd)
+{
+	if (find_cmd(internal_cmds, internal_cmds_n, cmd))
+		return 1;
+
+	/* Check for the search and history functions as well */
+	if ((*cmd == '/' && access(cmd, F_OK) != 0) || (*cmd == '!'
+	&& (IS_DIGIT(cmd[1]) || (cmd[1] == '-' && IS_DIGIT(cmd[2]))
+	|| cmd[1] == '!')))
+		return 1;
+
+	return 0;
+}
+
 /* Check cmd against a list of internal commands. Used by parse_input_str()
  * to know whether it should perform additional expansions, like glob, regex,
  * tilde, and so on. Only internal commands dealing with ELN/filenames
@@ -601,7 +571,7 @@ is_action_name(const char *s)
 int
 is_internal(char *restrict cmd)
 {
-	static const struct cmdslist_t int_cmds[] = {
+	static struct cmdslist_t const int_cmds[] = {
 		{"ac", 2},
 		{"ad", 2},
 		{"bb", 2},
@@ -612,7 +582,6 @@ is_internal(char *restrict cmd)
 		{"br", 2},
 		{"bulk", 4},
 		{"c", 1},
-//		{"cp", 2},
 		{"cd", 2},
 		{"d", 1},
 		{"dup", 3},
@@ -623,7 +592,6 @@ is_internal(char *restrict cmd)
 		{"l", 1},
 		{"le", 2},
 		{"m", 1},
-//		{"mv", 2},
 		{"mm", 2},
 		{"mime", 4},
 		{"n", 1},
@@ -671,7 +639,7 @@ is_internal(char *restrict cmd)
 int
 is_internal_f(const char *restrict cmd)
 {
-	/* If we are completing/suggesting, do not take 'ws', 'mf', and 'st'
+	/* If we are completing/suggesting, do not take 'ws', 'mf', and 'st/sort'
 	 * commands into account: they do not take ELN/filenames as parameters,
 	 * but just numbers, in which case no ELN-filename completion should
 	 * be made. */
@@ -680,7 +648,7 @@ is_internal_f(const char *restrict cmd)
 	|| (*cmd == 's' && (*(cmd + 1) == 't' || *(cmd + 1) == 'o')) ) )
 		return 0;
 
-	static const struct cmdslist_t int_cmds[] = {
+	static struct cmdslist_t const int_cmds[] = {
 		{"ac", 2},
 		{"ad", 2},
 		{"alias", 5}, /* 'alias import' takes file names */
@@ -695,7 +663,6 @@ is_internal_f(const char *restrict cmd)
 		{"br", 2},
 		{"bulk", 4},
 		{"c", 1},
-//		{"cp", 2},
 		{"cd", 2},
 		{"d", 1},
 		{"dup", 3},
@@ -708,9 +675,7 @@ is_internal_f(const char *restrict cmd)
 		{"m", 1},
 		{"mime", 4},
 		{"mm", 2},
-//		{"mv", 2},
 		{"md", 2},
-//		{"mkdir", 5},
 		{"mf", 2},
 		{"n", 1},
 		{"new", 3},
@@ -726,7 +691,6 @@ is_internal_f(const char *restrict cmd)
 		{"paste", 5},
 		{"pin", 3},
 		{"r", 1},
-//		{"rm", 2},
 		{"rr", 2},
 		{"s", 1},
 		{"sel", 3},
@@ -739,7 +703,6 @@ is_internal_f(const char *restrict cmd)
 		{"ta", 2},
 		{"te", 2},
 		{"tl", 2},
-		{"unlink", 6},
 		{"v", 1},
 		{"vv", 2},
 		{"ws", 2},
@@ -751,7 +714,7 @@ is_internal_f(const char *restrict cmd)
 	static int n = 0;
 	if (n == 0)
 		n = (int)(sizeof(int_cmds) / sizeof(struct cmdslist_t)) - 1;
-	size_t cmd_len = strlen(cmd);
+	const size_t cmd_len = strlen(cmd);
 
 	int i = n;
 	while (--i >= 0) {
@@ -801,7 +764,7 @@ int
 check_regex(char *str)
 {
 	if (!str || !*str)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	int char_found = 0;
 	char *p = str;
@@ -819,9 +782,9 @@ check_regex(char *str)
 
 	/* And if STR is not a file name, take it as a possible regex */
 	if (char_found == 1 && access(str, F_OK) == -1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Returns the parsed aliased command in an array of strings if
@@ -865,8 +828,8 @@ check_for_alias(char **args)
 		/* Add input parameters, if any, to alias_comm */
 		if (args[1]) {
 			for (j = 1; args[j]; j++) {
-				alias_comm = (char **)xrealloc(alias_comm,
-				    (++args_n + 2) * sizeof(char *));
+				alias_comm = xnrealloc(alias_comm,
+				    ++args_n + 2, sizeof(char *));
 				alias_comm[args_n] = savestring(args[j], strlen(args[j]));
 			}
 		}
@@ -888,76 +851,66 @@ check_for_alias(char **args)
 void
 truncate_file(char *file, const int max, const int check_dups)
 {
-	if (config_ok == 0 || !file)
+	if (config_ok == 0 || !file || !*file)
 		return;
 
-	/* Create the file, if it doesn't exist */
-	FILE *fp = (FILE *)NULL;
+	char *tmp_name = (char *)NULL;
+	FILE *orig_fp = (FILE *)NULL;
 	struct stat attr;
 
-	int fd = 0;
+	int orig_fd = 0;
 	if (stat(file, &attr) == -1) {
-		fp = open_fwrite(file, &fd);
-		if (!fp) {
-			err(0, NOPRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, file,
+		/* File doesn't exist: create it and exit. */
+		if (!(orig_fp = open_fwrite(file, &orig_fd))) {
+			err('w', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME, file,
 				strerror(errno));
 		} else {
-			fclose(fp);
+			fclose(orig_fp);
 		}
 
-		return; /* Exit: we do not need to truncate a new empty file */
+		return;
 	}
 
-	/* Once we know the files exists, keep only MAX entries. */
-	fp = open_fread(file, &fd);
-	if (!fp) {
-		err(0, NOPRINT_PROMPT, "%s: log: %s: %s\n", PROGRAM_NAME,
-		    file, strerror(errno));
+	/* Open the original file for read. */
+	if (!(orig_fp = open_fread(file, &orig_fd))) {
+		err('w', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME, file,
+			strerror(errno));
 		return;
 	}
 
 	int n = 0, c;
 
-	/* Count newline chars to get amount of lines in the file */
-	while ((c = fgetc(fp)) != EOF) {
+	/* Count newline chars to get the amount of lines in the file. */
+	while ((c = fgetc(orig_fp)) != EOF && n < INT_MAX) {
 		if (c == '\n')
 			n++;
 	}
 
-	if (n <= max) {
-		fclose(fp);
-		return;
-	}
+	if (n <= max)
+		goto EXIT;
 
-	/* Set the file pointer to the beginning of the log file */
-	fseek(fp, 0, SEEK_SET);
+	/* Set the file pointer to the beginning of the original file. */
+	fseek(orig_fp, 0, SEEK_SET);
 
-#if !defined(__OpenBSD__)
-	char *tmp = (char *)xnmalloc(config_dir_len + 12, sizeof(char));
-	snprintf(tmp, config_dir_len + 12, "%s/log.XXXXXX", config_dir);
-#else
-	char *tmp = (char *)xnmalloc(config_dir_len + 16, sizeof(char));
-	snprintf(tmp, config_dir_len + 16, "%s/log.XXXXXXXXXX", config_dir);
-#endif /* !__OpenBSD__ */
+	const size_t len = config_dir_len + (sizeof(TMP_FILENAME) - 1) + 2;
+	tmp_name = xnmalloc(len, sizeof(char));
+	snprintf(tmp_name, len, "%s/%s", config_dir, TMP_FILENAME);
 
-	int fdd = mkstemp(tmp);
-	if (fdd == -1) {
-		xerror("log: %s: %s", tmp, strerror(errno));
-		fclose(fp);
-		free(tmp);
-		return;
-	}
-
-#ifndef HAVE_DPRINTF
-	FILE *fpp = fopen(tmp, "w");
-	if (!fpp) {
-		err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, tmp,
+	int tmp_fd = mkstemp(tmp_name);
+	if (tmp_fd == -1) {
+		err('w', PRINT_PROMPT, "%s: '%s': %s", PROGRAM_NAME, tmp_name,
 			strerror(errno));
-		fclose(fp);
-		free(tmp);
-		return;
+		goto EXIT;
+	}
+
+	FILE *tmp_fp = fdopen(tmp_fd, "w");
+	if (!tmp_fp) {
+		unlinkat(tmp_fd, tmp_name, 0);
+		err('w', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME, tmp_name,
+			strerror(errno));
+		close(tmp_fd);
+		goto EXIT;
 	}
-#endif /* !HAVE_DPRINTF */
 
 	int i = 1;
 	size_t line_size = 0;
@@ -966,19 +919,15 @@ truncate_file(char *file, const int max, const int check_dups)
 	char *prev_line = (char *)NULL;
 	size_t prev_line_size = 0;
 
-	while (getline(&line, &line_size, fp) > 0) {
-		/* Skip consecutive equal entries */
+	while (getline(&line, &line_size, orig_fp) > 0) {
+		/* Skip consecutive equal entries. */
 		if (check_dups == 1 && prev_line && line_size == prev_line_size
 		&& strcmp(line, prev_line) == 0)
 			continue;
 
-		/* Delete old entries = copy only new ones */
+		/* Delete old entries, i.e., copy only new ones. */
 		if (i++ >= n - (max - 1))
-#ifdef HAVE_DPRINTF
-			dprintf(fdd, "%s", line);
-#else
-			fprintf(fpp, "%s", line);
-#endif /* HAVE_DPRINTF */
+			fprintf(tmp_fp, "%s", line);
 
 		if (check_dups == 1) {
 			free(prev_line);
@@ -988,15 +937,12 @@ truncate_file(char *file, const int max, const int check_dups)
 	}
 
 	free(prev_line);
-
-#ifndef HAVE_DPRINTF
-	fclose(fpp);
-#endif /* !HAVE_DPRINTF */
-
 	free(line);
-	unlinkat(fd, file, 0);
-	renameat(fdd, tmp, fd, file);
-	close(fdd);
-	fclose(fp);
-	free(tmp);
+
+	renameat(tmp_fd, tmp_name, orig_fd, file);
+	fclose(tmp_fp);
+
+EXIT:
+	free(tmp_name);
+	fclose(orig_fp);
 }
