@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,25 +25,42 @@
 #ifndef COLORS_H
 #define COLORS_H
 
+#define RL_PRINTABLE    1
+#define RL_NO_PRINTABLE 0 /* Add non-printing flags (\001 and \002)*/
+
+/* Tell split_color_line weather we're dealing with interface or file
+ * type colors */
+#define SPLIT_INTERFACE_COLORS 0
+#define SPLIT_FILETYPE_COLORS  1
+
+/* Max amount of custom color variables in the color scheme file */
+#define MAX_DEFS 128
+
+/* Macros for the set_shades function */
+#define DATE_SHADES 0
+#define SIZE_SHADES 1
+
 __BEGIN_DECLS
 
 void color_codes(void);
-void colors_list(char *, const int, const int, const int);
-int  cschemes_function(char **);
+void colors_list(char *ent, const int eln, const int pad, const int new_line);
+int  cschemes_function(char **args);
 #ifndef CLIFM_SUCKLESS
 size_t get_colorschemes(void);
 #endif /* CLIFM_SUCKLESS */
-char *get_dir_color(const char *, const mode_t, const nlink_t, const filesn_t);
-char *get_ext_color(const char *, size_t *);
-char *get_file_color(const char *, const struct stat *);
-char *get_regfile_color(const char *, const struct stat *, int *);
-int  import_color_scheme(const char *);
+char *get_dir_color(const char *filename, const mode_t mode,
+	const nlink_t links, const filesn_t count);
+char *get_ext_color(const char *ext, size_t *val_len);
+char *get_file_color(const char *filename, const struct stat *attr);
+char *get_regfile_color(const char *filename, const struct stat *attr,
+	size_t *is_ext);
+int  import_color_scheme(const char *name);
 void update_warning_prompt_text_color(void);
-void remove_bold_attr(char *);
-char *remove_trash_ext(char **);
+void remove_bold_attr(char *str);
+char *remove_trash_ext(char **ent);
 void reset_filetype_colors(void);
 void reset_iface_colors(void);
-int  set_colors(const char *, const int);
+int  set_colors(const char *colorscheme, const int check_env);
 void set_default_colors(void);
 
 __END_DECLS
