@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,19 +25,24 @@
 #ifndef CONFIG_H
 #define CONFIG_H
 
+#define DUMP_CONFIG_STR  0
+#define DUMP_CONFIG_INT  1
+#define DUMP_CONFIG_BOOL 2
+
 __BEGIN_DECLS
 
 int  create_bm_file(void);
-int  create_main_config_file(char *);
 int  create_kbinds_file(void);
-int  create_mime_file(char *, const int);
+int  create_main_config_file(char *file);
+int  create_mime_file(char *file, const int new_prof);
 void create_tmp_files(void);
-int  edit_function(char **);
+int  edit_function(char **args);
 void init_config(void);
 int  reload_config(void);
-void set_div_line(char *);
-void set_env(const int);
+void set_div_line(char *line);
+void set_env(const int reload);
 void set_sel_file(void);
+void set_time_style(char *line, char **str, const int ptime);
 
 __END_DECLS
 
