@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -28,8 +28,8 @@
 #define PROGRAM_NAME_UPPERCASE "CliFM"
 #define PROGRAM_NAME "clifm"
 #define PROGRAM_DESC "The command line file manager"
-#define VERSION "1.15"
-#define DATE "Oct 25, 2023"
+#define VERSION "1.18"
+#define DATE "Mar 23, 2024"
 #define AUTHOR "L. Abramovich"
 #define CONTACT "https://github.com/leo-arch/clifm"
 #define LICENSE "GPL2+"
@@ -99,16 +99,12 @@
 #ifdef _BE_POSIX
 # undef  xstrcasestr
 # define xstrcasestr x_strcasestr
-# define _NO_GETTEXT
-# ifndef ALLOW_MEDIA
-#  define NO_MEDIA_FUNC
-# endif /* !ALLOW_MEDIA */
-# ifndef ALLOW_LIRA
-#  define _NO_LIRA
-# endif /* !ALLOW_LIRA */
-# ifndef ALLOW_ARCHIVING
-#  define _NO_ARCHIVING
-# endif /* ALLOW_ARCHIVING */
+# ifndef _NO_GETTEXT
+#  define _NO_GETTEXT
+# endif /* !_NO_GETTEXT */
+# ifndef _NO_MAGIC
+#  define _NO_MAGIC
+# endif /* !_NO_MAGIC */
 #endif /* _BE_POSIX */
 
 /* _NO_LIRA implies _NO_MAGIC */
@@ -127,6 +123,7 @@
 #endif /* !_NO_GETTEXT*/
 
 #include <glob.h>
+#include <limits.h>
 #include <regex.h>
 #include <stdlib.h>
 #include <sys/stat.h> /* S_BLKSIZE */
@@ -134,6 +131,16 @@
 /* Included here to test _DIRENT_HAVE_D_TYPE and DT macros. */
 #include <dirent.h>
 
+#if !defined(__GLIBC__) && !defined(_DIRENT_HAVE_D_TPYE)
+# if !defined(__sun) && !defined(__HAIKU__)
+#  define _DIRENT_HAVE_D_TYPE
+# endif /* !__sun && !__HAIKU__ */
+#endif /* !__GLIBC__ && !_DIRENT_HAVE_D_TYPE */
+
+#if defined(_BE_POSIX) && defined(_DIRENT_HAVE_D_TYPE)
+# undef _DIRENT_HAVE_D_TYPE
+#endif /* _BE_POSIX && _DIRENT_HAVE_D_TYPE */
+
 #ifdef CLIFM_LEGACY
 /* Replace functions not available before POSIX-1.2008. More precisely,
  * let's try to be POSIX-1.2001 compliant. This is still experimental:
@@ -153,10 +160,6 @@
 # define XAT_FDCWD AT_FDCWD /* defined in fcntl.h */
 #endif /* __sun */
 
-#if !defined(__HAIKU__) && !defined(__sun) && !defined(CLIFM_LEGACY)
-# define HAVE_DPRINTF
-#endif /* !__HAIKU__ && !__SUN && !CLIFM_LEGACY */
-
 #if !defined(_DIRENT_HAVE_D_TYPE) || !defined(DT_DIR)
 /* Systems not providing a d_type member for the stat struct do not provide
  * these macros either. We use them to convert an st_mode value to the
@@ -300,8 +303,9 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 #include "strings.h"
 #include "settings.h"
 
-#define EXIT_SUCCESS 0
-#define EXIT_FAILURE 1
+/* General exit codes for functions */
+#define FUNC_SUCCESS 0 /* True/success */
+#define FUNC_FAILURE 1 /* False/error */
 
 #ifndef PATH_MAX
 # ifdef __linux__
@@ -391,6 +395,14 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 # endif /* LINUX_STATX */
 #endif /* !_BE_POSIX */
 
+/* Do we have birth time when running in long view AND light mode? Only if
+ * the timestamp is provided directly by stat(2).
+ * For the time being, we need an extra call (statx(2)) on Linux. This should
+ * be fixed. */
+#if defined(ST_BTIME) && !defined(LINUX_STATX) && !defined(__sun)
+# define ST_BTIME_LIGHT
+#endif /* ST_BTIME && !LINUX_STATX && !__sun */
+
 /* File system events handling */
 #if defined(LINUX_INOTIFY)
 # define NUM_EVENT_SLOTS 32 /* Make room for 32 events */
@@ -444,6 +456,11 @@ extern time_t curdir_mtime;
 # define HAVE_STATFS
 #endif
 
+#if !defined(_BE_POSIX) && !defined(NO_PLEDGE) \
+&& defined(__OpenBSD__) && OpenBSD >= 201603 /* 5.9? CHECK! */
+# define HAVE_PLEDGE
+#endif
+
 #if !defined(_BE_POSIX) && defined(__linux__)
 # define LINUX_FSINFO
 #endif
@@ -455,6 +472,15 @@ extern time_t curdir_mtime;
  * this value). */
 #define MAX_SHELL_LEVEL 1000 /* This is the value used by bash 5.2 */
 
+#define MAX_UMASK 0777
+
+#define MAX_SEL       INT_MAX
+#define MAX_TRASH     INT_MAX
+#define MAX_BOOKMARKS INT_MAX
+
+/* Max length of a file size in human format */
+#define MAX_HUMAN_SIZE 10 /* "1023.99YB\0" */
+
 /* The following flags are used via an integer (FLAGS). If an integer has
  * 4 bytes, then we can use a total of 32 flags (0-31)
  * 4 * 8 == 32 bits == (1 << 31)
@@ -484,7 +510,6 @@ extern time_t curdir_mtime;
 #define KITTY_TERM          (1 << 12)
 #define NO_FIX_RL_POINT     (1 << 13)
 #define FAILED_ALIAS        (1 << 14)
-#define REMOVE_ELN          (1 << 15) // non-expanded ELN removed via 'r'
 
 /* Flags for third party binaries */
 #define FZF_BIN_OK     (1 << 0)
@@ -523,6 +548,11 @@ extern time_t curdir_mtime;
 #define GLOB_CHARS "*?[{"
 #define GLOB_REGEX_CHARS "*?[{|^+$."
 
+/* Possible values for PagerView (conf.pager_view) */
+#define PAGER_AUTO  0
+#define PAGER_LONG  1
+#define PAGER_SHORT 2
+
 /* Used by log_msg() to know wether to tell prompt() to print messages or not */
 #define PRINT_PROMPT   1
 #define NOPRINT_PROMPT 0
@@ -550,6 +580,16 @@ extern time_t curdir_mtime;
 #define BACKGROUND 1
 #define FOREGROUND 0
 
+#define EXEC_BG_PROC 0
+#define EXEC_FG_PROC 1
+
+#define E_NOEXEC    126
+#define E_NOTFOUND  127
+#define E_SIGINT    128
+
+#define NOTFOUND_MSG "Command not found"
+#define NOEXEC_MSG   "Permission denied"
+
 /* A few fixed colors */
 #define BOLD_RED   (conf.colorize == 1 ? "\x1b[1;31m" : "")
 #define BOLD_GREEN (conf.colorize == 1 ? "\x1b[1;32m" : "")
@@ -561,12 +601,12 @@ extern time_t curdir_mtime;
 /* Colors for the prompt: */
 /* \001 and \002 tell readline that color codes between them are
  * non-printing chars. This is specially useful for the prompt, i.e.,
- * when passing color codes to readline */
+ * when passing color codes to readline. */
 #define RL_NC "\001\x1b[0m\002"
 
-#define UNSET -1
-/* MinJumpRank takes -1 as a valid value. So, let's use -2 to mark it unset */
-#define JUMP_UNSET -2
+#define UNSET  -1
+/* Some options take -1 as a valid value. So, let's use -2 to mark it unset */
+#define UNSET2 -2
 
 /* Macros for the cp and mv cmds */
 #define CP_CP            0 /* cp -iRp */
@@ -600,7 +640,9 @@ extern time_t curdir_mtime;
 #define SINO       9
 #define SOWN       10
 #define SGRP       11
-#define SORT_TYPES 11
+#define SBLK       12
+#define SLNK       13
+#define SORT_TYPES 13
 
 /* Macros for the colors_list function */
 #define NO_ELN        0
@@ -666,7 +708,7 @@ extern time_t curdir_mtime;
 
 
 #define FZF_INTERNAL_PREVIEWER 1 /* clifm itself */
-/* --preview is set from either from FzfOpts in the color scheme file or from
+/* --preview is set either from FzfOpts in the color scheme file or from
  * FZF_DEFAULT_OPTS environment variable. */
 #define FZF_EXTERNAL_PREVIEWER 2
 
@@ -698,7 +740,7 @@ extern time_t curdir_mtime;
 #define TRIM_NO_EXT 1
 #define TRIM_EXT    2
 
-/* OpenBSD recommends the use of 10 trailing X's. See mkstemp(3) */
+/* OpenBSD recommends the use of 10 trailing X's. See mkstemp(3). */
 #if defined(__OpenBSD__)
 # define TMP_FILENAME ".tempXXXXXXXXXX"
 #else
@@ -709,7 +751,10 @@ extern time_t curdir_mtime;
 # define P_tmpdir "/tmp"
 #endif /* P_tmpdir */
 
-/* Macros for the get_sys_shell function */
+/* Length of random suffix appended to temp files. Used by gen_rand_str(). */
+#define RAND_SUFFIX_LEN 10
+
+/* Macros for the get_sys_shell function. */
 #define SHELL_NONE 0
 #define SHELL_BASH 1
 #define SHELL_DASH 2
@@ -727,9 +772,9 @@ extern time_t curdir_mtime;
 #define SECURE_ENV_FULL   1
 #define SECURE_ENV_IMPORT 0
 
-/* Macros for the sanitization function */
+/* Macros for the sanitization function. */
 /* Commands send to the system shell and taken from an untrusted source,
- * mostly config files, need to be sanitized first */
+ * mostly config files, need to be sanitized first. */
 #define SNT_MIME      0
 #define SNT_PROMPT    1
 #define SNT_PROFILE   2
@@ -741,30 +786,34 @@ extern time_t curdir_mtime;
 #define SNT_NONE      8 /* Trusted command: do not sanitize */
 #define SNT_BLACKLIST 9
 
-/* Macros for the TYPE field of the filter_t struct */
+/* Macros for the TYPE field of the filter_t struct. */
 #define FILTER_NONE      0
 #define FILTER_FILE_NAME 1 /* Regex */
 #define FILTER_FILE_TYPE 2 /* =x */
 #define FILTER_MIME_TYPE 3 /* @query */
 
-/* Macros for properties string fields in long view */
-#if defined(LINUX_FILE_XATTRS)
-# define PROP_FIELDS_SIZE 7 /* Seven available fields */
-#else
-# define PROP_FIELDS_SIZE 6 /* Six available fields */
-#endif /* LINUX_FILE_XATTRS */
+/* Macros for properties string fields in long view. */
+#define PROP_FIELDS_SIZE 10 /* Ten available fields */
 
 #define PERM_SYMBOLIC 1
 #define PERM_NUMERIC  2
 
+#define PROP_ID_NUM  1
+#if defined(__ANDROID__)
+# define PROP_ID_NAME PROP_ID_NUM
+#else
+# define PROP_ID_NAME 2
+#endif /* __ANDROID__ */
+
 #define PROP_TIME_ACCESS 1
 #define PROP_TIME_MOD    2
 #define PROP_TIME_CHANGE 3
+#define PROP_TIME_BIRTH  4
 
 #define PROP_SIZE_BYTES  1
 #define PROP_SIZE_HUMAN  2
 
-/* Macros for fzf_preview_border_type */
+/* Macros for fzf_preview_border_type. */
 #define FZF_BORDER_BOLD    0
 #define FZF_BORDER_BOTTOM  1
 #define FZF_BORDER_DOUBLE  2
@@ -776,14 +825,16 @@ extern time_t curdir_mtime;
 #define FZF_BORDER_TOP     8
 #define FZF_BORDER_VERT    9
 
-/* Flags to skip fuzzy matching based on what we're comparing */
+/* Flags to skip fuzzy matching based on what we're comparing. */
 #define FUZZY_FILES_ASCII 0
 #define FUZZY_FILES_UTF8  1
 #define FUZZY_BM_NAMES    2
 #define FUZZY_HISTORY     3
 #define FUZZY_ALGO_MAX    2 /* We have two fuzzy algorithms */
 
-#define JUMP_ENTRY_PURGED -1
+#define JUMP_ENTRY_PURGED        -1
+#define JUMP_ENTRY_PERMANENT     2
+#define JUMP_ENTRY_PERMANENT_CHR '+'
 
 #define MAX_TIME_STR 256
 
@@ -802,7 +853,6 @@ extern time_t curdir_mtime;
 #define FULL_STAT   2
 
 /* Function macros */
-#define itoa xitoa /* itoa does not exist in some OS's */
 #define atoi xatoi /* xatoi is just a secure atoi */
 
 #ifndef _NO_GETTEXT
@@ -811,7 +861,7 @@ extern time_t curdir_mtime;
 # define _(String) String
 #endif /* !_GETTEXT */
 
-#define strlen(s) xstrnlen(s)
+//#define strlen(s) xstrlen(s)
 
 /* Log the message and print it to STDERR, but do not store it into the
  * messages array. */
@@ -841,8 +891,13 @@ extern time_t curdir_mtime;
 #define FILE_SIZE(s) (conf.apparent_size == 1 ? (s).st_size \
 		: (s).st_blocks * S_BLKSIZE)
 
+/* Do we have the sort method S in light mode? */
+#define ST_IN_LIGHT_MODE(s) ((s) == SNAME || (s) == SVER || (s) == SINO \
+		|| (s) == SEXT || (s) == SNONE)
+
 #define UNUSED(x) (void)(x) /* Just silence the compiler's warning */
 #define TOUPPER(c) (((c) >= 'a' && (c) <= 'z') ? ((c) - 'a' + 'A') : (c))
+#define TOLOWER(c) (((c) >= 'A' && (c) <= 'Z') ? ((c) - 'A' + 'a') : (c))
 
 /* UINT_MAX is 4294967295 == 10 digits */
 #define DIGINUM(n) (((n) < 10) ? 1 \
@@ -878,8 +933,10 @@ extern time_t curdir_mtime;
                                              : 19)
 
 #define IS_DIGIT(c)    ((unsigned int)(c) >= '0' && (unsigned int)(c) <= '9')
-//#define IS_DIGIT(c)    ((unsigned int)(c) - '0' <= '9')
 #define IS_ALPHA(c)    ((unsigned int)(c) >= 'a' && (unsigned int)(c) <= 'z')
+#define IS_ALPHA_UP(c) ((unsigned int)(c) >= 'A' && (unsigned int)(c) <= 'Z')
+#define IS_ALNUM(c)    (IS_ALPHA((c)) || IS_ALPHA_UP((c)) || IS_DIGIT((c)))
+
 #define IS_COMMENT(c)  ((unsigned int)(c) == '#' || (unsigned int)(c) == ';')
 #define IS_CTRL_CHR(c) ((unsigned int)(c) < ' ')
 #define IS_NEWLINE(c)  ((unsigned int)(c) == '\n')
@@ -1027,6 +1084,7 @@ struct config_t {
 	int disk_usage;
 	int ext_cmd_ok;
 	int files_counter;
+	int follow_symlinks_long;
 	int full_dir_size;
 	int fuzzy_match;
 	int fuzzy_match_algo;
@@ -1053,10 +1111,15 @@ struct config_t {
 	int no_eln;
 	int only_dirs;
 	int pager;
+	int pager_once;
+	int pager_view;
 	int purge_jumpdb;
 	int print_selfiles;
 	int private_ws_settings;
+	int prop_fields_gap;
 	int quoting_style;
+	int read_autocmd_files;
+	int read_dothidden;
 	int readonly;
 	int relative_time;
 	int restore_last_path;
@@ -1064,6 +1127,7 @@ struct config_t {
 	int search_strategy;
 	int share_selbox;
 	int show_hidden;
+	int skip_non_alnum_prefix;
 	int sort;
 	int sort_reverse;
 	int splash_screen;
@@ -1079,7 +1143,6 @@ struct config_t {
 	int unicode;
 	int warning_prompt;
 	int welcome_message;
-	int pad2;
 };
 
 extern struct config_t conf;
@@ -1148,11 +1211,11 @@ extern struct kbinds_t *kbinds;
 /* Struct to store the dirjump database values */
 struct jump_t {
 	char *path;
+#if defined(__arm__) && !defined(__ANDROID__)
+	char *pad0;
+#endif /* __arm__ && !__ANDROID__ */
 	size_t len;
 	size_t visits;
-#ifdef __arm__
-	char *pad0;
-#endif /* __arm__ */
 	time_t first_visit;
 	time_t last_visit;
 	int keep;
@@ -1177,8 +1240,28 @@ struct alias_t {
 
 extern struct alias_t *aliases;
 
+struct groups_t {
+	char *name;
+	size_t namlen;
+	gid_t id;
+	int pad0;
+};
+
+extern struct groups_t *sys_users;
+extern struct groups_t *sys_groups;
+
+struct human_size_t {
+	char   str[MAX_HUMAN_SIZE + 6];
+	size_t len;
+	int    unit;
+	int    pad0;
+};
+
 /* Struct to store files information */
 struct fileinfo {
+	struct human_size_t human_size;
+	struct groups_t uid_i;
+	struct groups_t gid_i;
 	char *color;
 	char *ext_color;
 	char *ext_name;
@@ -1186,37 +1269,33 @@ struct fileinfo {
 	char *icon_color;
 	char *name;
 	filesn_t filesn;
-	size_t len;   /* File name len (columns needed to display file name) */
-	size_t bytes; /* Bytes consumed by file name */
-#if defined(__arm__) && !defined(__ANDROID__)
-	size_t pad0;
-#endif /* __arm__ && !__ANDROID__ */
-	time_t ltime; /* For long view mode */
+	blkcnt_t blocks;
+	size_t len;    /* File name len (columns needed to display file name) */
+	size_t bytes;  /* Bytes consumed by file name */
+	time_t ltime;  /* For long view mode */
 	time_t time;
 	ino_t inode;
 	off_t size;
-	dev_t rdev; /* 4 bytes in OpenBSD (used to calculate major and minor devs in long view) */
-	nlink_t linkn; /* 4 bytes in Solaris/OpenBSD */
-#if !defined(__sun)
-	int pad1;
-#endif /* __sun */
+	nlink_t linkn; /* 4 bytes on Solaris/BSD/HAIKU; 8 on Linux */
 	uid_t uid;
 	gid_t gid;
-	mode_t mode; /* Store st_mode (for long view mode) */
-	mode_t type; /* Store d_type value */
+	mode_t mode;   /* Store st_mode (for long view mode) */
+	mode_t type;   /* Store d_type value */
 	int dir;
-	int eln_n;
+	int eln_n;     /* Amount of digits in ELN */
 	int exec;
-	int ruser;  /* User read permission for dir */
+	int ruser;     /* User read permission for dir */
 	int symlink;
 	int sel;
 	int xattr;
 	int du_status; /* Exit status of du(1) for dir full sizes */
-	int utf8; /* Name contains at least one UTF-8 character */
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__DragonFly__) \
-|| defined(__arm__)
-	int pad2;
-#endif /* __FreeBSD__ || __NetBSD__ || __DragonFly__ || __arm__ */
+	int utf8;      /* Name contains at least one UTF-8 character */
+	int stat_err;  /* stat(2) failed for this entry */
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__APPLE__) || defined(__sun) || defined(__HAIKU__) \
+|| (defined(__arm__) && !defined(__ANDROID__))
+	int pad0;
+#endif
 };
 
 extern struct fileinfo *file_info;
@@ -1224,22 +1303,23 @@ extern struct fileinfo *file_info;
 /* Struct to store the length of the longest values in the current list of
  * files (needed to properly construct columns in long view). */
 struct maxes_t {
-	int ids;
+	int id_group;
+	int id_user;
 	int inode;
 	int files_counter;
+	int links;
 	int name;
 	int size;
+	int blocks;
 };
 
 struct devino_t {
 	ino_t ino;
 	dev_t dev; /* 4 bytes on OpenBSD, DragonFly, and Haiku */
 	char mark;
-	char pad0;
-	char pad1;
-	char pad2;
+	char pad0[3];
 #if !defined(__OpenBSD__) && !defined(__DragonFly__) && !defined(__HAIKU__)
-	int pad3;
+	int pad1;
 #endif /* !__OpenBSD__ && !__DragonFly__ && !__HAIKU__ */
 };
 
@@ -1299,6 +1379,7 @@ struct param_t {
 	int case_sens_path_comp;
 	int case_sens_list;
 	int clear_screen;
+	int color_lnk_as_target;
 	int colorize;
 	int columns;
 	int config;
@@ -1317,6 +1398,7 @@ struct param_t {
 	int dirs_first;
 	int files_counter;
 	int follow_symlinks;
+	int follow_symlinks_long;
 	int full_dir_size;
 	int fuzzy_match;
 	int fuzzy_match_algo;
@@ -1340,6 +1422,7 @@ struct param_t {
 	int list_and_quit;
 	int light;
 	int longview;
+	int lscolors;
 	int max_dirhist;
 	int max_files;
 	int max_path;
@@ -1350,9 +1433,12 @@ struct param_t {
 	int only_dirs;
 	int open;
 	int pager;
+	int pager_view;
 	int path;
 	int preview;
 	int printsel;
+	int prop_fields_str;
+	int ptime_style;
 	int readonly;
 	int refresh_on_empty_line;
 	int refresh_on_resize;
@@ -1372,6 +1458,7 @@ struct param_t {
 #ifndef _NO_SUGGESTIONS
 	int suggestions;
 #endif /* !_NO_SUGGESTIONS */
+	int time_style;
 	int tips;
 #ifndef _NO_TRASH
 	int trasrm;
@@ -1503,14 +1590,18 @@ extern struct termcaps_t term_caps;
 /* Data to be displayed in the properties string in long mode */
 struct props_t {
 	int counter; /* Files counter */
-	int perm; /* File permissions; either NUMERIC or SYMBOLIC */
-	int ids; /* User and group IDs */
-	int time; /* Time: either ACCESS, MOD, or CHANGE */
-	int size; /* File size: either HUMAN or BYTES */
+	int ids; /* User/group IDs: either NUMBER or NAME */
 	int inode; /* File inode number */
-	int xattr; /* Extended attributes */
 	int len; /* Approx len of the entire properties string taking into account
-	the above values */
+			  * the all fields and their length. */
+	int links; /* File links */
+	int blocks;
+	int no_group; /* Should we display group if IDS is set? */
+	int perm; /* File permissions: either NUMERIC or SYMBOLIC */
+	int size; /* File size: either HUMAN or BYTES */
+	int time; /* Time: either ACCESS, MOD, CHANGE, or BIRTH */
+	int xattr; /* Extended attributes */
+	int pad0;
 };
 extern struct props_t prop_fields;
 
@@ -1560,7 +1651,7 @@ struct ext_t {
 	char  *value;
 	size_t len; /* Name length */
 	size_t value_len;
-//	size_t hash;
+	size_t hash;
 };
 extern struct ext_t *ext_colors;
 
@@ -1651,14 +1742,16 @@ enum comp_type {
 
 extern enum comp_type cur_comp_type;
 
-extern struct termios orig_termios;
-
 /* Bit flag holders */
 extern int
 	flags,
 	bin_flags,
 	search_flags;
 
+extern int
+	date_shades_old_style,
+	size_shades_old_style;
+
 /* Internal state flags */
 extern int
 	argc_bk, /* A copy of argc taken from main() */
@@ -1673,7 +1766,6 @@ extern int
 	curcol,
 	dequoted,
 	dir_changed, /* flag to know is dir was changed: used by autocmds */
-	dir_out, /* Autocommands: A .cfm.out file was found in CWD*/
 	dirhist_cur_index,
 	dirhist_total_index,
 	exit_code,
@@ -1686,6 +1778,7 @@ extern int
 	int_vars,
 	internal_cmd,
 	is_sel,
+	is_cdpath,
 	jump_total_rank,
 	kbind_busy,
 	max_files,
@@ -1711,6 +1804,7 @@ extern int
 #ifndef _NO_TRASH
 	trash_ok,
 #endif /* !_NO_TRASH */
+	virtual_dir,
 	wrong_cmd,
 	xrename; /* We're running a secondary prompt for the rename function */
 
@@ -1753,7 +1847,6 @@ extern size_t *dir_icons_hashes;
 extern size_t *ext_icons_hashes;
 #endif /* !_NO_ICONS */
 
-extern struct termios shell_tmodes;
 extern pid_t own_pid;
 extern time_t props_now;
 
@@ -1761,10 +1854,6 @@ extern char
 	cur_prompt_name[NAME_MAX + 1],
 	div_line[NAME_MAX + 1],
 	hostname[HOST_NAME_MAX + 1],
-#ifndef _NO_FZF
-	finder_in_file[PATH_MAX + 1],
-	finder_out_file[PATH_MAX + 1],
-#endif /* _NO_FZF */
 	fz_match[PATH_MAX + 1], /* First regular match if fuzzy matching is enabled */
 	prop_fields_str[PROP_FIELDS_SIZE + 1],
 	invalid_time_str[MAX_TIME_STR],
@@ -1775,6 +1864,7 @@ extern char
 
 	*actions_file,
 	*alt_config_dir,
+	*alt_trash_dir,
 	*alt_bm_file,
 	*alt_config_file,
 	*alt_kbinds_file,
@@ -1809,7 +1899,6 @@ extern char
 /*	*right_prompt, */
 	*sel_file,
 	*smenutab_options_env,
-	*stat_filename,
 	*stdin_tmp_dir,
 	*sudo_cmd,
 #ifndef _NO_SUGGESTIONS
@@ -1840,6 +1929,9 @@ extern regex_t regex_hist;    /* Commands history */
 extern regex_t regex_dirhist; /* Directory history */
 extern char **environ;
 
+/* A buffer to store file names to be displayed (wide string) */
+extern char name_buf[(NAME_MAX + 1) * sizeof(wchar_t)];
+
 /* To store all the 39 color variables we use, with 46 bytes each, we need
  * a total of 1,8Kb. It's not much but it could be less if we'd use
  * dynamically allocated arrays for them (which, on the other side,
@@ -1849,15 +1941,15 @@ extern char **environ;
 extern char
 	/* File types */
 	bd_c[MAX_COLOR], /* Block device */
-	bk_c[MAX_COLOR], /* Backup files (ending with tilde) */
+	bk_c[MAX_COLOR], /* Backup/temp files */
 	ca_c[MAX_COLOR], /* Cap file */
 	cd_c[MAX_COLOR], /* Char device */
+	di_c[MAX_COLOR], /* Directory */
 	ed_c[MAX_COLOR], /* Empty dir */
 	ee_c[MAX_COLOR], /* Empty executable */
 	ex_c[MAX_COLOR], /* Executable */
 	ef_c[MAX_COLOR], /* Empty reg file */
 	fi_c[MAX_COLOR], /* Reg file */
-	di_c[MAX_COLOR], /* Directory */
 	ln_c[MAX_COLOR], /* Symlink */
 	mh_c[MAX_COLOR], /* Multi-hardlink file */
 	nd_c[MAX_COLOR], /* No read directory */
@@ -1878,10 +1970,10 @@ extern char
 
 	/* Interface */
 	bm_c[MAX_COLOR], /* Bookmarked directory */
-	fc_c[MAX_COLOR], /* Files counter */
 	df_c[MAX_COLOR], /* Default color */
 	dl_c[MAX_COLOR], /* Dividing line */
 	el_c[MAX_COLOR], /* ELN color */
+	fc_c[MAX_COLOR], /* Files counter */
 	lc_c[MAX_COLOR], /* Symlink character (ColorLinkAsTarget only) */
 	mi_c[MAX_COLOR], /* Misc indicators */
 	ts_c[MAX_COLOR], /* TAB completion suffix */
@@ -1917,16 +2009,19 @@ extern char
 	hw_c[MAX_COLOR], /* Backslash (aka whack) */
 
 	/* File properties */
+	db_c[MAX_COLOR],  /* File allocated blocks */
+	dd_c[MAX_COLOR],  /* Date (fixed color: no shading) */
+	de_c[MAX_COLOR],  /* Inode number */
+	dg_c[MAX_COLOR],  /* UID, GID */
+	dk_c[MAX_COLOR],  /* Number of links */
+	dn_c[MAX_COLOR],  /* dash (none) */
+	do_c[MAX_COLOR],  /* Octal perms */
+	dp_c[MAX_COLOR],  /* Special files (SUID, SGID, etc) */
 	dr_c[MAX_COLOR],  /* Read */
 	dw_c[MAX_COLOR],  /* Write */
 	dxd_c[MAX_COLOR], /* Execute (dirs) */
 	dxr_c[MAX_COLOR], /* Execute (reg files) */
-	dg_c[MAX_COLOR],  /* UID, GID */
-	dd_c[MAX_COLOR],  /* Date */
 	dz_c[MAX_COLOR],  /* Size (dirs) */
-	do_c[MAX_COLOR],  /* Octal representation */
-	dp_c[MAX_COLOR],  /* Special files (SUID, SGID, etc) */
-	dn_c[MAX_COLOR],  /* dash (none) */
 
     /* Colors used in the prompt, so that \001 and \002 needs to
 	 * be added. This is why MAX_COLOR + 2 */
@@ -1951,6 +2046,8 @@ extern char
 	wm_c[MAX_COLOR + 2], /* Warning msg */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
-	tmp_color[MAX_COLOR + 2]; /* A temp buffer to store color codes */
+
+	tmp_color[MAX_COLOR + 2], /* A temp buffer to store color codes */
+	dim_c[5]; /* Dimming code: not supported by all terminals */
 
 #endif /* HELPERS_H */
