@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 
 __BEGIN_DECLS
 
-char *rl_highlight(const char *, const size_t, const int);
+char *rl_highlight(char *str, const size_t pos, const int flag);
 void recolorize_line(void);
 
 __END_DECLS
