@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -29,78 +29,16 @@
 #include <string.h>
 #include <time.h>
 #include <readline/history.h>
-#include <limits.h> /* INT_MIN */
 
 #include "aux.h"
 #include "checks.h"
-#include "exec.h"
+#include "file_operations.h"
 #include "history.h"
 #include "init.h"
-#include "misc.h"
 #include "messages.h"
-#include "file_operations.h"
-
-#ifndef _NO_SPLIT_LOG
-////// TEMPORAL CODE
-/* Split the old log file: messages go into the message logs file, and commands
- * into the command logs file. */
-void
-split_old_log_file(void)
-{
-	/* 1) Create the old file path */
-	char *new_file = msgs_log_file ? msgs_log_file : cmds_log_file;
-	if (!new_file)
-		return;
-
-	char *p = strrchr(new_file, '/');
-	if (!p)
-		return;
-
-	*p = '\0';
-	char old_file[PATH_MAX];
-	snprintf(old_file, sizeof(old_file), "%s/log.clifm", new_file);
-	*p = '/';
-
-	/* 2) Open the 3 files */
-	FILE *old = fopen(old_file, "r");
-	if (!old)
-		return;
-
-	FILE *msg = fopen(msgs_log_file, "a");
-	if (!msg) {
-		fclose(old);
-		return;
-	}
-
-	FILE *cmd = fopen(cmds_log_file, "a");
-	if (!cmd) {
-		fclose(msg);
-		fclose(old);
-		return;
-	}
-
-	/* 3) Read the old file and move lines to the new ones accordingly */
-	size_t line_size = 0;
-	char *line = (char *)NULL;
-
-	while (getline(&line, &line_size, old) > 0) {
-		if (!line || !*line || (*line != 'c' && *line != 'm')
-		|| line[1] != ':' || !line[2])
-			continue;
-
-		fprintf(*line == 'm' ? msg : cmd, "%s", line + 2);
-	}
-
-	free(line);
-	fclose(old);
-	fclose(msg);
-	fclose(cmd);
-
-	/* 4) Remove the old file */
-	remove(old_file);
-}
-////////////////////
-#endif /* _NO_SPLIT_LOG */
+#include "misc.h"
+#include "readline.h" /* rl_get_y_or_n */
+#include "spawn.h"
 
 /* Return a string with the current date.
  * Used to compose log entries. */
@@ -113,7 +51,7 @@ get_date(void)
 		return (char *)NULL;
 
 	size_t date_max = MAX_TIME_STR;
-	char *date = (char *)xnmalloc(date_max + 1, sizeof(char));
+	char *date = xnmalloc(date_max + 1, sizeof(char));
 	*date = '\0';
 
 	strftime(date, date_max, "%Y-%m-%dT%T%z", &t);
@@ -128,8 +66,8 @@ print_logs(const int flag)
 
 	FILE *log_fp = fopen(file, "r");
 	if (!log_fp) {
-		err(0, NOPRINT_PROMPT, "log: %s: %s\n", file, strerror(errno));
-		return EXIT_FAILURE;
+		err(0, NOPRINT_PROMPT, "log: '%s': %s\n", file, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	size_t line_size = 0;
@@ -140,7 +78,7 @@ print_logs(const int flag)
 
 	free(line_buff);
 	fclose(log_fp);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -151,10 +89,10 @@ gen_file(char *file)
 
 	fp = open_fwrite(file, &fd);
 	if (!fp)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	fclose(fp);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Clear logs (the message logs if FLAG is CLR_MSG_LOGS, or the command logs
@@ -167,16 +105,16 @@ clear_logs(const int flag)
 	char *file = flag == MSG_LOGS ? msgs_log_file : cmds_log_file;
 
 	if (!file || !*file)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (remove(file) == -1) {
-		xerror("log: %s: %s\n", file, strerror(errno));
+		xerror("log: '%s': %s\n", file, strerror(errno));
 		return errno;
 	}
 
 	int ret = gen_file(file);
-	if (ret != EXIT_SUCCESS)
-		return EXIT_FAILURE;
+	if (ret != FUNC_SUCCESS)
+		return FUNC_FAILURE;
 
 	free(last_cmd);
 	last_cmd = savestring(flag == MSG_LOGS
@@ -186,7 +124,7 @@ clear_logs(const int flag)
 	log_cmd();
 	conf.log_cmds = bk;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Log LAST_CMD (global) into LOG_FILE */
@@ -196,11 +134,11 @@ log_cmd(void)
 	if (xargs.stealth_mode == 1 || !last_cmd || conf.log_cmds == 0) {
 		free(last_cmd);
 		last_cmd = (char *)NULL;
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (config_ok == 0 || !cmds_log_file)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Construct the log line */
 	char *date = get_date();
@@ -209,7 +147,7 @@ log_cmd(void)
 		+ (workspaces[cur_ws].path ? strlen(workspaces[cur_ws].path) : 2)
 		+ strlen(last_cmd) + 6;
 
-	char *full_log = (char *)xnmalloc(log_len, sizeof(char));
+	char *full_log = xnmalloc(log_len, sizeof(char));
 	snprintf(full_log, log_len, "[%s] %s:%s\n", date ? date : "unknown",
 		workspaces[cur_ws].path ? workspaces[cur_ws].path : "?", last_cmd);
 
@@ -218,41 +156,38 @@ log_cmd(void)
 	last_cmd = (char *)NULL;
 
 	/* Write the log into LOG_FILE */
-	FILE *log_fp;
-
-	log_fp = fopen(cmds_log_file, "a");
+	FILE *log_fp = open_fappend(cmds_log_file);
 	if (!log_fp) {
-		err('e', PRINT_PROMPT, "log: %s: %s\n", cmds_log_file, strerror(errno));
+		err('e', PRINT_PROMPT, "log: '%s': %s\n", cmds_log_file, strerror(errno));
 		free(full_log);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	fputs(full_log, log_fp);
 	free(full_log);
 	fclose(log_fp);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Write _MSG into the log file: [date] _MSG */
 static void
 write_msg_into_logfile(const char *_msg)
 {
-	FILE *msg_fp = fopen(msgs_log_file, "a");
-	if (!msg_fp) {
+	FILE *fp = open_fappend(msgs_log_file);
+	if (!fp) {
 		/* Do not log this error: We might enter into an infinite loop
 		 * trying to access a file that cannot be accessed. Just warn the user
-		 * and print the error to STDERR */
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, msgs_log_file, strerror(errno));
-		fputs("Press any key to continue... ", stdout);
-		xgetchar();
-		putchar('\n');
+		 * and print the error to STDERR. */
+		fprintf(stderr, "%s: '%s': %s\n", PROGRAM_NAME,
+			msgs_log_file, strerror(errno));
+		press_any_key_to_continue(0);
 		return;
 	}
 
 	char *date = get_date();
-	fprintf(msg_fp, "[%s] %s", date ? date : "unknown", _msg);
-	fclose(msg_fp);
+	fprintf(fp, "[%s] %s", date ? date : "unknown", _msg);
+	fclose(fp);
 	free(date);
 }
 
@@ -266,6 +201,7 @@ send_desktop_notification(char *msg)
 
 	char type[12];
 	*type = '\0';
+
 	switch (pmsg) {
 #if defined(__HAIKU__)
 	case ERROR: snprintf(type, sizeof(type), "error"); break;
@@ -298,7 +234,7 @@ send_desktop_notification(char *msg)
 	char name[NAME_MAX];
 	snprintf(name, sizeof(name), "%s: ", PROGRAM_NAME);
 	char *p = msg;
-	size_t nlen = strnlen(name, sizeof(name));
+	const size_t nlen = strnlen(name, sizeof(name));
 	int ret = strncmp(p, name, nlen);
 	if (ret == 0) {
 		if (mlen <= nlen)
@@ -309,47 +245,50 @@ send_desktop_notification(char *msg)
 #if defined(__HAIKU__)
 	char *cmd[] = {"notify", "--type", type, "--title", PROGRAM_NAME, p, NULL};
 	ret = launch_execv(cmd, FOREGROUND, E_MUTE);
-#elif defined (__APPLE__)
-	size_t msg_len = strlen(msg) + strlen(type) + strlen(PROGRAM_NAME) + 60;
-	char *_msg = (char *)xnmalloc(msg_len, sizeof(char));
-	snprintf(_msg, msg_len,
+#elif defined(__APPLE__)
+	size_t msg_len = strlen(msg) + strlen(type)
+		+ (sizeof(PROGRAM_NAME) - 1) + 60;
+	char *tmp_msg = xnmalloc(msg_len, sizeof(char));
+	snprintf(tmp_msg, msg_len,
 		"'display notification \"%s\" subtitle \"%s\" with title \"%s\"'",
 		msg, type, PROGRAM_NAME);
-	char *cmd[] = {"osascript", "-e", _msg, NULL};
+	char *cmd[] = {"osascript", "-e", tmp_msg, NULL};
 	ret = launch_execv(cmd, FOREGROUND, E_MUTE);
-	free(_msg);
+	free(tmp_msg);
 #else
 	char *cmd[] = {"notify-send", "-u", type, PROGRAM_NAME, p, NULL};
 	ret = launch_execv(cmd, FOREGROUND, E_MUTE);
 #endif /* __HAIKU__ */
 
-	if (ret == EXIT_SUCCESS)
+	if (ret == FUNC_SUCCESS)
 		return;
 
 	/* Error: warn and print the original message */
 	xerror(_("%s: Notification daemon error: %s\n"
 		"Disable desktop notifications (run 'help desktop-notifications' "
 		"for details) or %s to silence this "
-		"warning (original message printed below)\n"), PROGRAM_NAME,
-		strerror(ret), ret == ENOENT ? _("install a notification daemon")
-		: _("fix this error (consult your daemon's documentation)"));
+		"warning (original message printed below)\n"),
+		PROGRAM_NAME, strerror(ret),
+		ret == ENOENT ? _("install a notification daemon")
+		: _("fix the error (consult your daemon's documentation)"));
 	xerror("%s\n", msg);
 }
 
 /* Handle the error message MSG.
  *
  * If ADD_TO_MSGS_LIST is 1, store MSG into the messages array: MSG will be
- * accessible to the user via the 'msg' command
+ * accessible to the user via the 'msg' command.
  *
  * If PRINT_PROMPT is 1, either raise a flag to tell the next prompt to print
  * the message itself, or, if desktop notifications are enabled and LOGME is
- * not zero (ERR_NO_LOG), send the notification to the notification daemon
- * NOTE: if not zero, LOGME could be either 1 (error/warning) or -1 (notice)
+ * not zero (ERR_NO_LOG), send the notification to the notification daemon.
+ * NOTE: if not zero, LOGME could be either 1 (error/warning) or -1 (notice).
  *
- * If PRINT_PROMPT is not 1, MSG is printed directly here
+ * If PRINT_PROMPT is not 1, MSG is printed directly here.
  *
  * Finally, if logs are enabled and LOGME is 1, write the message into the log
- * file as follows: "m:[date] msg", where 'date' is YYYY-MM-DDTHH:MM:SS */
+ * file as follows: "m:[date] msg", where 'date' is YYYY-MM-DDTHH:MM:SS.
+ * */
 void
 log_msg(char *_msg, const int print_prompt, const int logme,
 	const int add_to_msgs_list)
@@ -363,7 +302,7 @@ log_msg(char *_msg, const int print_prompt, const int logme,
 
 	if (add_to_msgs_list == 1) {
 		msgs_n++;
-		messages = (char **)xrealloc(messages, (size_t)(msgs_n + 1) * sizeof(char *));
+		messages = xnrealloc(messages, (size_t)(msgs_n + 1), sizeof(char *));
 		messages[msgs_n - 1] = savestring(_msg, msg_len);
 		messages[msgs_n] = (char *)NULL;
 	}
@@ -390,9 +329,9 @@ append_to_dirhist_file(const char *dir_path)
 	if (!dirhist_file || !dir_path || !*dir_path || xargs.stealth_mode == 1)
 		return;
 
-	FILE *fp = fopen(dirhist_file, "a");
+	FILE *fp = open_fappend(dirhist_file);
 	if (!fp) {
-		xerror(_("%s: %s: Error saving directory entry: %s\n"),
+		xerror(_("%s: '%s': Error saving directory entry: %s\n"),
 			PROGRAM_NAME, dir_path, strerror(errno));
 		return;
 	}
@@ -414,8 +353,8 @@ add_to_dirhist(const char *dir_path)
 		&& strcmp(dir_path, old_pwd[dirhist_total_index - 1]) == 0)
 			return;
 
-		old_pwd = (char **)xrealloc(old_pwd,
-		    (size_t)(dirhist_total_index + 2) * sizeof(char *));
+		old_pwd = xnrealloc(old_pwd, (size_t)(dirhist_total_index + 2),
+			sizeof(char *));
 
 		dirhist_cur_index = dirhist_total_index;
 		old_pwd[dirhist_total_index] = savestring(dir_path, strlen(dir_path));
@@ -428,8 +367,8 @@ add_to_dirhist(const char *dir_path)
 
 	/* If not at the end of dirhist, add previous AND new entry */
 	else {
-		old_pwd = (char **)xrealloc(old_pwd,
-		    (size_t)(dirhist_total_index + 3) * sizeof(char *));
+		old_pwd = xnrealloc(old_pwd, (size_t)(dirhist_total_index + 3),
+			sizeof(char *));
 
 		old_pwd[dirhist_total_index] = savestring(
 		    old_pwd[dirhist_cur_index],
@@ -462,13 +401,13 @@ edit_history(char **args)
 {
 	struct stat attr;
 	if (stat(hist_file, &attr) == -1) {
-		int tmp_err = errno;
-		xerror("history: %s: %s\n", hist_file, strerror(errno));
-		return tmp_err;
+		xerror("history: '%s': %s\n", hist_file, strerror(errno));
+		return errno;
 	}
-	time_t mtime_bfr = (time_t)attr.st_mtime;
 
-	int ret = EXIT_SUCCESS;
+	const time_t mtime_bfr = attr.st_mtime;
+
+	int ret = FUNC_SUCCESS;
 
 	/* If we have an opening application (2nd argument) */
 	if (args[2]) {
@@ -480,30 +419,38 @@ edit_history(char **args)
 		open_in_foreground = 0;
 	}
 
-	if (ret != EXIT_SUCCESS)
+	if (ret != FUNC_SUCCESS)
 		return ret;
 
 	/* Get modification time after opening the config file. */
-	stat(hist_file, &attr);
+	if (stat(hist_file, &attr) == -1) {
+		xerror("history: '%s': %s\n", hist_file, strerror(errno));
+		return errno;
+	}
+
 	/* If modification times differ, the file was modified after being
 	 * opened. */
-	if (mtime_bfr != (time_t)attr.st_mtime) {
+	if (mtime_bfr != attr.st_mtime) {
 		ret = reload_history();
 		print_reload_msg(_("File modified. History entries reloaded\n"));
 		return ret;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 clear_history_func(char **args)
 {
+	if (rl_get_y_or_n(_("Clear history? [y/n] ")) == 0)
+		return FUNC_SUCCESS;
+
 	/* Let's overwrite whatever was there. */
-	FILE *hist_fp = fopen(hist_file, "w+");
+	int fd = 0;
+	FILE *hist_fp = open_fwrite(hist_file, &fd);
 	if (!hist_fp) {
-		err(0, NOPRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
-		return EXIT_FAILURE;
+		err(0, NOPRINT_PROMPT, "history: '%s': %s\n", hist_file, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	/* Do not create an empty file */
@@ -528,7 +475,7 @@ print_history_list(const int timestamp)
 		printf(" %s%-*zu%s %s\n", el_c, n, i + 1, df_c, history[i].cmd);
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -550,28 +497,28 @@ print_last_items(char *str, const int timestamp)
 		printf(" %s%-*zu%s %s\n", el_c, n, i + 1, df_c, history[i].cmd);
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 print_hist_status(void)
 {
 	printf(_("History is %s\n"), hist_status == 1 ? "enabled" : "disabled");
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 toggle_history(const char *arg)
 {
 	if (!arg || !*arg)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	switch (*arg) {
 	case 'o':
 		hist_status = (*(arg + 1) == 'n' ? 1 : 0);
 		return print_hist_status();
 	case 's': return print_hist_status();
-	default: puts(_(HISTORY_USAGE)); return EXIT_FAILURE;
+	default: puts(_(HISTORY_USAGE)); return FUNC_FAILURE;
 	}
 }
 
@@ -580,12 +527,12 @@ history_function(char **args)
 {
 	if (xargs.stealth_mode == 1) {
 		printf(_("%s: history: %s\n"), PROGRAM_NAME, STEALTH_DISABLED);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (config_ok == 0) {
 		xerror(_("%s: History function disabled\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	/* If no arguments, print the history list */
@@ -614,28 +561,28 @@ history_function(char **args)
 
 	/* None of the above */
 	puts(_(HISTORY_USAGE));
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
 get_history(void)
 {
-	if (config_ok == 0 || !hist_file) return EXIT_FAILURE;
+	if (config_ok == 0 || !hist_file) return FUNC_FAILURE;
 
 	if (current_hist_n == 0) { /* Coming from main() */
-		history = (struct history_t *)xcalloc(1, sizeof(struct history_t));
+		history = xcalloc(1, sizeof(struct history_t));
 	} else { /* Only true when comming from 'history clear' */
 		size_t i;
 		for (i = 0; history[i].cmd; i++)
 			free(history[i].cmd);
-		history = (struct history_t *)xrealloc(history, 1 * sizeof(struct history_t));
+		history = xnrealloc(history, 1, sizeof(struct history_t));
 		current_hist_n = 0;
 	}
 
 	FILE *hist_fp = fopen(hist_file, "r");
 	if (!hist_fp) {
-		err('e', PRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
-		return EXIT_FAILURE;
+		err('e', PRINT_PROMPT, "history: '%s': %s\n", hist_file, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	size_t line_size = 0;
@@ -657,8 +604,8 @@ get_history(void)
 			continue;
 		}
 
-		history = (struct history_t *)xrealloc(history,
-				(current_hist_n + 2) * sizeof(struct history_t));
+		history = xnrealloc(history, current_hist_n + 2,
+			sizeof(struct history_t));
 		history[current_hist_n].cmd = savestring(line_buff, (size_t)line_len);
 		history[current_hist_n].len = (size_t)line_len;
 		history[current_hist_n].date = tdate;
@@ -672,7 +619,7 @@ get_history(void)
 	history[current_hist_n].date = -1;
 	free(line_buff);
 	fclose(hist_fp);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 void
@@ -700,8 +647,8 @@ add_to_cmdhist(char *cmd)
 	/* For us */
 	/* Add the new input to the history array */
 	time_t tdate = time(NULL);
-	history = (struct history_t *)xrealloc(history,
-			(size_t)(current_hist_n + 2) * sizeof(struct history_t));
+	history = xnrealloc(history, (size_t)(current_hist_n + 2),
+		sizeof(struct history_t));
 	history[current_hist_n].cmd = savestring(cmd, cmd_len);
 	history[current_hist_n].len = cmd_len;
 	history[current_hist_n].date = tdate;
@@ -720,7 +667,7 @@ record_cmd(char *input)
 
 	/* Ignore entries matching HistIgnore */
 	if (conf.histignore_regex && *conf.histignore_regex
-	&& regexec(&regex_hist, input, 0, NULL, 0) == EXIT_SUCCESS)
+	&& regexec(&regex_hist, input, 0, NULL, 0) == FUNC_SUCCESS)
 		return 0;
 
 	/* Consequtively equal commands in history */
