@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -35,21 +35,16 @@
 
 __BEGIN_DECLS
 
-void add_to_cmdhist(char *);
-void add_to_dirhist(const char *);
-int  clear_logs(const int);
+void add_to_cmdhist(char *cmd);
+void add_to_dirhist(const char *dir_path);
+int  clear_logs(const int flag);
 int  get_history(void);
-int  history_function(char **);
+int  history_function(char **args);
 int  log_cmd(void);
-void log_msg(char *, const int, const int, const int);
-int  print_logs(const int);
-int  record_cmd(char *);
-
-////// TEMPORAL CODE
-#ifndef _NO_SPLIT_LOG
-void split_old_log_file(void);
-#endif /* !_NO_SPLIT_LOG */
-////////////////////
+void log_msg(char *_msg, const int print_prompt, const int logme,
+	const int add_to_msgs_list);
+int  print_logs(const int flag);
+int  record_cmd(char *input);
 
 __END_DECLS
 
