@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -38,16 +38,17 @@
 #include "aux.h"
 #include "checks.h"
 #include "colors.h" /* get_dir_color() */
-#include "exec.h"
 #include "file_operations.h"
 #include "init.h"
-#include "navigation.h"
 #include "messages.h"
 #include "misc.h"
+#include "navigation.h"
+#include "spawn.h"
 
 /* Macros to calculate directories rank extra points */
 #define BASENAME_BONUS 	300
 #define BOOKMARK_BONUS  500
+#define PERMANENT_BONUS 300
 #define PINNED_BONUS    1000
 #define WORKSPACE_BONUS 300
 #define VISIT_BONUS     200
@@ -64,10 +65,8 @@
 struct jump_entry_t {
 	char *match;
 	char *needle;
+	size_t keep;
 	size_t visits;
-#ifdef __arm__
-	char *pad0;
-#endif /* __arm__ */
 	time_t first;
 	time_t last;
 };
@@ -167,7 +166,7 @@ static int
 rank_entry(const int i, const time_t now, int *days_since_first,
 	int *hours_since_last)
 {
-	/* 86400 = 60 secs / 60 misc / 24 hours */
+	/* 86400 = 60 secs / 60 mins / 24 hours */
 	*days_since_first = (int)(now - jump_db[i].first_visit) / 86400;
 	/* 3600 = 60 secs / 60 mins */
 	*hours_since_last = (int)(now - jump_db[i].last_visit) / 3600;
@@ -177,7 +176,10 @@ rank_entry(const int i, const time_t now, int *days_since_first,
 		(int)jump_db[i].visits, &keep);
 
 	rank += calculate_bonus_credit(jump_db[i].path, (char *)NULL, &keep);
-	jump_db[i].keep = keep;
+	if (jump_db[i].keep == JUMP_ENTRY_PERMANENT)
+		rank += PERMANENT_BONUS;
+	else
+		jump_db[i].keep = keep;
 
 	return rank;
 }
@@ -198,9 +200,9 @@ free_jump_database(void)
 static int
 add_new_jump_entry(const char *dir, const size_t dir_len)
 {
-	jump_db = (struct jump_t *)xrealloc(jump_db, (jump_n + 2) * sizeof(struct jump_t));
+	jump_db = xnrealloc(jump_db, jump_n + 2, sizeof(struct jump_t));
 	jump_db[jump_n].visits = 1;
-	time_t now = time(NULL);
+	const time_t now = time(NULL);
 	jump_db[jump_n].first_visit = now;
 	jump_db[jump_n].last_visit = now;
 	jump_db[jump_n].rank = 0;
@@ -217,7 +219,7 @@ add_new_jump_entry(const char *dir, const size_t dir_len)
 	jump_db[jump_n].first_visit = -1;
 	jump_db[jump_n].last_visit = -1;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Add DIR to the jump database. If already there, just update the number of
@@ -226,7 +228,7 @@ int
 add_to_jumpdb(char *dir)
 {
 	if (xargs.no_dirjump == 1 || !dir || !*dir)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	size_t dir_len = strlen(dir);
 	if (dir_len > 1 && dir[dir_len - 1] == '/') {
@@ -235,7 +237,7 @@ add_to_jumpdb(char *dir)
 	}
 
 	if (!jump_db) {
-		jump_db = (struct jump_t *)xnmalloc(1, sizeof(struct jump_t));
+		jump_db = xnmalloc(1, sizeof(struct jump_t));
 		jump_n = 0;
 	}
 
@@ -256,7 +258,7 @@ add_to_jumpdb(char *dir)
 	}
 
 	if (new_entry == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	return add_new_jump_entry(dir, dir_len);
 }
@@ -269,7 +271,7 @@ save_jumpdb(void)
 	|| jump_n == 0)
 		return;
 
-	char jump_file[PATH_MAX];
+	char jump_file[PATH_MAX + 1];
 	snprintf(jump_file, sizeof(jump_file), "%s/jump.clifm", config_dir);
 
 	int fd = 0;
@@ -278,9 +280,9 @@ save_jumpdb(void)
 		return;
 
 	int i, reduce = 0, total_rank = 0;
-	time_t now = time(NULL);
+	const time_t now = time(NULL);
 
-	/* Calculate both total rank sum, and rank for each entry */
+	/* Calculate both total rank sum, and rank for each entry. */
 	i = (int)jump_n;
 	while (--i >= 0) {
 		if (!IS_VALID_JUMP_ENTRY(i))
@@ -295,10 +297,12 @@ save_jumpdb(void)
 	}
 
 	/* Once we have the total rank, check if we need to reduce ranks,
-	 * and then write entries into the database */
-	if (total_rank > conf.max_jump_total_rank)
+	 * and then write entries into the database. */
+	if (total_rank > conf.max_jump_total_rank && conf.max_jump_total_rank > 0)
 		reduce = (total_rank / conf.max_jump_total_rank) + 1;
 
+	char perm_chr_str[2] = "";
+
 	for (i = 0; i < (int)jump_n; i++) {
 		if (total_rank > conf.max_jump_total_rank) {
 			if (reduce) {
@@ -307,15 +311,18 @@ save_jumpdb(void)
 			}
 		}
 
-		/* Forget directories ranked below MIN_JUMP_RANK */
-		if (jump_db[i].keep != 1 && jump_db[i].rank < conf.min_jump_rank) {
+		/* Forget directories ranked below MIN_JUMP_RANK. */
+		if (jump_db[i].keep < 1 && jump_db[i].rank < conf.min_jump_rank) {
 			/* Discount from TOTAL_RANK the rank of the now forgotten
-			 * directory to keep this total up to date */
+			 * directory to keep this total up to date. */
 			total_rank -= jump_db[i].rank;
 			continue;
 		}
 
-		fprintf(fp, "%zu:%jd:%jd:%s\n", jump_db[i].visits,
+		perm_chr_str[0] = jump_db[i].keep == JUMP_ENTRY_PERMANENT
+			? JUMP_ENTRY_PERMANENT_CHR : '\0';
+		fprintf(fp, "%s%zu:%jd:%jd:%s\n",
+			perm_chr_str, jump_db[i].visits,
 			(intmax_t)jump_db[i].first_visit,
 			(intmax_t)jump_db[i].last_visit, jump_db[i].path);
 	}
@@ -327,23 +334,25 @@ save_jumpdb(void)
 int
 edit_jumpdb(char *app)
 {
-	if (config_ok == 0 || !config_dir)
-		return EXIT_FAILURE;
+	if (config_ok == 0 || !config_dir) {
+		xerror(_("je: Configuration directory not found\n"));
+		return FUNC_FAILURE;
+	}
 
 	save_jumpdb();
 
-	char jump_file[PATH_MAX];
+	char jump_file[PATH_MAX + 1];
 	snprintf(jump_file, sizeof(jump_file), "%s/jump.clifm", config_dir);
 
 	struct stat attr;
 	if (stat(jump_file, &attr) == -1) {
-		xerror("jump: %s: %s\n", jump_file, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("jump: '%s': %s\n", jump_file, strerror(errno));
+		return errno;
 	}
 
-	time_t mtime_bfr = (time_t)attr.st_mtime;
+	const time_t mtime_bfr = attr.st_mtime;
 
-	int ret = EXIT_FAILURE;
+	int ret = FUNC_FAILURE;
 	if (app && *app) {
 		char *cmd[] = {app, jump_file, NULL};
 		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
@@ -353,19 +362,22 @@ edit_jumpdb(char *app)
 		open_in_foreground = 0;
 	}
 
-	if (ret != EXIT_SUCCESS)
+	if (ret != FUNC_SUCCESS)
 		return ret;
 
-	stat(jump_file, &attr);
+	if (stat(jump_file, &attr) == -1) {
+		xerror("jump: '%s': %s\n", jump_file, strerror(errno));
+		return errno;
+	}
 
 	if (mtime_bfr == (time_t)attr.st_mtime)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (jump_db)
 		free_jump_database();
 
 	load_jumpdb();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Save jump entry into the suggestions buffer. */
@@ -373,32 +385,30 @@ static int
 save_jump_suggestion(char *str)
 {
 	if (!str || !*str)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	free(jump_suggestion);
-	size_t len = strlen(str);
+	const size_t len = strlen(str);
 
-	int slash = 0;
-	if (len > 0 && str[len - 1] == '/')
-		slash = 1;
+	const int slash = (len > 0 && str[len - 1] == '/');
 
-	size_t jump_sug_len = len + (slash == 1 ? 1 : 2);
+	const size_t jump_sug_len = len + (slash == 1 ? 1 : 2);
 	jump_suggestion = xnmalloc(jump_sug_len, sizeof(char));
 	if (slash == 0)
 		snprintf(jump_suggestion, jump_sug_len, "%s/", str);
 	else
 		xstrsncpy(jump_suggestion, str, jump_sug_len);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static char *
-get_directory_color(const char *filename, const struct stat a)
+get_directory_color(const char *filename, const struct stat *a)
 {
-	if (check_file_access(a.st_mode, a.st_uid, a.st_gid) == 0)
+	if (check_file_access(a->st_mode, a->st_uid, a->st_gid) == 0)
 		return nd_c;
 
-	if (S_ISLNK(a.st_mode)) {
+	if (S_ISLNK(a->st_mode)) {
 		char *linkname = realpath(filename, (char *)NULL);
 		if (linkname) {
 			free(linkname);
@@ -407,7 +417,7 @@ get_directory_color(const char *filename, const struct stat a)
 		return or_c;
 	}
 
-	return get_dir_color(filename, a.st_mode, a.st_nlink, -1);
+	return get_dir_color(filename, a->st_mode, a->st_nlink, -1);
 }
 
 /* Compare ranks A and B (used to sort jump entries by rank)*/
@@ -417,6 +427,12 @@ rank_cmp(const void *a, const void *b)
 	struct jump_t *pa = (struct jump_t *)a;
 	struct jump_t *pb = (struct jump_t *)b;
 
+	if (!pa->path)
+		return 0;
+
+	if (!pb->path)
+		return 1;
+
 	return (pa->rank > pb->rank);
 }
 
@@ -433,6 +449,9 @@ print_jump_table_header(void)
 		"either because it was visited in the last 24 hours, or because "
 		"it is bookmarked, pinned, or currently active in some "
 		"workspace.\n"), item);
+	printf(_("%s A plus sign next rank values means that the "
+		"corresponding directory is marked as permanent (it won't be "
+		"removed).\n"), item);
 
 	if (conf.min_jump_rank <= 0) {
 		printf(_("%s MinJumpRank is set to %d: entries will not be removed "
@@ -452,17 +471,18 @@ print_jump_table(const int reduce, const time_t now)
 {
 	if (jump_n == 0) {
 		printf("jump: Database still empty\n");
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
+	HIDE_CURSOR;
 	print_jump_table_header();
 
 	size_t i;
 	int ranks_sum = 0, visits_sum = 0;
 	int max_rank = 0, max_visits = 0, max_first = 0, max_last = 0;
-	int max_order = DIGINUM(jump_n);
+	const int max_order = DIGINUM(jump_n);
 
-	struct jump_t *tmp_jump = (struct jump_t *)xnmalloc(jump_n + 1, sizeof(struct jump_t));
+	struct jump_t *tmp_jump = xnmalloc(jump_n + 1, sizeof(struct jump_t));
 
 	for (i = 0; i < jump_n; i++) {
 		if (!IS_VALID_JUMP_ENTRY(i)) {
@@ -472,8 +492,8 @@ print_jump_table(const int reduce, const time_t now)
 
 		int days_since_first = 0, hours_since_last = 0;
 		int rank = rank_entry((int)i, now, &days_since_first, &hours_since_last);
-		int keep = jump_db[i].keep;
-		jump_db[i].keep = 0;
+//		int keep = jump_db[i].keep;
+//		jump_db[i].keep = 0;
 
 		if (reduce) {
 			int tmp_rank = rank;
@@ -484,7 +504,8 @@ print_jump_table(const int reduce, const time_t now)
 		visits_sum += (int)jump_db[i].visits;
 
 		tmp_jump[i].path = jump_db[i].path;
-		tmp_jump[i].keep = keep;
+		tmp_jump[i].keep = jump_db[i].keep;
+//		tmp_jump[i].keep = keep;
 		tmp_jump[i].rank = rank;
 		tmp_jump[i].len = jump_db[i].len;
 		tmp_jump[i].visits = jump_db[i].visits;
@@ -521,7 +542,11 @@ print_jump_table(const int reduce, const time_t now)
 		if (lstat(tmp_jump[i].path, &a) == -1)
 			color = uf_c;
 
-		char *dir_color = color == uf_c ? uf_c : get_directory_color(tmp_jump[i].path, a);
+		char *dir_color = color == uf_c ? uf_c : get_directory_color(tmp_jump[i].path, &a);
+		const int keep = tmp_jump[i].keep;
+		const char keep_char = keep == 1 ? '*'
+			: ((keep == JUMP_ENTRY_PERMANENT)
+			? JUMP_ENTRY_PERMANENT_CHR : 0);
 
 		printf(" %s%*zu\t%*zu\t%*d\t%*d\t%s%*d%s%s%c%s\t%c%s%s%s\n",
 			color != uf_c ? color : df_c, max_order, i + 1,
@@ -530,8 +555,9 @@ print_jump_table(const int reduce, const time_t now)
 			max_last, (int)tmp_jump[i].last_visit,
 		    conf.colorize == 1 ? BOLD : "", max_rank, tmp_jump[i].rank,
 		    color != uf_c ? color : "",
-		    tmp_jump[i].keep == 1 ? li_c : "", tmp_jump[i].keep == 1
-		    ? '*' : 0, (tmp_jump[i].keep == 1 && color != uf_c) ? color : "",
+		    keep == 1 ? li_c : (keep == 2 ? mi_c : ""),
+		    keep > 0 ? keep_char : 0,
+		    (keep > 0 && color != uf_c) ? color : "",
 		    (conf.colorize == 0 && color == uf_c) ? '!' : 0,
 		    dir_color, tmp_jump[i].path, df_c);
 	}
@@ -541,7 +567,8 @@ print_jump_table(const int reduce, const time_t now)
 	printf(_("\nTotal rank: %d/%d\nTotal visits: %d\n"), ranks_sum,
 	    conf.max_jump_total_rank, visits_sum);
 
-	return EXIT_SUCCESS;
+	UNHIDE_CURSOR;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -555,6 +582,9 @@ purge_invalid_entries(void)
 		if (!IS_VALID_JUMP_ENTRY(i))
 			continue;
 
+		if (jump_db[i].keep == JUMP_ENTRY_PERMANENT)
+			continue;
+
 		if (stat(jump_db[i].path, &a) == -1) {
 			printf("%s->%s %s%s%s\n", mi_c, df_c, uf_c, jump_db[i].path, df_c);
 			jump_db[i].rank = JUMP_ENTRY_PURGED;
@@ -565,9 +595,10 @@ purge_invalid_entries(void)
 	if (c == 0)
 		puts(_("jump: No invalid entries"));
 	else
-		printf(_("\njump: Purged %d invalid %s\n"), c, c == 1 ? _("entry") : _("entries"));
+		printf(_("\njump: Purged %d invalid %s\n"), c,
+			c == 1 ? _("entry") : _("entries"));
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -581,8 +612,12 @@ purge_low_ranked_entries(const int limit)
 		if (!IS_VALID_JUMP_ENTRY(i))
 			continue;
 
+		if (jump_db[i].keep == JUMP_ENTRY_PERMANENT)
+			continue;
+
 		int days_since_first = 0, hours_since_last = 0;
-		int rank = rank_entry(i, now, &days_since_first, &hours_since_last);
+		const int rank =
+			rank_entry(i, now, &days_since_first, &hours_since_last);
 		UNUSED(days_since_first); UNUSED(hours_since_last);
 
 		if (rank < limit) {
@@ -603,7 +638,7 @@ purge_low_ranked_entries(const int limit)
 		printf(_("\njump: Purged %d %s\n"),
 			c, c == 1 ? _("entry") : _("entries"));
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -614,13 +649,13 @@ purge_jump_database(char *arg)
 
 	if (!is_number(arg)) {
 		puts(JUMP_USAGE);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	int n = atoi(arg);
+	const int n = atoi(arg);
 	if (n < 0) {
-		xerror(_("jump: %s: Invalid value\n"), arg);
-		return EXIT_FAILURE;
+		xerror(_("jump: '%s': Invalid value\n"), arg);
+		return FUNC_FAILURE;
 	}
 
 	return purge_low_ranked_entries(n);
@@ -633,14 +668,14 @@ handle_jump_order(char *arg, const int mode)
 	if (!arg) {
 		if (mode == NO_SUG_JUMP)
 			fprintf(stderr, "%s\n", _(JUMP_USAGE));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (!is_number(arg)) {
 		if (mode == NO_SUG_JUMP)
 			return cd_function(arg, CD_PRINT_ERROR);
 
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	int int_order = atoi(arg);
@@ -648,7 +683,7 @@ handle_jump_order(char *arg, const int mode)
 	|| !jump_db || !jump_db[int_order - 1].path) {
 		if (mode == NO_SUG_JUMP)
 			xerror(_("jump: %s: No such order number\n"), arg);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (mode == NO_SUG_JUMP)
@@ -668,7 +703,7 @@ check_jump_params(char **args, const time_t now, const int reduce)
 
 	if (IS_HELP(args[1])) {
 		puts(_(JUMP_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*args[1] == '-' && strcmp(args[1], "--edit") == 0)
@@ -683,7 +718,7 @@ check_jump_params(char **args, const time_t now, const int reduce)
 static inline int
 mark_target_segment(char *str)
 {
-	size_t len = strlen(str);
+	const size_t len = strlen(str);
 	int segment = 0;
 
 	if (len > 0 && str[len - 1] == '/') {
@@ -719,7 +754,7 @@ get_needle(const char *needle, const char *match, const char *query,
 		return (char *)NULL;
 
 	if (segment & FIRST_SEGMENT) {
-		char p = *ret;
+		const char p = *ret;
 		*ret = '\0';
 
 		if (strrchr(match, '/') != match) {
@@ -734,23 +769,26 @@ get_needle(const char *needle, const char *match, const char *query,
 }
 
 static inline int
-rank_tmp_entry(const struct jump_entry_t entry, const time_t now,
+rank_tmp_entry(const struct jump_entry_t *entry, const time_t now,
 	const int reduce, const char *query)
 {
 	/* 86400 = 60 secs / 60 misc / 24 hours */
-	int days_since_first = (int)(now - entry.first) / 86400;
+	const int days_since_first = (int)(now - entry->first) / 86400;
 	/* 3600 = 60 secs / 60 mins */
-	int hours_since_last = (int)(now - entry.last) / 3600;
+	const int hours_since_last = (int)(now - entry->last) / 3600;
 
 	int keep = 0;
 	int rank = calculate_base_credit(days_since_first, hours_since_last,
-		(int)entry.visits, &keep);
+		(int)entry->visits, &keep);
+
+	rank += calculate_bonus_credit(entry->match, query, &keep);
+	if (entry->keep == JUMP_ENTRY_PERMANENT)
+		rank += PERMANENT_BONUS;
 
-	rank += calculate_bonus_credit(entry.match, query, &keep);
 	UNUSED(keep);
 
 	if (reduce > 0) {
-		int tmp_rank = rank;
+		const int tmp_rank = rank;
 		rank = tmp_rank / reduce;
 	}
 
@@ -779,15 +817,15 @@ dirjump(char **args, const int mode)
 {
 	if (xargs.no_dirjump == 1 && mode == NO_SUG_JUMP) {
 		printf(_("%s: Directory jumper function disabled\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	time_t now = time(NULL);
+	const time_t now = time(NULL);
 	int reduce = 0;
 
 	/* If the sum total of ranks is greater than max, divide each entry
 	 * to make the sum total less than or equal to max. */
-	if (jump_total_rank > conf.max_jump_total_rank)
+	if (jump_total_rank > conf.max_jump_total_rank && conf.max_jump_total_rank > 0)
 		reduce = (jump_total_rank / conf.max_jump_total_rank) + 1;
 
 	if (mode == NO_SUG_JUMP) {
@@ -807,7 +845,7 @@ dirjump(char **args, const int mode)
 	default:
 		xerror(_("jump: '%c': Invalid option\n"), args[0][1]);
 		fprintf(stderr, "%s\n", _(JUMP_USAGE));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	/* If ARG is an actual directory, just cd into it. */
@@ -822,8 +860,8 @@ dirjump(char **args, const int mode)
 	size_t i;
 	int j, match = 0;
 
-	struct jump_entry_t *entry = (struct jump_entry_t *)xnmalloc(
-		jump_n + 1, sizeof(struct jump_entry_t));
+	struct jump_entry_t *entry =
+		xnmalloc(jump_n + 1, sizeof(struct jump_entry_t));
 
 	for (i = 1; args[i]; i++) {
 		/* 1) Using the first parameter, get a list of matches in the
@@ -833,7 +871,7 @@ dirjump(char **args, const int mode)
 		 * string to match only the LAST segment of the path, and if it ends
 		 * with a backslash instead, we want this query to match only the
 		 * FIRST segment of the path. */
-		int segment = mark_target_segment(args[i]);
+		const int segment = mark_target_segment(args[i]);
 
 		if (match == 0) {
 			j = (int)jump_n;
@@ -884,6 +922,7 @@ dirjump(char **args, const int mode)
 				entry[match].last = jump_db[j].last_visit;
 				entry[match].needle = needle;
 				entry[match].match = jump_db[j].path;
+				entry[match].keep = (size_t)jump_db[j].keep;
 				match++;
 			}
 		}
@@ -917,7 +956,7 @@ dirjump(char **args, const int mode)
 	/* 3) Further filter the list of matches by frecency, so that only
 	 * the best ranked directory will be returned. */
 
-	int found = 0, exit_status = EXIT_FAILURE, max = -1;
+	int found = 0, exit_status = FUNC_FAILURE, max = -1;
 	char *best_ranked = (char *)NULL;
 
 	j = match;
@@ -933,7 +972,7 @@ dirjump(char **args, const int mode)
 			continue;
 		}
 
-		int rank = rank_tmp_entry(entry[j], now, reduce, args[args_n]);
+		const int rank = rank_tmp_entry(&entry[j], now, reduce, args[args_n]);
 
 		if (rank > max) {
 			max = rank;
@@ -944,12 +983,12 @@ dirjump(char **args, const int mode)
 	if (found == 0) {
 		if (mode == NO_SUG_JUMP)
 			puts(_("jump: No matches found"));
-		exit_status = EXIT_FAILURE;
+		exit_status = FUNC_FAILURE;
 		goto END;
 	}
 
 	if (jump_opt == JLIST) {
-		exit_status = EXIT_SUCCESS;
+		exit_status = FUNC_SUCCESS;
 		goto END;
 	}
 
