@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -50,14 +50,16 @@ typedef char *rl_cpvfunc_t;
 #include "aux.h"
 #include "config.h"
 #include "exec.h"
+#include "file_operations.h"
 #include "keybinds.h"
 #include "listing.h"
+#include "messages.h"
 #include "misc.h"
 #include "profiles.h"
 #include "prompt.h"
-#include "messages.h"
 #include "readline.h"
-#include "file_operations.h"
+#include "spawn.h"
+#include "tabcomp.h" /* tab_complete() */
 
 #ifndef _NO_SUGGESTIONS
 # include "suggestions.h"
@@ -69,10 +71,6 @@ typedef char *rl_cpvfunc_t;
 
 #include "strings.h" /* quote_str() */
 
-/* Let's use these word delimiters to print first suggested word
- * and to delete last typed word */
-#define WORD_DELIMITERS " /.-_=,:;@+*&$#<>%~|({[]})¿?¡!"
-
 #ifndef _NO_SUGGESTIONS
 static int accept_first_word = 0;
 #endif /* !_NO_SUGGESTIONS */
@@ -96,20 +94,26 @@ xrl_reset_line_state(void)
 int
 kbinds_reset(void)
 {
-	int exit_status = EXIT_SUCCESS;
-	struct stat file_attrib;
+	int exit_status = FUNC_SUCCESS;
+	struct stat attr;
 
-	if (stat(kbinds_file, &file_attrib) == -1) {
-		exit_status = create_kbinds_file();
-	} else {
-		char *cmd[] = {"rm", "--", kbinds_file, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) == EXIT_SUCCESS)
-			exit_status = create_kbinds_file();
-		else
-			exit_status = EXIT_FAILURE;
+	if (!kbinds_file || !*kbinds_file) {
+		xerror(_("kb: No keybindings file found\n"));
+		return FUNC_FAILURE;
 	}
 
-	if (exit_status == EXIT_SUCCESS)
+	if (stat(kbinds_file, &attr) == -1) {
+		exit_status = create_kbinds_file();
+	} else {
+		if (unlink(kbinds_file) == 0) {
+			exit_status = create_kbinds_file();
+		} else {
+			xerror("kb: '%s': %s\n", kbinds_file, strerror(errno));
+			exit_status = FUNC_FAILURE;
+		}
+	}
+
+	if (exit_status == FUNC_SUCCESS)
 		err('n', PRINT_PROMPT, _("kb: Restart %s for changes "
 			"to take effect\n"), PROGRAM_NAME);
 
@@ -121,11 +125,13 @@ kbinds_edit(char *app)
 {
 	if (xargs.stealth_mode == 1) {
 		printf("%s: kb: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	if (!kbinds_file)
-		return EXIT_FAILURE;
+	if (!kbinds_file) {
+		xerror(_("kb: No keybindings file found\n"));
+		return FUNC_FAILURE;
+	}
 
 	struct stat attr;
 	if (stat(kbinds_file, &attr) == -1) {
@@ -133,9 +139,9 @@ kbinds_edit(char *app)
 		stat(kbinds_file, &attr);
 	}
 
-	time_t mtime_bfr = (time_t)attr.st_mtime;
+	const time_t mtime_bfr = attr.st_mtime;
 
-	int ret = EXIT_SUCCESS;
+	int ret = FUNC_SUCCESS;
 	if (app && *app) {
 		char *cmd[] = {app, kbinds_file, NULL};
 		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
@@ -145,39 +151,39 @@ kbinds_edit(char *app)
 		open_in_foreground = 0;
 	}
 
-	if (ret != EXIT_SUCCESS)
+	if (ret != FUNC_SUCCESS)
 		return ret;
 
 	stat(kbinds_file, &attr);
-	if (mtime_bfr == (time_t)attr.st_mtime)
-		return EXIT_SUCCESS;
+	if (mtime_bfr == attr.st_mtime)
+		return FUNC_SUCCESS;
 
 	err('n', PRINT_PROMPT, _("kb: Restart %s for changes to "
 			"take effect\n"), PROGRAM_NAME);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
 kbinds_function(char **args)
 {
 	if (!args)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (!args[1] || strcmp(args[1], "list") == 0) {
 		if (kbinds_n == 0) {
 			printf(_("%s: kb: No keybindings defined\n"), PROGRAM_NAME);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		}
 		size_t i;
 		for (i = 0; i < kbinds_n; i++)
 			printf("%s: %s\n", kbinds[i].key, kbinds[i].function);
 
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (IS_HELP(args[1])) {
 		puts(_(KB_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*args[1] == 'e' && strcmp(args[1], "edit") == 0)
@@ -188,11 +194,11 @@ kbinds_function(char **args)
 
 	if (*args[1] == 'r' && strcmp(args[1], "readline") == 0) {
 		rl_function_dumper(1);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	fprintf(stderr, "%s\n", _(KB_USAGE));
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Store keybinds from the keybinds file into a struct */
@@ -200,7 +206,7 @@ int
 load_keybinds(void)
 {
 	if (config_ok == 0 || !kbinds_file)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Free the keybinds struct array */
 	if (kbinds_n > 0) {
@@ -212,14 +218,14 @@ load_keybinds(void)
 		}
 
 		free(kbinds);
-		kbinds = (struct kbinds_t *)xnmalloc(1, sizeof(struct kbinds_t));
+		kbinds = xnmalloc(1, sizeof(struct kbinds_t));
 		kbinds_n = 0;
 	}
 
 	/* Open the keybinds file */
 	FILE *fp = fopen(kbinds_file, "r");
 	if (!fp)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	size_t line_size = 0;
 	char *line = (char *)NULL;
@@ -239,7 +245,7 @@ load_keybinds(void)
 
 		/* Now copy left and right value of each keybind into the
 		 * keybinds struct */
-		kbinds = xrealloc(kbinds, (kbinds_n + 1) * sizeof(struct kbinds_t));
+		kbinds = xnrealloc(kbinds, kbinds_n + 1, sizeof(struct kbinds_t));
 		kbinds[kbinds_n].key = savestring(tmp + 1, strlen(tmp + 1));
 
 		*tmp = '\0';
@@ -250,7 +256,7 @@ load_keybinds(void)
 
 	fclose(fp);
 	free(line);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* This call to prompt() just updates the prompt in case it was modified by
@@ -271,6 +277,7 @@ rl_update_prompt(void)
 	UNHIDE_CURSOR;
 }
 
+#if defined(__HAIKU__) || !defined(_NO_PROFILES)
 /* Old version of rl_update_prompt(). Used only by rl_profile_previous() and
  * rl_profile_next(): if any of these functions use rl_update_prompt() instead,
  * no prompt is printed (not sure why). FIX. */
@@ -284,6 +291,7 @@ rl_update_prompt_old(void)
 	free(input);
 	xargs.refresh_on_empty_line = b;
 }
+#endif /* __HAIKU__ || !_NO_PROFILES */
 
 /* Runs any command recognized by CliFM via a keybind. Example:
  * keybind_exec_cmd("sel *") */
@@ -298,7 +306,7 @@ keybind_exec_cmd(char *str)
 		clear_suggestion(CS_FREEBUF);
 #endif /* !_NO_SUGGESTIONS */
 
-	int exit_status = EXIT_FAILURE;
+	int exit_status = FUNC_FAILURE;
 	char **cmd = parse_input_str(str);
 	putchar('\n');
 
@@ -332,17 +340,28 @@ static int
 run_kb_cmd(char *cmd)
 {
 	if (!cmd || !*cmd)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
+#ifndef _NO_SUGGESTIONS
 	if (conf.colorize == 1 && wrong_cmd == 1)
 		recover_from_wrong_cmd();
+#endif /* _NO_SUGGESTIONS */
+
+	int exit_code_bk = exit_code;
 
 	keybind_exec_cmd(cmd);
 	rl_reset_line_state();
-	return EXIT_SUCCESS;
+
+	if (exit_code != exit_code_bk)
+		/* The exit code was changed by the executed command. Force the
+		 * input taking function (my_rl_getc) to update the value of
+		 * prompt_offset to correctly calculate the cursor position. */
+		prompt_offset = UNSET;
+
+	return FUNC_SUCCESS;
 }
 
 /* Retrieve the key sequence associated to FUNCTION */
@@ -368,7 +387,7 @@ rl_toggle_max_filename_len(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	static int mnl_bk = 0, flag = 0;
 
@@ -397,7 +416,7 @@ rl_toggle_max_filename_len(int count, int key)
 		print_reload_msg(_("Max name length set to %d\n"), conf.max_name_len);
 
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Prepend authentication program name (usually sudo or doas) to the current
@@ -421,7 +440,7 @@ rl_prepend_sudo(int count, int key)
 	if (t) {
 		len = strlen(t);
 		if (len > 0 && t[len - 1] != ' ') {
-			s = (char *)xnmalloc(len + 2, sizeof(char));
+			s = xnmalloc(len + 2, sizeof(char));
 			snprintf(s, len + 2, "%s ", t);
 			len++;
 		} else {
@@ -430,7 +449,7 @@ rl_prepend_sudo(int count, int key)
 		}
 	} else {
 		len = strlen(DEF_SUDO_CMD) + 1;
-		s = (char *)xnmalloc(len + 1, sizeof(char));
+		s = xnmalloc(len + 1, sizeof(char));
 		snprintf(s, len + 1, "%s ", DEF_SUDO_CMD);
 	}
 
@@ -478,7 +497,7 @@ rl_prepend_sudo(int count, int key)
 	}
 #endif /* !_NO_HIGHLIGHT */
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -554,7 +573,7 @@ my_insert_text(char *text, char *s, const char _s)
 			}
 		}
 
-		char q[PATH_MAX];
+		char q[PATH_MAX + 1];
 		int l = 0;
 		for (i = 0; t[i]; i++) {
 			rl_highlight(t, i, SET_COLOR);
@@ -610,7 +629,7 @@ rl_accept_suggestion(int count, int key)
 		 * forward one column */
 		if (rl_point < rl_end)
 			rl_point++;
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (!wrong_cmd && cur_color != hq_c) {
@@ -627,7 +646,7 @@ rl_accept_suggestion(int count, int key)
 			int mlen = mblen(rl_line_buffer + rl_point, MB_CUR_MAX);
 			rl_point += mlen;
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	/* If accepting the first suggested word, accept only up to next
@@ -818,7 +837,7 @@ rl_accept_suggestion(int count, int key)
 	}
 
 	flags &= ~BAEJ_SUGGESTION;
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -855,7 +874,7 @@ rl_dir_parent(int count, int key)
 	UNUSED(count); UNUSED(key);
 	/* If already root dir, do nothing */
 	if (*workspaces[cur_ws].path == '/' && !workspaces[cur_ws].path[1])
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	exec_prompt_cmds = 1;
 	return run_kb_cmd("cd ..");
@@ -867,7 +886,7 @@ rl_dir_root(int count, int key)
 	UNUSED(count); UNUSED(key);
 	/* If already root dir, do nothing */
 	if (*workspaces[cur_ws].path == '/' && !workspaces[cur_ws].path[1])
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	return run_kb_cmd("cd /");
 }
@@ -879,7 +898,7 @@ rl_dir_home(int count, int key)
 	/* If already in home, do nothing */
 	if (*workspaces[cur_ws].path == *user.home
 	&& strcmp(workspaces[cur_ws].path, user.home) == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	exec_prompt_cmds = 1;
 	return run_kb_cmd("cd");
@@ -891,7 +910,7 @@ rl_dir_previous(int count, int key)
 	UNUSED(count); UNUSED(key);
 	/* If already at the beginning of dir hist, do nothing */
 	if (dirhist_cur_index == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	exec_prompt_cmds = 1;
 	return run_kb_cmd("b");
@@ -904,7 +923,7 @@ rl_dir_next(int count, int key)
 
 	/* If already at the end of dir hist, do nothing */
 	if (dirhist_cur_index + 1 == dirhist_total_index)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	exec_prompt_cmds = 1;
 	return run_kb_cmd("f");
@@ -915,7 +934,7 @@ rl_toggle_long_view(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1 || xargs.disk_usage_analyzer == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -935,7 +954,35 @@ rl_toggle_long_view(int count, int key)
 	print_reload_msg(_("Long view %s\n"),
 		conf.long_view == 1 ? _("enabled") : _("disabled"));
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
+}
+
+int
+rl_toggle_follow_link_long(int count, int key)
+{
+	UNUSED(count); UNUSED(key);
+	if (kbind_busy == 1 || conf.long_view == 0 || conf.light_mode == 1)
+		return FUNC_SUCCESS;
+
+#ifndef _NO_SUGGESTIONS
+	if (suggestion.printed && suggestion_buf)
+		free_suggestion();
+#endif /* !_NO_SUGGESTIONS */
+
+	conf.follow_symlinks_long = conf.follow_symlinks_long == 1 ? 0 : 1;
+
+	if (conf.autols == 1) {
+		if (conf.clear_screen == 0)
+		/* Without this putchar(), the first entries of the directories
+		 * list are printed in the prompt line. */
+			putchar('\n');
+		reload_dirlist();
+	}
+
+	print_reload_msg(_("Follow links %s\n"),
+		conf.follow_symlinks_long == 1 ? _("enabled") : _("disabled"));
+	xrl_reset_line_state();
+	return FUNC_SUCCESS;
 }
 
 int
@@ -943,7 +990,7 @@ rl_toggle_dirs_first(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -961,7 +1008,7 @@ rl_toggle_dirs_first(int count, int key)
 	print_reload_msg(_("Directories first %s\n"),
 		conf.list_dirs_first ? _("enabled") : _("disabled"));
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
@@ -969,7 +1016,7 @@ rl_toggle_light_mode(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -994,7 +1041,7 @@ rl_toggle_light_mode(int count, int key)
 	if (rl_dispatching == 1)
 		rl_update_prompt();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
@@ -1002,7 +1049,7 @@ rl_toggle_hidden_files(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -1017,10 +1064,13 @@ rl_toggle_hidden_files(int count, int key)
 		reload_dirlist();
 	}
 
-	print_reload_msg(_("Hidden files %s\n"),
-		conf.show_hidden == 1 ? "enabled" : "disabled");
+	if (conf.show_hidden == 1)
+		print_reload_msg(_("Showing dotfiles\n"));
+	else
+		print_reload_msg(_("Hiding dotfiles\n"));
+
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1063,9 +1113,9 @@ rl_open_preview(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (!config_dir || kbind_busy == 1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	char *file = (char *)xnmalloc(config_dir_len + 15, sizeof(char));
+	char *file = xnmalloc(config_dir_len + 15, sizeof(char));
 	snprintf(file, config_dir_len + 15, "%s/preview.clifm", config_dir);
 
 	int ret = open_file(file);
@@ -1123,7 +1173,7 @@ rl_clear_line(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1 && xrename == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	words_num = 0;
 
@@ -1141,7 +1191,7 @@ rl_clear_line(int count, int key)
 	}
 	if (suggestion.nlines > term_lines) {
 		rl_on_new_line();
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (suggestion_buf) {
@@ -1153,7 +1203,7 @@ rl_clear_line(int count, int key)
 	curhistindex = current_hist_n;
 	rl_kill_text(0, rl_end);
 	rl_point = rl_end = 0;
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1161,12 +1211,22 @@ rl_sort_next(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
 		free_suggestion();
 #endif /* !_NO_SUGGESTIONS */
+
+#ifndef ST_BTIME
+	if (conf.sort + 1 == SBTIME)
+		conf.sort++;
+#endif /* !ST_BTIME */
+
+	if (conf.light_mode == 1)
+		while (!ST_IN_LIGHT_MODE(conf.sort + 1) && conf.sort + 1 <= SORT_TYPES)
+			conf.sort++;
+
 	conf.sort++;
 	if (conf.sort > SORT_TYPES)
 		conf.sort = 0;
@@ -1180,7 +1240,7 @@ rl_sort_next(int count, int key)
 	}
 
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1188,15 +1248,25 @@ rl_sort_previous(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
 		free_suggestion();
 #endif /* !_NO_SUGGESTIONS */
+
+#ifndef ST_BTIME
+	if (conf.sort - 1 == SBTIME)
+		conf.sort--;
+#endif /* !ST_BTIME */
+
+	if (conf.light_mode == 1)
+		while (!ST_IN_LIGHT_MODE(conf.sort - 1) && conf.sort - 1 >= 0)
+			conf.sort--;
+
 	conf.sort--;
 	if (conf.sort < 0)
-		conf.sort = SORT_TYPES;
+		conf.sort = conf.light_mode == 1 ? SINO : SORT_TYPES;
 
 	if (conf.autols == 1) {
 		sort_switch = 1;
@@ -1207,14 +1277,14 @@ rl_sort_previous(int count, int key)
 	}
 
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 rl_lock(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	int ret = EXIT_SUCCESS;
+	int ret = FUNC_SUCCESS;
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
 		free_suggestion();
@@ -1236,18 +1306,18 @@ rl_lock(int count, int key)
 	rl_prep_terminal(0);
 	xrl_reset_line_state();
 
-	if (ret != EXIT_SUCCESS)
-		return EXIT_FAILURE;
-	return EXIT_SUCCESS;
+	if (ret != FUNC_SUCCESS)
+		return FUNC_FAILURE;
+	return FUNC_SUCCESS;
 }
 
 static int
 handle_no_sel(const char *func)
 {
+#ifndef _NO_SUGGESTIONS
 	if (conf.colorize == 1 && wrong_cmd == 1)
 		recover_from_wrong_cmd();
 
-#ifndef _NO_SUGGESTIONS
 	if (suggestion.printed == 1 && suggestion_buf)
 		clear_suggestion(CS_FREEBUF);
 #endif /* !_NO_SUGGESTIONS */
@@ -1260,7 +1330,7 @@ handle_no_sel(const char *func)
 	printf(_("\n%s: sel: %s\n"), func, strerror(ENOENT));
 	rl_reset_line_state();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1268,7 +1338,7 @@ rl_archive_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("ac");
@@ -1281,7 +1351,7 @@ rl_remove_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("r");
@@ -1291,7 +1361,7 @@ rl_remove_sel(int count, int key)
 	keybind_exec_cmd(cmd);
 	rl_prep_terminal(0);
 	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1299,7 +1369,7 @@ rl_export_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("exp");
@@ -1312,7 +1382,7 @@ rl_move_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("m");
@@ -1326,7 +1396,7 @@ rl_rename_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("br");
@@ -1340,7 +1410,7 @@ rl_paste_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (sel_n == 0)
 		return handle_no_sel("c");
@@ -1351,7 +1421,7 @@ rl_paste_sel(int count, int key)
 	keybind_exec_cmd(cmd);
 	rl_prep_terminal(0);
 	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
@@ -1359,15 +1429,14 @@ rl_quit(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	conf.cd_on_quit = 0;
 	puts("\n");
 
-	/* Reset terminal attributes before exiting. Without this line, the program
-	 * quits, but terminal input is not printed to STDOUT. */
-	tcsetattr(STDIN_FILENO, TCSANOW, &shell_tmodes);
-	exit(EXIT_SUCCESS);
+	/* Reset terminal attributes before exiting. */
+	rl_deprep_terminal();
+	exit(FUNC_SUCCESS);
 }
 
 #ifndef _NO_PROFILES
@@ -1397,7 +1466,7 @@ rl_profile_previous(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -1409,7 +1478,7 @@ rl_profile_previous(int count, int key)
 
 	if (cur_prof == -1 || !profile_names[cur_prof]
 	|| total_profs <= 1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	prev_prof = cur_prof - 1;
 	total_profs--;
@@ -1423,10 +1492,10 @@ rl_profile_previous(int count, int key)
 		putchar('\n');
 	}
 
-	if (profile_set(profile_names[prev_prof]) == EXIT_SUCCESS)
-		rl_update_prompt_old();
+	profile_set(profile_names[prev_prof]);
+	rl_update_prompt_old();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1434,7 +1503,7 @@ rl_profile_next(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
@@ -1445,7 +1514,7 @@ rl_profile_next(int count, int key)
 	get_cur_prof(&cur_prof, &total_profs);
 
 	if (cur_prof == -1 || !profile_names[cur_prof])
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	next_prof = cur_prof + 1;
 	total_profs--;
@@ -1460,10 +1529,10 @@ rl_profile_next(int count, int key)
 		putchar('\n');
 	}
 
-	if (profile_set(profile_names[next_prof]) == EXIT_SUCCESS)
-		rl_update_prompt_old();
+	profile_set(profile_names[next_prof]);
+	rl_update_prompt_old();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 #endif /* _NO_PROFILES */
 
@@ -1511,9 +1580,9 @@ rl_open_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
-	char cmd[PATH_MAX + 3];
+	char cmd[PATH_MAX + 4];
 	snprintf(cmd, sizeof(cmd), "o %s", (sel_n && sel_elements[sel_n - 1].name)
 		? sel_elements[sel_n - 1].name : "sel");
 
@@ -1527,12 +1596,12 @@ run_man_cmd(char *str)
 	char *p = getenv("MANPAGER");
 	if (p) {
 		size_t len = strlen(p);
-		mp = (char *)xnmalloc(len + 1, sizeof(char *));
+		mp = xnmalloc(len + 1, sizeof(char *));
 		xstrsncpy(mp, p, len + 1);
 		unsetenv("MANPAGER");
 	}
 
-	int ret = launch_execl(str) != EXIT_SUCCESS;
+	int ret = launch_execl(str) != FUNC_SUCCESS;
 
 	if (mp) {
 		setenv("MANPAGER", mp, 1);
@@ -1546,7 +1615,7 @@ static int
 rl_kbinds_help(int count, int key)
 {
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	UNUSED(count); UNUSED(key);
 #ifndef _NO_SUGGESTIONS
@@ -1560,15 +1629,15 @@ rl_kbinds_help(int count, int key)
 		PROGRAM_NAME);
 	int ret = run_man_cmd(cmd);
 	if (!ret)
-		return EXIT_FAILURE;
-	return EXIT_SUCCESS;
+		return FUNC_FAILURE;
+	return FUNC_SUCCESS;
 }
 
 static int
 rl_cmds_help(int count, int key)
 {
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	UNUSED(count); UNUSED(key);
 #ifndef _NO_SUGGESTIONS
@@ -1582,15 +1651,15 @@ rl_cmds_help(int count, int key)
 		PROGRAM_NAME);
 	int ret = run_man_cmd(cmd);
 	if (!ret)
-		return EXIT_FAILURE;
-	return EXIT_SUCCESS;
+		return FUNC_FAILURE;
+	return FUNC_SUCCESS;
 }
 
 static int
 rl_manpage(int count, int key)
 {
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	UNUSED(count); UNUSED(key);
 #ifndef _NO_SUGGESTIONS
@@ -1598,9 +1667,9 @@ rl_manpage(int count, int key)
 		free_suggestion();
 #endif /* !_NO_SUGGESTIONS */
 	char *cmd[] = {"man", PROGRAM_NAME, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		return EXIT_FAILURE;
-	return EXIT_SUCCESS;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		return FUNC_FAILURE;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1610,7 +1679,7 @@ rl_dir_pinned(int count, int key)
 	if (!pinned_dir) {
 		printf(_("\n%s: No pinned file\n"), PROGRAM_NAME);
 		rl_reset_line_state();
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	return run_kb_cmd(",");
@@ -1632,7 +1701,7 @@ rl_ws1(int count, int key)
 			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
 			return run_kb_cmd(t);
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	return run_kb_cmd("ws 1");
@@ -1652,7 +1721,7 @@ rl_ws2(int count, int key)
 			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
 			return run_kb_cmd(t);
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	return run_kb_cmd("ws 2");
@@ -1672,7 +1741,7 @@ rl_ws3(int count, int key)
 			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
 			return run_kb_cmd(t);
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	return run_kb_cmd("ws 3");
@@ -1692,7 +1761,7 @@ rl_ws4(int count, int key)
 			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
 			return run_kb_cmd(t);
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	return run_kb_cmd("ws 4");
@@ -1755,7 +1824,7 @@ rl_toggle_only_dirs(int count, int key)
 	UNUSED(count); UNUSED(key);
 
 	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	conf.only_dirs = conf.only_dirs ? 0 : 1;
 
@@ -1786,7 +1855,7 @@ print_highlight_string(char *s, const int insert_point)
 	fputs(tx_c, stdout);
 	cur_color = tx_c;
 
-	char q[PATH_MAX];
+	char q[PATH_MAX + 1];
 	for (i = 0; s[i]; i++) {
 		rl_highlight(s, i, SET_COLOR);
 
@@ -1839,7 +1908,7 @@ print_cmdhist_line(int n, int beg_line)
 	cur_color = df_c;
 	fputs(df_c, stdout);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static inline int
@@ -1850,19 +1919,19 @@ handle_cmdhist_beginning(int key)
 
 	if (key == 65) { /* Up arrow key */
 		if (--p < 0)
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 	} else { /* Down arrow key */
 		if (rl_end == 0)
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		if (++p >= (int)current_hist_n) {
 			rl_replace_line("", 1);
 			curhistindex++;
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		}
 	}
 
 	if (!history[p].cmd)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	curhistindex = (size_t)p;
 
@@ -1875,7 +1944,7 @@ handle_cmdhist_middle(int key)
 	int found = 0, p = (int)curhistindex;
 
 	if (key == 65) { /* Up arrow key */
-		if (--p < 0) return EXIT_FAILURE;
+		if (--p < 0) return FUNC_FAILURE;
 
 		while (p >= 0 && history[p].cmd) {
 			if (strncmp(rl_line_buffer, history[p].cmd, (size_t)rl_point) == 0
@@ -1885,7 +1954,7 @@ handle_cmdhist_middle(int key)
 			p--;
 		}
 	} else { /* Down arrow key */
-		if (++p >= (int)current_hist_n)	return EXIT_FAILURE;
+		if (++p >= (int)current_hist_n)	return FUNC_FAILURE;
 
 		while (history[p].cmd) {
 			if (strncmp(rl_line_buffer, history[p].cmd, (size_t)rl_point) == 0
@@ -1898,7 +1967,7 @@ handle_cmdhist_middle(int key)
 
 	if (found == 0) {
 		rl_ring_bell();
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	return print_cmdhist_line(p, 0);
@@ -1910,7 +1979,7 @@ rl_cmdhist(int count, int key)
 {
 	UNUSED(count);
 	if (rl_nohist == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 #ifndef _NO_SUGGESTIONS
 	if (suggestion_buf) {
@@ -1925,7 +1994,7 @@ rl_cmdhist(int count, int key)
 		key = 66;  /* Down */
 
 	if (key != 65 && key != 66)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* If the cursor is at the beginning of the line */
 	if (rl_point == 0 || cmdhist_flag == 1)
@@ -1984,7 +2053,7 @@ rl_tab_comp(int count, int key)
 	UNUSED(count); UNUSED(key);
 
 	tab_complete('!');
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1997,7 +2066,7 @@ rl_del_last_word(int count, int key)
 	UNUSED(count); UNUSED(key);
 
 	if (rl_point == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	char *end_buf = (char *)NULL;
 	if (rl_point < rl_end) { /* Somewhere before the end of the line */
@@ -2034,7 +2103,7 @@ rl_del_last_word(int count, int key)
 		recover_from_wrong_cmd();
 #endif /* !_NO_SUGGESTIONS */
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -2042,7 +2111,7 @@ rl_toggle_virtualdir_full_paths(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (!stdin_tmp_dir || strcmp(stdin_tmp_dir, workspaces[cur_ws].path) != 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	xchmod(stdin_tmp_dir, "0700", 1);
 	xargs.virtual_dir_full_paths = xargs.virtual_dir_full_paths == 1 ? 0 : 1;
@@ -2079,7 +2148,17 @@ rl_toggle_virtualdir_full_paths(int count, int key)
 	print_reload_msg(_("Switched to %s names\n"),
 		xargs.virtual_dir_full_paths == 1 ? _("long") : _("short"));
 	xrl_reset_line_state();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
+}
+
+static int
+rl_run_pager(int count, int key)
+{
+	UNUSED(count); UNUSED(key);
+	if (kbind_busy == 1 || conf.pager == 1)
+		return FUNC_SUCCESS;
+
+	return run_kb_cmd("pg");
 }
 
 /* Used to disable keybindings. */
@@ -2087,7 +2166,7 @@ static int
 do_nothing(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static void
@@ -2166,11 +2245,13 @@ set_keybinds_from_file(void)
 	rl_bind_keyseq(find_key("toggle-hidden"), rl_toggle_hidden_files);
 	rl_bind_keyseq(find_key("toggle-hidden2"), rl_toggle_hidden_files);
 	rl_bind_keyseq(find_key("toggle-long"), rl_toggle_long_view);
+	rl_bind_keyseq(find_key("toggle-follow-links-long"), rl_toggle_follow_link_long);
 	rl_bind_keyseq(find_key("toggle-light"), rl_toggle_light_mode);
 	rl_bind_keyseq(find_key("dirs-first"), rl_toggle_dirs_first);
 	rl_bind_keyseq(find_key("sort-previous"), rl_sort_previous);
 	rl_bind_keyseq(find_key("sort-next"), rl_sort_next);
 	rl_bind_keyseq(find_key("only-dirs"), rl_toggle_only_dirs);
+	rl_bind_keyseq(find_key("run-pager"), rl_run_pager);
 
 	/* Misc */
 	rl_bind_keyseq(find_key("launch-view"), rl_launch_view);
@@ -2259,11 +2340,13 @@ set_default_keybinds(void)
 	rl_bind_keyseq("\\M-i", rl_toggle_hidden_files);
 	rl_bind_keyseq("\\M-.", rl_toggle_hidden_files);
 	rl_bind_keyseq("\\M-l", rl_toggle_long_view);
+	rl_bind_keyseq("\\M-+", rl_toggle_follow_link_long);
 	rl_bind_keyseq("\\M-y", rl_toggle_light_mode);
 	rl_bind_keyseq("\\M-g", rl_toggle_dirs_first);
 	rl_bind_keyseq("\\M-z", rl_sort_previous);
 	rl_bind_keyseq("\\M-x", rl_sort_next);
 	rl_bind_keyseq("\\M-,", rl_toggle_only_dirs);
+	rl_bind_keyseq("\\M-0", rl_run_pager);
 
 	rl_bind_keyseq("\\M--", rl_launch_view);
 	rl_bind_keyseq("\\C-x", rl_new_instance);
@@ -2280,6 +2363,8 @@ set_default_keybinds(void)
 static void
 set_hardcoded_keybinds(void)
 {
+	rl_bind_keyseq("\\M-*", do_nothing);
+
 #ifndef __HAIKU__
 	rl_bind_keyseq("\\C-l", rl_refresh);
 	rl_bind_keyseq("\\C-p", rl_cmdhist);
