@@ -8,7 +8,7 @@
 			 *  # 	  The command line file manager    #
 			 *  ######################################## */
 
-/* Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+/* Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * This program is free software; you can redistribute it and/or modify
@@ -95,6 +95,8 @@ struct ext_t *ext_colors = (struct ext_t *)NULL;
 #ifdef LINUX_FSINFO
 struct ext_mnt_t *ext_mnt = (struct ext_mnt_t *)NULL;
 #endif /* LINUX_FSINFO */
+struct groups_t *sys_users = (struct groups_t *)NULL;
+struct groups_t *sys_groups = (struct groups_t *)NULL;
 
 const struct sort_t sort_methods[] = {
     {"none", 0, 0},
@@ -109,7 +111,9 @@ const struct sort_t sort_methods[] = {
     {"inode", 9, 0},
     {"owner", 10, 0},
     {"group", 11, 0},
-    {NULL, 12, 0},
+    {"blocks", 12, 0},
+    {"links", 13, 0},
+    {NULL, 14, 0},
 };
 
 /* pmsg holds the current program message type */
@@ -126,9 +130,9 @@ int
 	bin_flags = 0,
 	search_flags = 0;
 
-struct termios
-	orig_termios,
-	shell_tmodes;
+int
+	date_shades_old_style = 0,
+	size_shades_old_style = 0;
 
 pid_t own_pid = 0;
 time_t props_now = 0;
@@ -157,7 +161,6 @@ int
 	curcol = 0,
 	dequoted = 0,
 	dir_changed = 0,
-	dir_out = 0,
 	dirhist_cur_index = 0,
 	dirhist_total_index = 0,
 	exit_code = 0,
@@ -170,6 +173,7 @@ int
 	int_vars = UNSET,
 	internal_cmd = 0,
 	is_sel = 0,
+	is_cdpath = 0,
 	jump_total_rank = 0,
 	kbind_busy = 0,
 	list_dirs_first = UNSET,
@@ -185,6 +189,7 @@ int
 	prompt_offset = UNSET,
 	prompt_notif = UNSET,
 	recur_perm_error_flag = 0,
+
 	rl_nohist = 0,
 	rl_notab = 0,
 	sel_is_last = 0,
@@ -197,6 +202,7 @@ int
 #ifndef _NO_TRASH
 	trash_ok = 1,
 #endif /* !_NO_TRASH */
+	virtual_dir = 0,
 	wrong_cmd = 0,
 	xrename = 0;
 
@@ -242,10 +248,6 @@ char
 	cur_prompt_name[NAME_MAX + 1] = "",
 	div_line[NAME_MAX + 1],
 	hostname[HOST_NAME_MAX + 1],
-#ifndef _NO_FZF
-	finder_in_file[PATH_MAX + 1],
-	finder_out_file[PATH_MAX + 1],
-#endif /* _NO_FZF */
 	fz_match[PATH_MAX + 1],
 	prop_fields_str[PROP_FIELDS_SIZE + 1] = "",
 	invalid_time_str[MAX_TIME_STR] = "",
@@ -257,6 +259,7 @@ char
 	*actions_file = (char *)NULL,
 	*alt_bm_file = (char *)NULL,
 	*alt_config_dir = (char *)NULL,
+	*alt_trash_dir = (char *)NULL,
 	*alt_config_file = (char *)NULL,
 	*alt_kbinds_file = (char *)NULL,
 	*alt_preview_file = (char *)NULL,
@@ -290,7 +293,6 @@ char
 /*	*right_prompt = (char *)NULL, */
 	*sel_file = (char *)NULL,
 	*smenutab_options_env = (char *)NULL,
-	*stat_filename = (char *)NULL,
 	*stdin_tmp_dir = (char *)NULL,
 	*sudo_cmd = (char *)NULL,
 #ifndef _NO_SUGGESTIONS
@@ -320,7 +322,7 @@ char
 char
 	/* File types */
 	bd_c[MAX_COLOR], /* Block device */
-	bk_c[MAX_COLOR], /* Backup files (ending with tilde) */
+	bk_c[MAX_COLOR], /* Backup/temp files */
 	ca_c[MAX_COLOR], /* Cap file */
 	cd_c[MAX_COLOR], /* Char device */
 	di_c[MAX_COLOR], /* Directory */
@@ -349,10 +351,10 @@ char
 
 	/* Interface */
 	bm_c[MAX_COLOR], /* Bookmarked directory */
-	fc_c[MAX_COLOR], /* Files counter */
 	df_c[MAX_COLOR], /* Default color */
 	dl_c[MAX_COLOR], /* Dividing line index */
 	el_c[MAX_COLOR], /* ELN */
+	fc_c[MAX_COLOR], /* Files counter */
 	lc_c[MAX_COLOR], /* Symlink character (ColorLinkAsTarget only) */
 	mi_c[MAX_COLOR], /* Misc indicators */
 	ts_c[MAX_COLOR], /* TAB completion suffix */
@@ -387,16 +389,19 @@ char
 	hv_c[MAX_COLOR], /* Variables: $ */
 	hw_c[MAX_COLOR], /* Backslash (aka whack) */
 
+	db_c[MAX_COLOR],  /* File allocated blocks */
+	dd_c[MAX_COLOR],  /* Date (fixed color: no shading) */
+	de_c[MAX_COLOR],  /* Inode number */
+	dg_c[MAX_COLOR],  /* UID, GID */
+	dk_c[MAX_COLOR],  /* Number of links */
+	dn_c[MAX_COLOR],  /* dash (none) */
+	do_c[MAX_COLOR],  /* Octal perms */
+	dp_c[MAX_COLOR],  /* Special files (SUID, SGID, etc) */
 	dr_c[MAX_COLOR],  /* Read */
 	dw_c[MAX_COLOR],  /* Write */
 	dxd_c[MAX_COLOR], /* Execute (dirs) */
 	dxr_c[MAX_COLOR], /* Execute (reg files) */
-	dg_c[MAX_COLOR],  /* UID, GID */
-	dd_c[MAX_COLOR],  /* Date */
-	dz_c[MAX_COLOR],  /* Size (dirs) > */
-	do_c[MAX_COLOR],  /* Octal representation > */
-	dp_c[MAX_COLOR],  /* Special files (SUID, SGID, etc) */
-	dn_c[MAX_COLOR],  /* Dash (none) */
+	dz_c[MAX_COLOR],  /* Size (dirs) */
 
 	/* Colors used in the prompt, so that \001 and \002 needs to
 	 * be added. This is why MAX_COLOR + 2 */
@@ -421,7 +426,12 @@ char
 	tx_c[MAX_COLOR + 2], /* Text color */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
-	tmp_color[MAX_COLOR + 2]; /* A temp buffer to store color codes */
+
+	tmp_color[MAX_COLOR + 2], /* A temp buffer to store color codes */
+	dim_c[] = "\x1b[2m";
+
+/* A buffer to store file names to be displayed (wide string) */
+char name_buf[(NAME_MAX + 1) * sizeof(wchar_t)];
 
 /* A list of all internal commands, with short and long formats.
  * We use two more lists of commands: one of commands dealing with file names
@@ -491,6 +501,8 @@ const struct cmdslist_t internal_cmds[] = {
 	{"jc", 2},
 	{"jl", 2},
 	{"jp", 2},
+	{"k", 1},
+	{"kk", 2},
 //	{"jo", 2},
 	{"kb", 2},
 	{"keybinds", 8},
@@ -731,6 +743,8 @@ const struct cmdslist_t param_str[] = {
 	{"pager on", 8},
 	{"pg off", 6},
 	{"pager off", 9},
+	{"pg once", 7},
+	{"pager once", 10},
 	{"pg status", 9},
 	{"pager status", 12},
 	{"pf set", 6},
@@ -753,6 +767,8 @@ const struct cmdslist_t param_str[] = {
 	{"st none", 7},
 	{"st name", 7},
 	{"st size", 7},
+	{"st blocks", 9},
+	{"st links", 8},
 	{"st atime", 8},
 	{"st btime", 8},
 	{"st ctime", 8},
@@ -764,7 +780,9 @@ const struct cmdslist_t param_str[] = {
 	{"st version", 10},
 	{"sort none", 9},
 	{"sort name", 9},
+	{"sort blocks", 11},
 	{"sort size", 9},
+	{"sort links", 10},
 	{"sort atime", 10},
 	{"sort btime", 10},
 	{"sort ctime", 10},
@@ -826,7 +844,7 @@ time_t curdir_mtime = 0;
 #endif /* LINUX_INOTIFY */
 
 #ifdef RUN_CMD
-// Run the command passed via --cmd and exit
+/* Run the command passed via --cmd and exit */
 static void
 run_and_exit(void)
 {
@@ -937,6 +955,9 @@ list_files(void)
 #endif /* RUN_CMD */
 
 	if (conf.autols == 1 && isatty(STDIN_FILENO)) {
+		if (xargs.list_and_quit == 1)
+			goto LIST;
+
 #ifdef LINUX_INOTIFY
 		/* Initialize inotify */
 		inotify_fd = inotify_init1(IN_NONBLOCK);
@@ -952,6 +973,7 @@ list_files(void)
 		}
 #endif /* LINUX_INOTIFY */
 
+LIST:
 		if (conf.colorize == 1 && xargs.eln_use_workspace_color == 1)
 			set_eln_color();
 
@@ -1012,9 +1034,11 @@ static inline void
 init_trash(void)
 {
 	if (trash_ok) {
-		trash_n = (size_t)count_dir(trash_files_dir, NO_CPOP);
-		if (trash_n <= 2)
+		filesn_t ret = count_dir(trash_files_dir, NO_CPOP);
+		if (ret <= 2)
 			trash_n = 0;
+		else
+			trash_n = (size_t)ret;
 	}
 }
 #endif /* _NO_TRASH */
@@ -1095,6 +1119,44 @@ set_security_paranoid_mode(void)
 }
 #endif /* SECURITY_PARANOID */
 
+#ifdef HAVE_PLEDGE
+static void
+set_pledge(void)
+{
+	if (pledge("stdio rpath wpath cpath dpath tmppath fattr "
+	"chown flock getpw tty proc exec", NULL) == -1) {
+		fprintf(stderr, "%s: pledge: %s\n", PROGRAM_NAME, strerror(errno));
+		exit(errno);
+	}
+}
+#endif /* HAVE_PLEDGE */
+
+__attribute__ ((noreturn))
+static void
+list_files_and_quit(void)
+{
+	if (conf.light_mode == 1) {
+		if (conf.long_view == 1 && prop_fields.time == PROP_TIME_BIRTH) {
+			fprintf(stderr, _("%s: PropFields: 'b': Birth time is not "
+				"allowed in light mode\n"), PROGRAM_NAME);
+			exit(EXIT_FAILURE);
+		}
+		const int s = conf.sort;
+		if (s != SNONE && s != SNAME && s != SEXT && s != SVER && s != SINO) {
+			fprintf(stderr, _("%s: Invalid sort value: only none, "
+				"name, extension, version, and inode are allowed in "
+				"light mode\n"), PROGRAM_NAME);
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (xargs.full_dir_size == 1)
+		tmp_dir = savestring(P_tmpdir, P_tmpdir_len);
+
+	list_files();
+	exit(EXIT_SUCCESS); /* Never reached. */
+}
+
 /*
 static void
 init_file_flags(void)
@@ -1116,6 +1178,10 @@ init_file_flags(void)
 int
 main(int argc, char *argv[])
 {
+#ifdef HAVE_PLEDGE
+	set_pledge();
+#endif /* HAVE_PLEDGE */
+
 	/* Quite unlikely to happen, but one never knows. See
 	 * https://lwn.net/SubscriberLink/882799/cb8f313c57c6d8a6/
 	 * and
@@ -1125,6 +1191,9 @@ main(int argc, char *argv[])
 		exit(EINVAL);
 	}
 
+	/* Make sure all initialization is made with restrictive permissions. */
+	const mode_t old_mask = umask(0077);
+
 	check_term(); /* Let's check terminal capabilities. */
 
 	/* # 1. INITIALIZE EVERYTHING WE NEED # */
@@ -1173,7 +1242,7 @@ main(int argc, char *argv[])
 	 * used later by get_path_programs (for the autocomplete function)
 	 * and get_cmd_path(). */
 	if (!(flags & PATH_PROGRAMS_ALREADY_LOADED))
-		path_n = get_path_env();
+		path_n = get_path_env(1);
 	cdpath_n = get_cdpath();
 
 	check_env_filter();
@@ -1189,10 +1258,8 @@ main(int argc, char *argv[])
 	if (xargs.stat > 0) /* Running with --stat(-full). Print and exit. */
 		do_stat_and_exit(xargs.stat == FULL_STAT ? 1 : 0);
 
-	if (xargs.list_and_quit == 1) {
-		list_files();
-		exit(EXIT_SUCCESS); /* Never reached */
-	}
+	if (xargs.list_and_quit == 1)
+		list_files_and_quit();
 
 	set_sel_file();
 	create_tmp_files();
@@ -1247,6 +1314,7 @@ main(int argc, char *argv[])
 	if (!jump_db || xargs.path == 1)
 		add_to_jumpdb(workspaces[cur_ws].path);
 
+	init_shell();
 	initialize_readline();
 	get_prompt_cmds();
 
@@ -1255,7 +1323,6 @@ main(int argc, char *argv[])
 #endif /* !_NO_TRASH */
 
 	get_hostname();
-	init_shell();
 	set_env(0);
 
 	if (config_ok == 1)
@@ -1271,6 +1338,9 @@ main(int argc, char *argv[])
 	load_pinned_dir();
 	init_workspaces_opts();
 
+	/* Restore user umask */
+	umask(old_mask);
+
 	/* # 2. MAIN PROGRAM LOOP # */
 	run_main_loop();
 }
