@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -54,17 +54,16 @@
 #endif /* __linux__ || __CYGWIN__ */
 
 #include "aux.h"
-#include "readline.h"
-#include "navigation.h"
-#include "exec.h"
-#include "listing.h"
-#include "jump.h"
-#include "misc.h"
-#include "history.h"
-
 #ifdef __sun
 # include "checks.h" /* check_file_access() */
 #endif /* __sun */
+#include "history.h"
+#include "jump.h"
+#include "listing.h"
+#include "misc.h"
+#include "navigation.h"
+#include "readline.h"
+#include "spawn.h"
 
 /* Information about devices */
 struct mnt_t {
@@ -90,7 +89,7 @@ get_block_devices(void)
 
 	for (i = 0; (int)i < block_n; i++) {
 # ifndef _DIRENT_HAVE_D_TYPE
-		char bpath[PATH_MAX];
+		char bpath[PATH_MAX + 1];
 		snprintf(bpath, sizeof(bpath), "/dev/%s", blockdev[i]->d_name);
 		struct stat a;
 		if (stat(bpath, &a) == -1) {
@@ -115,14 +114,14 @@ get_block_devices(void)
 		}
 
 		/* Get only partition names, normally ending with a number */
-		size_t blen = strlen(name);
+		const size_t blen = strlen(name);
 		if (blen > 0 && (name[blen - 1] < '1' || name[blen - 1] > '9')) {
 			free(blockdev[i]);
 			continue;
 		}
 
-		bd = (char **)xrealloc(bd, (n + 2) * sizeof(char *));
-		bd[n] = (char *)xnmalloc(blen + 6, sizeof(char *));
+		bd = xnrealloc(bd, n + 2, sizeof(char *));
+		bd[n] = xnmalloc(blen + 6, sizeof(char *));
 		snprintf(bd[n], blen + 6, "/dev/%s", name);
 		n++;
 		bd[n] = (char *)NULL;
@@ -139,32 +138,32 @@ unmount_dev(const size_t i, const int n)
 	if (xargs.mount_cmd == UNSET) {
 		xerror(_("%s: No mount application found. Install either "
 			"udevil or udisks2\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if ((unsigned int)n + (unsigned int)1 < (unsigned int)1 || n + 1 > (int)i) {
 		xerror(_("%s: %d: Invalid ELN\n"), PROGRAM_NAME, n + 1);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	char *mnt = media[n].mnt;
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	/* Get out of mountpoint before unmounting */
-	size_t mlen = strlen(mnt);
+	const size_t mlen = strlen(mnt);
 	if (strncmp(mnt, workspaces[cur_ws].path, mlen) == 0) {
 		char *cmd[] = {"b", NULL};
-		if (back_function(cmd) == EXIT_FAILURE)
+		if (back_function(cmd) == FUNC_FAILURE)
 			cd_function(NULL, CD_PRINT_ERROR);
 		exit_status = (-1);
 	}
 
 	char *cmd[] = {xargs.mount_cmd == MNT_UDISKS2 ? "udisksctl" : "udevil",
 			"unmount", "-b", media[n].dev, NULL};
-	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+		exit_status = FUNC_FAILURE;
 
-	if (exit_status != EXIT_FAILURE && xargs.mount_cmd == MNT_UDEVIL)
+	if (exit_status != FUNC_FAILURE && xargs.mount_cmd == MNT_UDEVIL)
 		printf(_("%s: Unmounted %s\n"), PROGRAM_NAME, media[n].dev);
 
 	return exit_status;
@@ -173,9 +172,9 @@ unmount_dev(const size_t i, const int n)
 static char *
 get_dev_label(void)
 {
-	size_t n = mp_n;
+	const size_t n = mp_n;
 	struct dirent **labels = (struct dirent **)NULL;
-	int ln = scandir(DISK_LABELS_PATH, &labels, NULL, alphasort);
+	const int ln = scandir(DISK_LABELS_PATH, &labels, NULL, alphasort);
 	if (ln == - 1)
 		return (char *)NULL;
 
@@ -188,8 +187,8 @@ get_dev_label(void)
 			continue;
 		}
 
-		char *name = labels[i]->d_name;
-		char lpath[PATH_MAX];
+		const char *name = labels[i]->d_name;
+		char lpath[PATH_MAX + 1];
 		snprintf(lpath, sizeof(lpath), "%s/%s", DISK_LABELS_PATH, name);
 		char *rpath = realpath(lpath, NULL);
 		if (!rpath) {
@@ -197,7 +196,7 @@ get_dev_label(void)
 			continue;
 		}
 
-		int ret = strcmp(rpath, media[n].dev);
+		const int ret = strcmp(rpath, media[n].dev);
 		free(rpath);
 		if (ret == 0) {
 			/* Device label is encoded using hex. Let's decode it */
@@ -222,7 +221,7 @@ static void
 list_unmounted_devs(void)
 {
 	int i;
-	size_t mp_n_bk = mp_n;
+	const size_t mp_n_bk = mp_n;
 	char **unm_devs = get_block_devices();
 	if (!unm_devs)
 		return;
@@ -243,8 +242,7 @@ list_unmounted_devs(void)
 			continue;
 		}
 
-		media = (struct mnt_t *)xrealloc(media,
-			    (mp_n + 2) * sizeof(struct mnt_t));
+		media = xnrealloc(media, mp_n + 2, sizeof(struct mnt_t));
 		media[mp_n].dev = savestring(unm_devs[i], strlen(unm_devs[i]));
 		media[mp_n].mnt = (char *)NULL;
 
@@ -274,7 +272,7 @@ list_mounted_devs(const int mode)
 	FILE *fp = setmntent(_PATH_MOUNTED, "r");
 	if (!fp) {
 		xerror("mp: setmntent: %s: %s\n", _PATH_MOUNTED, strerror(errno));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	while ((ent = getmntent(fp)) != NULL) {
@@ -292,7 +290,7 @@ list_mounted_devs(const int mode)
 				: nd_c, ent->mnt_dir, df_c);
 		}
 
-		media = (struct mnt_t *)xrealloc(media, (mp_n + 2) * sizeof(struct mnt_t));
+		media = xnrealloc(media, mp_n + 2, sizeof(struct mnt_t));
 		media[mp_n].mnt = savestring(ent->mnt_dir, strlen(ent->mnt_dir));
 		media[mp_n].dev = savestring(ent->mnt_fsname, strlen(ent->mnt_fsname));
 		media[mp_n].label = (char *)NULL;
@@ -306,7 +304,7 @@ list_mounted_devs(const int mode)
 	media[mp_n].mnt = (char *)NULL;
 	media[mp_n].label = (char *)NULL;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Mount device and store mountpoint (at media[N]). */
@@ -316,16 +314,16 @@ mount_dev(const int n)
 	if (xargs.mount_cmd == UNSET) {
 		xerror(_("%s: No mount application found. Install either "
 			"udevil or udisks2\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	char file[PATH_MAX];
+	char file[PATH_MAX + 1];
 	snprintf(file, sizeof(file), "%s/%s", xargs.stealth_mode == 1
 		? P_tmpdir : tmp_dir, TMP_FILENAME);
 
 	int fd = mkstemp(file);
 	if (fd == -1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	int stdout_bk = dup(STDOUT_FILENO); /* Save original stdout */
 	dup2(fd, STDOUT_FILENO); /* Redirect stdout to the desired file */
@@ -345,25 +343,25 @@ mount_dev(const int n)
 	FILE *fp = open_fread(file, &fd);
 	if (!fp) {
 		unlink(file);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	char out_line[PATH_MAX];
+	char out_line[PATH_MAX + 1]; *out_line = '\0';
 	if (fgets(out_line, (int)sizeof(out_line), fp) == NULL) {
 		/* Error is printed by the mount command itself */
+		unlinkat(fd, file, 0);
 		fclose(fp);
-		unlink(file);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
+	unlinkat(fd, file, 0);
 	fclose(fp);
-	unlink(file);
 
 	/* Recover the mountpoint used by the mounting command. */
 	char *p = strstr(out_line, " at ");
 	if (!p || !p[1] || !p[2] || !p[3] || p[4] != '/') {
 		xerror(_("%s: Error retrieving mountpoint\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 	p += 4;
 
@@ -376,7 +374,7 @@ mount_dev(const int n)
 
 	media[n].mnt = savestring(p, plen);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 #endif /* HAVE_PROC_MOUNTS */
 
@@ -398,17 +396,17 @@ static int
 print_dev_info(const int n)
 {
 	if (!media[n].dev || xargs.mount_cmd == UNSET)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	if (xargs.mount_cmd == MNT_UDEVIL) {
 		char *cmd[] = {"udevil", "info", media[n].dev, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	} else {
 		char *cmd[] = {"udisksctl", "info", "-b", media[n].dev, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 	}
 
 	return exit_status;
@@ -437,7 +435,7 @@ list_mountpoints_bsd(struct statfs *fslist)
 		    df_c, fslist[i].f_mntfromname);
 
 		/* Store the mountpoint into the mounpoints struct */
-		media = (struct mnt_t *)xrealloc(media, (j + 2) * sizeof(struct mnt_t));
+		media = xnrealloc(media, j + 2, sizeof(struct mnt_t));
 		media[j].mnt = savestring(fslist[i].f_mntonname,
 			strlen(fslist[i].f_mntonname));
 		media[j].label = (char *)NULL;
@@ -491,7 +489,7 @@ get_mnt_input(const int mode, int *info)
 			++p;
 		}
 
-		int atoi_num = atoi(p);
+		const int atoi_num = atoi(p);
 		if (atoi_num <= 0 || atoi_num > (int)mp_n) {
 			xerror("%s: %s: Invalid ELN\n", PROGRAM_NAME, input);
 			free(input);
@@ -509,13 +507,10 @@ get_mnt_input(const int mode, int *info)
 static int
 print_mnt_info(const int n)
 {
-	int exit_status = print_dev_info(n);
+	const int exit_status = print_dev_info(n);
 
-	if (exit_status == EXIT_SUCCESS) {
-		printf("\nPress any key to continue... ");
-		xgetchar();
-		putchar('\n');
-	}
+	if (exit_status == FUNC_SUCCESS)
+		press_any_key_to_continue(1);
 
 	free_media();
 	return exit_status;
@@ -543,11 +538,11 @@ xgetmntinfo_sun(void)
 		if (stat(mp, &a) == -1 || !S_ISDIR(a.st_mode))
 			continue;
 
-		int perm = check_file_access(a.st_mode, a.st_uid, a.st_gid);
+		const int perm = check_file_access(a.st_mode, a.st_uid, a.st_gid);
 		printf("%s%zu%s %s%s%s [%s]\n", el_c, n + 1, df_c,
 			perm == 1 ? di_c : nd_c, mp, df_c, ent.mnt_special);
 
-		media = (struct mnt_t *)xrealloc(media, (n + 2) * sizeof(struct mnt_t));
+		media = xnrealloc(media, n + 2, sizeof(struct mnt_t));
 		media[n].mnt = savestring(mp, strlen(mp));
 		media[n].label = (char *)NULL;
 		media[n].dev = (char *)NULL;
@@ -567,29 +562,32 @@ xgetmntinfo_sun(void)
 /* If MODE is MEDIA_MOUNT (used by the 'media' command) list mounted and
  * unmounted devices allowing the user to mount or unmount any of them.
  * If MODE is rather MEDIA_LIST (used by the 'mp' command), just list
- * available mountpoints and allow the user to cd into the selected one */
+ * available mountpoints and allow the user to cd into the selected one. */
 int
 media_menu(const int mode)
 {
 #if defined(__HAIKU__)
 	xerror(_("%s: This feature is not available on Haiku\n"),
 		mode == MEDIA_LIST ? _("mountpoints") : _("media"));
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 #endif /* __HAIKU__ */
 
 #ifndef HAVE_PROC_MOUNTS
 	if (mode == MEDIA_MOUNT) {
 		xerror("%s\n", _("media: Function only available on Linux systems"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 #endif /* HAVE_PROC_MOUNTS */
 
 	if (mode == MEDIA_MOUNT && xargs.mount_cmd == UNSET) {
 		xerror("%s\n", _("media: No mount application found. Install either "
 			"udevil or udisks2"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
+	if (conf.clear_screen == 1)
+		CLEAR;
+
 #ifdef HAVE_PROC_MOUNTS
 	printf("%s%s%s\n\n", BOLD, mode == MEDIA_LIST ? _("Mountpoints")
 			: _("Mounted devices"), df_c);
@@ -597,17 +595,17 @@ media_menu(const int mode)
 	printf(_("%sMountpoints%s\n\n"), BOLD, df_c);
 #endif /* HAVE_PROC_MOUNTS */
 
-	media = (struct mnt_t *)xnmalloc(1, sizeof(struct mnt_t));
+	media = xnmalloc(1, sizeof(struct mnt_t));
 	mp_n = 0;
 
 #ifdef HAVE_PROC_MOUNTS
-	if (list_mounted_devs(mode) == EXIT_FAILURE) {
+	if (list_mounted_devs(mode) == FUNC_FAILURE) {
 		free(media);
 		media = (struct mnt_t *)NULL;
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	size_t mp_n_bk = mp_n;
+	const size_t mp_n_bk = mp_n;
 
 	if (mode == MEDIA_MOUNT)
 		list_unmounted_devs();
@@ -633,7 +631,7 @@ media_menu(const int mode)
 #else
 		fputs(_("mp: There are no available mountpoints\n"), stdout);
 #endif /* HAVE_PROC_MOUNTS */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
@@ -643,9 +641,9 @@ media_menu(const int mode)
 
 	putchar('\n');
 	int info = 0;
-	int n = get_mnt_input(mode, &info);
+	const int n = get_mnt_input(mode, &info);
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	if (n == -1)
 		goto EXIT;
 
@@ -659,23 +657,23 @@ media_menu(const int mode)
 	if (mode == MEDIA_MOUNT) {
 		if (!media[n].mnt) {
 			/* The device is unmounted: mount it */
-			if (mount_dev(n) == EXIT_FAILURE) {
-				exit_status = EXIT_FAILURE;
+			if (mount_dev(n) == FUNC_FAILURE) {
+				exit_status = FUNC_FAILURE;
 				goto EXIT;
 			}
 		} else {
 			/* The device is mounted: unmount it */
 			int ret = unmount_dev(mp_n_bk, n);
-			if (ret == EXIT_FAILURE)
-				exit_status = EXIT_FAILURE;
+			if (ret == FUNC_FAILURE)
+				exit_status = FUNC_FAILURE;
 			goto EXIT;
 		}
 	}
 #endif /* HAVE_PROC_MOUNTS */
 
-	if (xchdir(media[n].mnt, SET_TITLE) != EXIT_SUCCESS) {
-		xerror("%s: %s: %s\n", PROGRAM_NAME, media[n].mnt, strerror(errno));
-		exit_status = EXIT_FAILURE;
+	if (xchdir(media[n].mnt, SET_TITLE) != FUNC_SUCCESS) {
+		xerror("%s: '%s': %s\n", PROGRAM_NAME, media[n].mnt, strerror(errno));
+		exit_status = FUNC_FAILURE;
 		goto EXIT;
 	}
 
