@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,11 +27,12 @@
 
 __BEGIN_DECLS
 
-int  mime_open(char **);
-char *xmagic(const char *, const int);
-int  mime_open_with(char *filename, char **);
-char **mime_open_with_tab(char *, const char *, const int);
-int  mime_open_url(char *);
+int  mime_open(char **args);
+int  mime_open_url(char *url);
+int  mime_open_with(char *filename, char **args);
+char **mime_open_with_tab(char *filename, const char *prefix,
+	const int only_names);
+char *xmagic(const char *file, const int query_mime);
 
 __END_DECLS
 
