@@ -1,9 +1,9 @@
-/* properties.h -- Functions to generate/handle file properties */
+/* properties.h -- Home of the p/pp, pc, oc, and stats commands */
 
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -53,16 +53,55 @@
 # define XFS_CASEFOLD_FL     0x40000000 /* Folder is case insensitive */
 #endif /* LINUX_FILE_ATTRS */
 
+/* Max size for a file size color (used by get_color_size()) */
+#define MAX_SHADE_LEN 26 /* "\x1b[0;x;38;2;xxx;xxx;xxxm\0" */
+
+/* Macros to calculate relative timestamps (used by get_color_age()) */
+#define RT_SECOND 1
+#define RT_MINUTE (60  * RT_SECOND)
+#define RT_HOUR   (60  * RT_MINUTE)
+#define RT_DAY    (24  * RT_HOUR)
+#define RT_WEEK   (7   * RT_DAY)
+#define RT_MONTH  (30  * RT_DAY)
+#define RT_YEAR   (365 * RT_DAY)
+
+/* Struct used by get_file_perms() */
+struct perms_t {
+	/* Field colors */
+	char *cur;
+	char *cuw;
+	char *cux;
+	char *cgr;
+	char *cgw;
+	char *cgx;
+	char *cor;
+	char *cow;
+	char *cox;
+	/* Field values */
+	char ur;
+	char uw;
+	char ux;
+	char gr;
+	char gw;
+	char gx;
+	char or;
+	char ow;
+	char ox;
+	char pad[7];
+};
+
 __BEGIN_DECLS
 
-int  properties_function(char **, const int);
-void print_analysis_stats(const off_t, const off_t, const char *, const char *);
-int  print_entry_props(const struct fileinfo *, const struct maxes_t *,
-	const uint8_t);
-int  set_file_perms(char **);
-int  set_file_owner(char **);
-
-void do_stat_and_exit(const int);
+void do_stat_and_exit(const int full_stat);
+void get_color_age(const time_t t, char *str, const size_t len);
+void get_color_size(const off_t s, char *str, const size_t len);
+struct perms_t
+     get_file_perms(const mode_t mode);
+int  properties_function(char **args, const int follow_link);
+void print_analysis_stats(const off_t total, const off_t largest,
+	const char *color, const char *name);
+int  set_file_perms(char **args);
+int  set_file_owner(char **args);
 
 __END_DECLS
 
