@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 
 __BEGIN_DECLS
 
-int remotes_function(char **);
+int remotes_function(char **args);
 int automount_remotes(void);
 int autounmount_remotes(void);
 
