@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -32,12 +32,11 @@
 #include <errno.h>
 #include <sys/resource.h> /* getrlimit(3), setrlimit(3) */
 #include <unistd.h> /* close(2), sysconf(3) */
-#include <limits.h> /* OPEN_MAX */
 
-#include "sanitize.h"
 #include "aux.h"
 #include "checks.h" /* is_number() */
 #include "misc.h"
+#include "sanitize.h"
 
 #define UNSAFE_CMD "Unsafe command. Consult the manpage for more information"
 /* If PATH cannot be retrieved from any other source, let's use this value */
@@ -46,7 +45,7 @@
 #endif
 
 /* Unset environ: little implementation of clearenv(3), not available
- * on some systems (not POSIX) */
+ * on some systems (not POSIX). */
 static void
 xclearenv(void)
 {
@@ -66,15 +65,15 @@ xclearenv(void)
 			continue;
 		}
 		if (len > lastlen) {
-			namebuf = (char *)xrealloc(namebuf, (len + 1) * sizeof(char));
+			namebuf = xnrealloc(namebuf, len + 1, sizeof(char));
 			lastlen = len;
 		}
 		memcpy(namebuf, environ[0], len);
 		namebuf[len] = '\0';
 		if (unsetenv(namebuf) == -1) {
-			xerror("%s: unsetenv: %s: %s\n", PROGRAM_NAME,
+			xerror("%s: unsetenv: '%s': %s\n", PROGRAM_NAME,
 				namebuf, strerror(errno));
-			exit(EXIT_FAILURE);
+			exit(FUNC_FAILURE);
 		}
 	}
 #endif /* !__NetBSD__ && !__HAIKU__ && !__APPLE__ && !__TERMUX__ */
@@ -90,7 +89,7 @@ set_path_env(void)
 #elif defined(_CS_PATH)
 	char *p = (char *)NULL;
 	size_t n = confstr(_CS_PATH, NULL, 0); /* Get value's size */
-	p = (char *)xnmalloc(n, sizeof(char)); /* Allocate space */
+	p = xnmalloc(n, sizeof(char)); /* Allocate space */
 	confstr(_CS_PATH, p, n);               /* Get value */
 	ret = setenv("PATH", p, 1);            /* Set it */
 	free(p);
@@ -100,7 +99,7 @@ set_path_env(void)
 
 	if (ret == -1) {
 		xerror("%s: setenv: PATH: %s\n", PROGRAM_NAME, strerror(errno));
-		exit(EXIT_FAILURE);
+		exit(FUNC_FAILURE);
 	}
 }
 
@@ -108,7 +107,7 @@ static void
 xsetenv(const char *name, const char *value)
 {
 	if (setenv(name, value, 1) == -1)
-		xerror("%s: setenv: %s: %s\n", PROGRAM_NAME, name, strerror(errno));
+		xerror("%s: setenv: '%s': %s\n", PROGRAM_NAME, name, strerror(errno));
 }
 
 /* See https://www.oreilly.com/library/view/secure-programming-cookbook/0596003943/ch01s09.html */
@@ -161,25 +160,67 @@ static int
 sanitize_shell_level(char *str)
 {
 	if (!str || !*str || !is_number(str))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	int a = atoi(str);
 	if (a < 1 || a > MAX_SHELL_LEVEL)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
+}
+
+/* Drop SUID/SGID privileges, if set. */
+static void
+drop_privs(void)
+{
+	uid_t ruid = getuid();
+	uid_t euid = geteuid();
+	gid_t rgid = getgid();
+	gid_t egid = getegid();
+
+	if (rgid != egid) {
+		int err = 0;
+#ifndef __linux__
+		setegid(rgid);
+		if (setgid(rgid) == -1) err = 1;
+#else
+		if (setregid(rgid, rgid) == -1) err = 1;
+#endif /* __linux__ */
+		if (err == 1 || setegid(egid) != -1 || getegid() != rgid) {
+			fprintf(stderr, _("%s: Error dropping group privileges. "
+				"Aborting.\n"), PROGRAM_NAME);
+			exit(FUNC_FAILURE);
+		}
+	}
+
+	if (ruid != euid) {
+		int err = 0;
+#ifndef __linux__
+		seteuid(ruid);
+		if (setuid(ruid) == -1) err = 1;
+#else
+		if (setreuid(ruid, ruid) == -1) err = 1;
+#endif /* __linux__ */
+		if (err == 1 || seteuid(euid) != -1 || geteuid() != ruid) {
+			fprintf(stderr, _("%s: Error dropping user privileges. "
+				"Aborting.\n"), PROGRAM_NAME);
+			exit(FUNC_FAILURE);
+		}
+	}
 }
 
 /* Sanitize the environment: set environ to NULL and then set a few
  * environment variables to get a minimally working environment.
  * Non-standard file descriptors are closed.
  * Core dumps are disabled.
+ * SUID/SGID privileges, if any, are dropped.
  * Umask is set to the most restrictive value: 0077. */
 int
 xsecure_env(const int mode)
 {
 	sanitize_file_descriptors();
 	disable_coredumps();
+	drop_privs();
 	umask(0077);
 
 	char *display = (char *)NULL,
@@ -212,7 +253,7 @@ xsecure_env(const int mode)
 	xsetenv("IFS", " \t\n");
 
 	if (mode == SECURE_ENV_FULL)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (user.name)
 		xsetenv("USER", user.name);
@@ -221,23 +262,23 @@ xsecure_env(const int mode)
 	if (user.shell)
 		xsetenv("SHELL", user.shell);
 
-	if (display && sanitize_cmd(display, SNT_DISPLAY) == EXIT_SUCCESS) {
+	if (display && sanitize_cmd(display, SNT_DISPLAY) == FUNC_SUCCESS) {
 		xsetenv("DISPLAY", display);
 	} else {
 		if (wayland_display)
 			xsetenv("WAYLAND_DISPLAY", wayland_display);
 	}
 
-	if (clifm_level && sanitize_shell_level(clifm_level) == EXIT_SUCCESS)
+	if (clifm_level && sanitize_shell_level(clifm_level) == FUNC_SUCCESS)
 		xsetenv("CLIFMLVL", clifm_level);
 
-	if (_term && sanitize_cmd(_term, SNT_MISC) == EXIT_SUCCESS)
+	if (_term && sanitize_cmd(_term, SNT_MISC) == FUNC_SUCCESS)
 		xsetenv("TERM", _term);
 
-	if (tz && sanitize_cmd(tz, SNT_MISC) == EXIT_SUCCESS)
+	if (tz && sanitize_cmd(tz, SNT_MISC) == FUNC_SUCCESS)
 		xsetenv("TZ", tz);
 
-	if (lang && sanitize_cmd(lang, SNT_MISC) == EXIT_SUCCESS) {
+	if (lang && sanitize_cmd(lang, SNT_MISC) == FUNC_SUCCESS) {
 		xsetenv("LANG", lang);
 		xsetenv("LC_ALL", lang);
 	} else {
@@ -247,7 +288,7 @@ xsecure_env(const int mode)
 	if (fzfopts)
 		xsetenv("FZF_DEFAULT_OPTS", fzfopts);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Sanitize cmd string coming from the mimelist file */
@@ -257,17 +298,17 @@ sanitize_mime(const char *cmd)
 	/* Only %f is allowed */
 	char *p = strchr(cmd, '%');
 	if (p && *(p + 1) != 'f')
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Disallow double ampersand */
 	p = strchr(cmd, '&');
 	if (p && *(p + 1) == '&')
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (strlen(cmd) > strspn(cmd, ALLOWED_CHARS_MIME))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Sanitize cmd string coming from the net file */
@@ -275,9 +316,9 @@ static int
 sanitize_net(const char *cmd)
 {
 	if (strlen(cmd) > strspn(cmd, ALLOWED_CHARS_NET))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Sanitize value of DISPLAY env var */
@@ -285,9 +326,9 @@ static int
 sanitize_misc(const char *str)
 {
 	if (strlen(str) > strspn(str, ALLOWED_CHARS_MISC))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Sanitize value of DISPLAY env var */
@@ -295,9 +336,9 @@ static int
 sanitize_display(const char *str)
 {
 	if (strlen(str) > strspn(str, ALLOWED_CHARS_DISPLAY))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Sanitize cmd string coming from profile, prompt, and autocmds */
@@ -305,18 +346,18 @@ static int
 sanitize_gral(const char *cmd)
 {
 	if (strlen(cmd) > strspn(cmd, ALLOWED_CHARS_GRAL))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
-/* Return EXIT_SUCCESS if at least one character in CMD is a non-escaped
- * blacklisted char (<>|;&$). Oterwise, return EXIT_FAILURE. */
+/* Return FUNC_SUCCESS if at least one character in CMD is a non-escaped
+ * blacklisted char (<>|;&$). Oterwise, return FUNC_FAILURE. */
 static int
 sanitize_blacklist(const char *cmd)
 {
 	if (!cmd || !*cmd)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	size_t i;
 	for (i = 0; cmd[i]; i++) {
@@ -329,13 +370,13 @@ sanitize_blacklist(const char *cmd)
 		case '$': /* fallthrough */
 		case '`':
 			if (i == 0 || cmd[i - 1] != '\\')
-				return EXIT_FAILURE;
+				return FUNC_FAILURE;
 			break;
 		default: break;
 		}
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Check if command name in STR contains slashes. Return 1 if found,
@@ -345,7 +386,7 @@ static int
 clean_cmd(const char *str)
 {
 	if (!str || !*str)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	char *p = strchr(str, ' ');
 	if (p)
@@ -356,30 +397,30 @@ clean_cmd(const char *str)
 		*p = ' ';
 
 	if (q) {
-		err('w', PRINT_PROMPT, _("%s: %s: Only command base names "
+		err('w', PRINT_PROMPT, _("%s: '%s': Only command base names "
 			"are allowed. Ex: 'nano' instead of '/usr/bin/nano'\n"),
 			PROGRAM_NAME, str);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
-/* Sanitize CMD according to TYPE. Returns EXIT_SUCCESS if command is safe or
- * EXIT_FAILURE if not. */
+/* Sanitize CMD according to TYPE. Returns FUNC_SUCCESS if command is safe or
+ * FUNC_FAILURE if not. */
 int
 sanitize_cmd(const char *str, const int type)
 {
 	if (!str || !*str)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	int exit_status = EXIT_FAILURE;
+	int exit_status = FUNC_FAILURE;
 
 	switch (type) {
 	case SNT_MIME:
-		if (clean_cmd(str) != EXIT_SUCCESS)
+		if (clean_cmd(str) != FUNC_SUCCESS)
 			/* Error message already pŕinted by clean_cmd() */
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 		exit_status = sanitize_mime(str);
 		break;
 	case SNT_NET: exit_status = sanitize_net(str); break;
@@ -389,20 +430,20 @@ sanitize_cmd(const char *str, const int type)
 	case SNT_PROMPT:  /* fallthrough */
 	case SNT_AUTOCMD: /* fallthrough */
 	case SNT_GRAL:
-		if (clean_cmd(str) != EXIT_SUCCESS)
-			return EXIT_FAILURE;
+		if (clean_cmd(str) != FUNC_SUCCESS)
+			return FUNC_FAILURE;
 		exit_status = sanitize_gral(str);
 		break;
 	case SNT_BLACKLIST: return sanitize_blacklist(str);
-	case SNT_NONE: return EXIT_SUCCESS;
-	default: return EXIT_SUCCESS;
+	case SNT_NONE: return FUNC_SUCCESS;
+	default: return FUNC_SUCCESS;
 	}
 
-	if (exit_status == EXIT_FAILURE) {
-		err('w', PRINT_PROMPT, "%s: %s: %s\n",
+	if (exit_status == FUNC_FAILURE) {
+		err('w', PRINT_PROMPT, "%s: '%s': %s\n",
 			PROGRAM_NAME, str, _(UNSAFE_CMD));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
