@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -31,8 +31,8 @@
 #include <unistd.h>
 
 #ifdef __sun
-# include <sys/termios.h> // TIOCGWINSZ
-#endif // __sun
+# include <sys/termios.h> /* TIOCGWINSZ */
+#endif /* __sun */
 
 /* We need rl_line_buffer in case of no matches and no metacharacter */
 #include <readline/readline.h>
@@ -40,11 +40,11 @@
 #include "aux.h"
 #include "checks.h"
 #include "colors.h"
-#include "exec.h"
 #include "messages.h"
 #include "misc.h"
 #include "navigation.h"
 #include "sort.h"
+#include "spawn.h"
 
 #define ERR_SKIP_REGEX 2
 
@@ -113,14 +113,12 @@ run_find(char *search_path, char *arg)
 # endif /* FIND_HAS_NO_REGEX */
 #endif /* _BE_POSIX */
 
-	int glob_char = check_glob_char(arg + 1, GLOB_REGEX);
+	const int glob_char = check_glob_char(arg + 1, GLOB_REGEX);
 	if (glob_char == 1)
 		return exec_find(name, _path, method, arg + 1);
 
-	int ret = EXIT_SUCCESS;
-
-	size_t pattern_len = strlen(arg + 1) + 5;
-	char *pattern = (char *)xnmalloc(pattern_len, sizeof(char));
+	const size_t pattern_len = strlen(arg + 1) + 5;
+	char *pattern = xnmalloc(pattern_len, sizeof(char));
 
 #if !defined(_BE_POSIX)
 	if (conf.search_strategy == REGEX_ONLY) {
@@ -139,7 +137,7 @@ run_find(char *search_path, char *arg)
 	snprintf(pattern, pattern_len, "*%s*", arg + 1);
 #endif /* !_BE_POSIX */
 
-	ret = exec_find(name, _path, method, pattern);
+	const int ret = exec_find(name, _path, method, pattern);
 	free(pattern);
 
 	return ret;
@@ -172,7 +170,7 @@ set_file_type_and_search_path(char **args, mode_t *file_type,
 	}
 
 	if (*file_type == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	/* Convert file type into a macro that can be decoded by stat(). If
 	 * file type is specified, matches will be checked against this value. */
@@ -188,31 +186,31 @@ set_file_type_and_search_path(char **args, mode_t *file_type,
 	case 'l': *file_type = invert == 1 ? DT_LNK : S_IFLNK; break;
 	case 'p': *file_type = invert == 1 ? DT_FIFO : S_IFIFO; break;
 	case 's': *file_type = invert == 1 ? DT_SOCK : S_IFSOCK; break;
-	case 'x': run_find(*search_path, args[0]); return EXIT_SUCCESS;
+	case 'x': run_find(*search_path, args[0]); return FUNC_SUCCESS;
 	default:
 		fprintf(stderr, _("search: '%c': Unrecognized file "
 			"type\n"), (char)*file_type);
 		break;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 chdir_search_path(char **search_path, const char *arg)
 {
 	if (strchr(*search_path, '\\')) {
-		char *deq_dir = dequote_str(*search_path, 0);
+		char *deq_dir = unescape_str(*search_path, 0);
 		if (!deq_dir) {
-			xerror(_("search: %s: Error dequoting file name\n"), arg);
-			return EXIT_FAILURE;
+			xerror(_("search: %s: Error unescaping file name\n"), arg);
+			return FUNC_FAILURE;
 		}
 
 		xstrsncpy(*search_path, deq_dir, strlen(deq_dir) + 1);
 		free(deq_dir);
 	}
 
-	size_t path_len = strlen(*search_path);
+	const size_t path_len = strlen(*search_path);
 	if (path_len > 1 && (*search_path)[path_len - 1] == '/')
 		(*search_path)[path_len - 1] = '\0';
 
@@ -223,40 +221,40 @@ chdir_search_path(char **search_path, const char *arg)
 			*search_path = (char *)NULL;
 	} else {
 		if (xchdir(*search_path, NO_TITLE) == -1) {
-			xerror("search: %s: %s\n", *search_path, strerror(errno));
-			return EXIT_FAILURE;
+			xerror("search: '%s': %s\n", *search_path, strerror(errno));
+			return FUNC_FAILURE;
 		}
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static char **
-glob_sort_dirs(glob_t globbed_files, size_t *g)
+glob_sort_dirs(glob_t *globbed_files, size_t *g)
 {
-	int *dirs = (int *)xnmalloc(globbed_files.gl_pathc + 1, sizeof(int));
-	char **gfiles = (char **)xnmalloc(globbed_files.gl_pathc + 1, sizeof(char *));
+	int *dirs = xnmalloc(globbed_files->gl_pathc + 1, sizeof(int));
+	char **gfiles = xnmalloc(globbed_files->gl_pathc + 1, sizeof(char *));
 	struct stat attr;
 	size_t i, n = 0;
 
-	for (i = 0; globbed_files.gl_pathv[i]; i++) {
-		if (stat(globbed_files.gl_pathv[i], &attr) != -1
+	for (i = 0; globbed_files->gl_pathv[i]; i++) {
+		if (stat(globbed_files->gl_pathv[i], &attr) != -1
 		&& S_ISDIR(attr.st_mode))
 			dirs[i] = 1;
 		else
 			dirs[i] = 0;
 	}
 
-	for (i = 0; globbed_files.gl_pathv[i]; i++) {
+	for (i = 0; globbed_files->gl_pathv[i]; i++) {
 		if (dirs[i] == 1) {
-			gfiles[n] = globbed_files.gl_pathv[i];
+			gfiles[n] = globbed_files->gl_pathv[i];
 			n++;
 		}
 	}
 
-	for (i = 0; globbed_files.gl_pathv[i]; i++) {
+	for (i = 0; globbed_files->gl_pathv[i]; i++) {
 		if (dirs[i] == 0) {
-			gfiles[n] = globbed_files.gl_pathv[i];
+			gfiles[n] = globbed_files->gl_pathv[i];
 			n++;
 		}
 	}
@@ -272,8 +270,7 @@ static struct search_t *
 get_glob_matches(char **gfiles, const char *search_path,
 	const mode_t file_type, const size_t g)
 {
-	struct search_t *matches = (struct search_t *)xnmalloc(
-		g + 1, sizeof(struct search_t));
+	struct search_t *matches = xnmalloc(g + 1, sizeof(struct search_t));
 
 	size_t i;
 	int n = 0;
@@ -339,8 +336,8 @@ get_non_matches_from_search_path(const char *search_path, char **gfiles,
 		return (struct search_t *)NULL;
 
 	int i, j, n = 0;
-	struct search_t *matches = (struct search_t *)xnmalloc(
-		(size_t)dir_entries + 1, sizeof(struct search_t));
+	struct search_t *matches = xnmalloc((size_t)dir_entries + 1,
+		sizeof(struct search_t));
 
 	for (i = 0; i < dir_entries; i++) {
 		int f = 0;
@@ -356,12 +353,11 @@ get_non_matches_from_search_path(const char *search_path, char **gfiles,
 			continue;
 
 #if !defined(_DIRENT_HAVE_D_TYPE)
-		mode_t type;
 		struct stat attr;
 		if (lstat(ent[i]->d_name, &attr) == -1)
 			continue;
 
-		type = get_dt(attr.st_mode);
+		const mode_t type = get_dt(attr.st_mode);
 		if (file_type && type != file_type)
 #else
 		if (file_type && ent[i]->d_type != file_type)
@@ -391,8 +387,7 @@ get_glob_matches_invert(char **gfiles, const char *search_path,
 		return get_non_matches_from_search_path(search_path, gfiles, file_type);
 
 	filesn_t i, j, n = 0;
-	struct search_t *matches =
-		(struct search_t *)xnmalloc((size_t)files + 1, sizeof(struct search_t));
+	struct search_t *matches = xnmalloc((size_t)files + 1, sizeof(struct search_t));
 
 	for (i = 0; file_info[i].name; i++) {
 		int f = 0;
@@ -472,10 +467,10 @@ construct_glob_query(char **arg, const int invert)
 	}
 
 	/* Search strategy is glob-only */
-	size_t len = strlen(*arg);
+	const size_t len = strlen(*arg);
 
 	char *q = savestring(query, len - (invert == 1 ? 2 : 1));
-	*arg = (char *)xrealloc(*arg, (len + 3) * sizeof(char));
+	*arg = xnrealloc(*arg, (len + 3), sizeof(char));
 
 	snprintf((*arg) + 1, len + 2, "*%s*", q);
 	free(q);
@@ -520,7 +515,7 @@ print_glob_matches(struct search_t *matches, const char *search_path)
 	int columns_n = calculate_glob_output_columns(flongest, found);
 
 	/* colors_list() makes use of TAB_OFFSET. We don't want it here. */
-	size_t tab_offset_bk = tab_offset;
+	const size_t tab_offset_bk = tab_offset;
 	tab_offset = 0;
 
 	int last_column = 0, i;
@@ -569,7 +564,7 @@ static int
 search_glob(char **args)
 {
 	if (!args || !args[0])
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	int invert = (args[0][1] == '!');
 
@@ -577,22 +572,22 @@ search_glob(char **args)
 	mode_t file_type = 0;
 
 	if (set_file_type_and_search_path(args, &file_type,
-	&search_path, invert) == EXIT_FAILURE) {
+	&search_path, invert) == FUNC_FAILURE) {
 		return ERR_SKIP_REGEX;
 	}
 
 	if (file_type == 'x') /* Recursive search via find(1) */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	/* If we have a path ("/str /path"), chdir into it, since glob(3)
 	 * works on CWD. */
 	if (search_path && *search_path
-	&& chdir_search_path(&search_path, args[1]) == EXIT_FAILURE)
+	&& chdir_search_path(&search_path, args[1]) == FUNC_FAILURE)
 		return ERR_SKIP_REGEX;
 
 	search_query = construct_glob_query(&args[0], invert);
 	if (!search_query && conf.search_strategy != GLOB_ONLY)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Get matches, if any. */
 	glob_t globbed_files;
@@ -602,9 +597,9 @@ search_glob(char **args)
 
 		/* Go back to the directory we came from */
 		if (search_path && xchdir(workspaces[cur_ws].path, NO_TITLE) == -1)
-			xerror("search: %s: %s\n", workspaces[cur_ws].path, strerror(errno));
+			xerror("search: '%s': %s\n", workspaces[cur_ws].path, strerror(errno));
 
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	/* We have matches */
@@ -614,7 +609,7 @@ search_glob(char **args)
 
 	/* glob(3) doesn't sort directories first. Let's do it ourselves */
 	if (conf.list_dirs_first == 1)
-		gfiles = glob_sort_dirs(globbed_files, &g);
+		gfiles = glob_sort_dirs(&globbed_files, &g);
 
 	/* We need to store pointers to matching file names in array of pointers,
 	 * just as the file name length (to construct the columned output), and,
@@ -643,11 +638,11 @@ search_glob(char **args)
 
 	/* If needed, go back to the directory we came from */
 	if (search_path && xchdir(workspaces[cur_ws].path, NO_TITLE) == -1) {
-		xerror("search: %s: %s\n", workspaces[cur_ws].path, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("search: '%s': %s\n", workspaces[cur_ws].path, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
-	return (matches == 0 ? EXIT_FAILURE : EXIT_SUCCESS);
+	return (matches == 0 ? FUNC_FAILURE : FUNC_SUCCESS);
 }
 
 /* Original string is either "/QUERY" or "/!QUERY". Let's extract QUERY.
@@ -658,13 +653,13 @@ construct_regex_query(char **arg, const int invert, int *regex_found)
 	char *query = *arg + (invert == 1 ? 2 : 1);
 
 	*regex_found = check_regex(query);
-	if (*regex_found == EXIT_SUCCESS)
+	if (*regex_found == FUNC_SUCCESS)
 		return query;
 
-	size_t len = strlen(*arg);
+	const size_t len = strlen(*arg);
 
 	char *q = savestring(query, len - (invert == 1 ? 2 : 1));
-	*arg = (char *)xrealloc(*arg, (len + 5) * sizeof(char));
+	*arg = xnrealloc(*arg, (len + 5), sizeof(char));
 
 	snprintf(*arg + 1, len + 4, ".*%s.*", q);
 	free(q);
@@ -675,14 +670,16 @@ construct_regex_query(char **arg, const int invert, int *regex_found)
 static void
 err_regex_no_match(const int regex_found, const char *arg)
 {
-	char *input = (conf.autocd == 1 && !arg && (regex_found == EXIT_FAILURE
+	char *input = (conf.autocd == 1 && !arg && (regex_found == FUNC_FAILURE
 			|| (search_flags & NO_GLOB_CHAR)) && rl_line_buffer)
 			? strrchr(rl_line_buffer, '/') : (char *)NULL;
 
 	if (input && input != rl_line_buffer) {
 		/* Input string contains at least two slashes. It looks like a path:
 		 * let's err like it was. */
-		xerror("cd: %s: %s\n", rl_line_buffer, strerror(ENOENT));
+		char *p = unescape_str(rl_line_buffer, 0);
+		xerror("cd: '%s': %s\n", p ? p : rl_line_buffer, strerror(ENOENT));
+		free(p);
 	} else if (search_flags & NO_GLOB_CHAR) {
 		fputs(_("search: No matches found\n"), stderr);
 	} else {
@@ -705,7 +702,7 @@ free_regex_dirlist(struct dirent ***dirlist, const int tmp_files)
 	free(*dirlist);
 
 	if (xchdir(workspaces[cur_ws].path, NO_TITLE) == -1)
-		xerror("search: %s: %s\n", workspaces[cur_ws].path, strerror(errno));
+		xerror("search: '%s': %s\n", workspaces[cur_ws].path, strerror(errno));
 }
 
 static int
@@ -717,7 +714,7 @@ check_regex_file_type(struct dirent **reg_dirlist, const int index,
 		mode_t type;
 		struct stat attr;
 		if (lstat(reg_dirlist[index]->d_name, &attr) == -1)
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 
 		switch (attr.st_mode & S_IFMT) {
 		case S_IFBLK: type = DT_BLK; break;
@@ -738,13 +735,13 @@ check_regex_file_type(struct dirent **reg_dirlist, const int index,
 #else
 		if (reg_dirlist[index]->d_type != file_type)
 #endif /* !_DIRENT_HAVE_D_TYPE */
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 	} else { /* Searching in CWD. */
 		if (file_info[index].type != file_type)
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static struct search_t
@@ -791,7 +788,7 @@ calc_columns(const size_t largest_file, const size_t matches)
 {
 	struct winsize w;
 	ioctl(STDOUT_FILENO, TIOCGWINSZ, &w);
-	unsigned short termcols = w.ws_col;
+	unsigned short termcols = w.ws_col > 0 ? w.ws_col : 80;
 
 	size_t columns;
 	if (largest_file == 0 || largest_file > termcols)
@@ -830,14 +827,13 @@ print_regex_matches(const mode_t file_type, struct dirent **reg_dirlist,
 	int *regex_index)
 {
 	/* colors_list() makes use of TAB_OFFSET. We don't need it here. */
-	size_t tab_offset_bk = tab_offset;
+	const size_t tab_offset_bk = tab_offset;
 	tab_offset = 0;
 
 	size_t total; /* Total number of matches (without file type filter) */
 	for (total = 0; regex_index[total] > -1; total++);
 
-	struct search_t *list =
-		(struct search_t *)xnmalloc(total + 1, sizeof(struct search_t));
+	struct search_t *list = xnmalloc(total + 1, sizeof(struct search_t));
 
 	size_t matches = 0; /* Number of filtered matches */
 
@@ -846,7 +842,7 @@ print_regex_matches(const mode_t file_type, struct dirent **reg_dirlist,
 		int index = regex_index[i];
 
 		if (file_type != 0 && check_regex_file_type(reg_dirlist,
-		index, file_type) == EXIT_FAILURE)
+		index, file_type) == FUNC_FAILURE)
 			continue;
 
 		list[matches] = load_entry_info(reg_dirlist, index);
@@ -860,8 +856,8 @@ print_regex_matches(const mode_t file_type, struct dirent **reg_dirlist,
 	}
 
 	int eln_pad = -1;
-	size_t largest_file = get_regex_largest(list, (int)matches, &eln_pad);
-	size_t columns = calc_columns(largest_file, matches);
+	const size_t largest_file = get_regex_largest(list, (int)matches, &eln_pad);
+	const size_t columns = calc_columns(largest_file, matches);
 
 	size_t last_col = 0, cur_col = 0;
 
@@ -903,34 +899,35 @@ static int
 search_regex(char **args)
 {
 	if (!args || !args[0])
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	int invert = (args[0][1] == '!');
 	char *search_query = (char *)NULL, *search_path = (char *)NULL;
 	mode_t file_type = 0;
 
 	if (set_file_type_and_search_path(args, &file_type,
-	&search_path, 1) == EXIT_FAILURE)
-		return EXIT_FAILURE;
+	&search_path, 1) == FUNC_FAILURE)
+		return FUNC_FAILURE;
 
 	if (file_type == 'x') /* Recursive search via find(1) */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	struct dirent **reg_dirlist = (struct dirent **)NULL;
 	int tmp_files = - 1;
 
 	if (search_path && *search_path) {
-		if (chdir_search_path(&search_path, args[1]) == EXIT_FAILURE)
-			return EXIT_FAILURE;
+		if (chdir_search_path(&search_path, args[1]) == FUNC_FAILURE)
+			return FUNC_FAILURE;
 
 		tmp_files = scandir(".", &reg_dirlist, skip_files, xalphasort);
 		if (tmp_files == -1) {
-			xerror("search: %s: %s\n", search_path, strerror(errno));
+			xerror("search: '%s': %s\n", search_path, strerror(errno));
 
 			if (xchdir(workspaces[cur_ws].path, NO_TITLE) == -1)
-				xerror("search: %s: %s\n", workspaces[cur_ws].path, strerror(errno));
+				xerror("search: '%s': %s\n", workspaces[cur_ws].path,
+					strerror(errno));
 
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 		}
 	}
 
@@ -945,23 +942,24 @@ search_regex(char **args)
 		: (REG_NOSUB | REG_EXTENDED | REG_ICASE);
 	int ret = regcomp(&regex_files, search_query, reg_flags);
 
-	if (ret != EXIT_SUCCESS) {
+	if (ret != FUNC_SUCCESS) {
 		xerror(_("'%s': Invalid regular expression\n"), search_query);
 		regfree(&regex_files);
 		free_regex_dirlist(&reg_dirlist, tmp_files);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	size_t found = 0;
-	int *regex_index = (int *)xnmalloc((search_path ? (size_t)tmp_files
+	int *regex_index = xnmalloc((search_path ? (size_t)tmp_files
 		: (size_t)files) + 2, sizeof(int));
-	size_t max = (search_path && *search_path) ? (size_t)tmp_files : (size_t)files;
+	const size_t max =
+		(search_path && *search_path) ? (size_t)tmp_files : (size_t)files;
 
 	for (i = 0; i < max; i++) {
 		char *name = (search_path && *search_path) ? reg_dirlist[i]->d_name
 		: file_info[i].name;
 
-		if (regexec(&regex_files, name, 0, NULL, 0) == EXIT_SUCCESS) {
+		if (regexec(&regex_files, name, 0, NULL, 0) == FUNC_SUCCESS) {
 			if (invert == 0) {
 				regex_index[found] = (int)i;
 				found++;
@@ -981,16 +979,17 @@ search_regex(char **args)
 		err_regex_no_match(regex_found, args[1]);
 		free(regex_index);
 		free_regex_dirlist(&reg_dirlist, tmp_files);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	/* We have matches: print them. */
-	size_t matches = print_regex_matches(file_type, reg_dirlist, regex_index);
+	const size_t matches =
+		print_regex_matches(file_type, reg_dirlist, regex_index);
 
 	free(regex_index);
 	free_regex_dirlist(&reg_dirlist, tmp_files);
 
-	return matches == 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+	return matches == 0 ? FUNC_FAILURE : FUNC_SUCCESS;
 }
 
 static int
@@ -1004,12 +1003,14 @@ err_glob_no_match(const char *arg)
 	if (input && input != rl_line_buffer) {
 		/* Input string contains two slashes: it looks like a path, so let's
 		 * err like it was. */
-		xerror("cd: %s: %s\n", rl_line_buffer, strerror(ENOENT));
-		return EXIT_FAILURE;
+		char *p = unescape_str(rl_line_buffer, 0);
+		xerror("cd: '%s': %s\n", p ? p : rl_line_buffer, strerror(ENOENT));
+		free(p);
+		return FUNC_FAILURE;
 	}
 
 	fputs(_("search: No matches found\n"), stderr);
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* We have three search strategies:
@@ -1021,14 +1022,14 @@ search_function(char **args)
 {
 	if (args[1] && IS_HELP(args[1])) {
 		puts(SEARCH_USAGE);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (conf.search_strategy == REGEX_ONLY)
 		return search_regex(args);
 
-	int ret = search_glob(args);
-	if (ret != EXIT_FAILURE)
+	const int ret = search_glob(args);
+	if (ret != FUNC_FAILURE)
 		return (ret == ERR_SKIP_REGEX ? 1 : ret);
 
 	if (conf.search_strategy == GLOB_ONLY)
