@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -37,7 +37,6 @@
 # endif /* __TINYC__ */
 # include <sys/ioctl.h>
 #endif /* __linux__ || __HAIKU__ || __APPLE__ || __sun || __CYGWIN__ */
-#include <limits.h>
 
 #ifdef __sun
 # include <sys/termios.h> /* TIOCGWINSZ */
@@ -46,40 +45,41 @@
 #include "aux.h"
 #include "checks.h"
 #include "colors.h"
+#include "file_operations.h"
+#include "init.h"
 #include "listing.h"
+#include "messages.h"
 #include "misc.h"
 #include "navigation.h"
+#include "properties.h" /* get_size_color() */
 #include "readline.h"
 #include "selection.h"
 #include "sort.h"
-#include "messages.h"
-#include "file_operations.h"
-#include "init.h"
 
-/* Save selected elements into a tmp file. Returns 1 if success and 0
- * if error. This function allows the user to work with multiple
- * instances of the program: he/she can select some files in the
+/* Save selected elements into a tmp file. Returns 1 on success or 0
+ * on error. This function allows the user to work with multiple
+ * instances of the program: they can select some files in the
  * first instance and then execute a second one to operate on those
- * files as he/she wishes. */
+ * files as they wish. */
 int
 save_sel(void)
 {
 	if (selfile_ok == 0 || config_ok == 0 || !sel_file)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (sel_n == 0) {
 		if (unlink(sel_file) == -1) {
-			xerror("sel: %s: %s\n", sel_file, strerror(errno));
-			return EXIT_FAILURE;
+			xerror("sel: '%s': %s\n", sel_file, strerror(errno));
+			return FUNC_FAILURE;
 		}
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	int fd = 0;
 	FILE *fp = open_fwrite(sel_file, &fd);
 	if (!fp) {
-		xerror("sel: %s: %s\n", sel_file, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("sel: '%s': %s\n", sel_file, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	size_t i;
@@ -89,7 +89,7 @@ save_sel(void)
 	}
 
 	fclose(fp);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 int
@@ -98,25 +98,46 @@ select_file(char *file)
 	if (!file || !*file)
 		return 0;
 
-	int exists = 0, new_sel = 0, j;
-	size_t flen = strlen(file);
+	if (sel_n == MAX_SEL) {
+		xerror(_("sel: Cannot select any more files"));
+		return 0;
+	}
+
+	static char buf[PATH_MAX + 1];
+	char *tfile = file;
+	struct stat a;
+	int exists = 0, new_sel = 0;
+
+	const size_t flen = strlen(file);
 	if (flen > 1 && file[flen - 1] == '/')
 		file[flen - 1] = '\0';
 
+	/* If we are in a virtual directory, dereference symlinks */
+	if (virtual_dir == 1 && lstat(file, &a) == 0 && S_ISLNK(a.st_mode)
+	&& is_file_in_cwd(file)) {
+		*buf = '\0';
+		const ssize_t ret = xreadlink(XAT_FDCWD, file, buf, sizeof(buf));
+		if (ret == -1) {
+			xerror(_("sel: Cannot select file '%s': %s\n"),
+				file, strerror(errno));
+			return 0;
+		}
+		tfile = buf;
+	}
+
 	/* Check if FILE is already in the selection box */
-	j = (int)sel_n;
+	filesn_t j = (filesn_t)sel_n;
 	while (--j >= 0) {
-		if (*file == *sel_elements[j].name
-		&& strcmp(sel_elements[j].name, file) == 0) {
+		if (*tfile == *sel_elements[j].name
+		&& strcmp(sel_elements[j].name, tfile) == 0) {
 			exists = 1;
 			break;
 		}
 	}
 
 	if (exists == 0) {
-		sel_elements = (struct sel_t *)xrealloc(sel_elements,
-			(sel_n + 2) * sizeof(struct sel_t));
-		sel_elements[sel_n].name = savestring(file, strlen(file));
+		sel_elements = xnrealloc(sel_elements, sel_n + 2, sizeof(struct sel_t));
+		sel_elements[sel_n].name = savestring(tfile, strlen(tfile));
 		sel_elements[sel_n].size = (off_t)UNSET;
 		sel_n++;
 		sel_elements[sel_n].name = (char *)NULL;
@@ -124,26 +145,26 @@ select_file(char *file)
 
 		new_sel++;
 	} else {
-		xerror(_("sel: %s: Already selected\n"), file);
+		xerror(_("sel: '%s': Already selected\n"), tfile);
 	}
 
 	return new_sel;
 }
 
 static char **
-load_matches_invert_cwd(glob_t gbuf, const mode_t filetype, int *matches)
+load_matches_invert_cwd(glob_t *gbuf, const mode_t filetype, int *matches)
 {
-	char **list = (char **)xnmalloc((size_t)files + 2, sizeof(char *));
+	char **list = xnmalloc((size_t)files + 2, sizeof(char *));
 
 	filesn_t i = files;
 	while (--i >= 0) {
 		if (filetype != 0 && file_info[i].type != filetype)
 			continue;
 
-		filesn_t j = (filesn_t)gbuf.gl_pathc;
+		filesn_t j = (filesn_t)gbuf->gl_pathc;
 		while (--j >= 0) {
-			if (*file_info[i].name == *gbuf.gl_pathv[j]
-			&& strcmp(file_info[i].name, gbuf.gl_pathv[j]) == 0)
+			if (*file_info[i].name == *gbuf->gl_pathv[j]
+			&& strcmp(file_info[i].name, gbuf->gl_pathv[j]) == 0)
 				break;
 		}
 
@@ -157,10 +178,10 @@ load_matches_invert_cwd(glob_t gbuf, const mode_t filetype, int *matches)
 }
 
 static char **
-load_matches_invert_nocwd(glob_t gbuf, struct dirent **ent,
+load_matches_invert_nocwd(glob_t *gbuf, struct dirent **ent,
 	const mode_t filetype, int *matches, const int ret)
 {
-	char **list = (char **)xnmalloc((size_t)ret + 2, sizeof(char *));
+	char **list = xnmalloc((size_t)ret + 2, sizeof(char *));
 
 	int i = ret;
 	while (--i >= 0) {
@@ -168,17 +189,17 @@ load_matches_invert_nocwd(glob_t gbuf, struct dirent **ent,
 		struct stat attr;
 		if (lstat(ent[i]->d_name, &attr) == -1)
 			continue;
-		mode_t type = get_dt(attr.st_mode);
+		const mode_t type = get_dt(attr.st_mode);
 		if (filetype != 0 && type != filetype)
 #else
 		if (filetype != 0 && ent[i]->d_type != filetype)
 #endif /* !_DIRENT_HAVE_D_TYPE */
 			continue;
 
-		int j = (int)gbuf.gl_pathc;
+		int j = (int)gbuf->gl_pathc;
 		while (--j >= 0) {
-			if (*ent[i]->d_name == *gbuf.gl_pathv[j]
-			&& strcmp(ent[i]->d_name, gbuf.gl_pathv[j]) == 0)
+			if (*ent[i]->d_name == *gbuf->gl_pathv[j]
+			&& strcmp(ent[i]->d_name, gbuf->gl_pathv[j]) == 0)
 				break;
 		}
 
@@ -207,27 +228,27 @@ convert_filetype_mask(const mode_t filetype)
 }
 
 static char **
-load_matches(glob_t gbuf, const mode_t filetype, int *matches)
+load_matches(glob_t *gbuf, const mode_t filetype, int *matches)
 {
-	char **list = (char **)xnmalloc(gbuf.gl_pathc + 2, sizeof(char *));
-	mode_t type = convert_filetype_mask(filetype);
+	char **list = xnmalloc(gbuf->gl_pathc + 2, sizeof(char *));
+	const mode_t type = convert_filetype_mask(filetype);
 
-	int i = (int)gbuf.gl_pathc;
+	int i = (int)gbuf->gl_pathc;
 	while (--i >= 0) {
-		char *basename = strrchr(gbuf.gl_pathv[i], '/');
-		if (!basename && SELFORPARENT(gbuf.gl_pathv[i]))
+		char *basename = strrchr(gbuf->gl_pathv[i], '/');
+		if (!basename && SELFORPARENT(gbuf->gl_pathv[i]))
 			continue;
 		if (basename && basename[1] && SELFORPARENT(basename + 1))
 			continue;
 
 		if (filetype != 0) {
 			struct stat attr;
-			if (lstat(gbuf.gl_pathv[i], &attr) == -1
+			if (lstat(gbuf->gl_pathv[i], &attr) == -1
 			|| (attr.st_mode & S_IFMT) != type)
 				continue;
 		}
 
-		list[*matches] = gbuf.gl_pathv[i];
+		list[*matches] = gbuf->gl_pathv[i];
 		(*matches)++;
 	}
 
@@ -245,8 +266,8 @@ select_matches(char **list, const char *sel_path, const int matches)
 			continue;
 
 		if (sel_path) {
-			size_t tmp_len = strlen(sel_path) + strlen(list[i]) + 2;
-			char *tmp = (char *)xnmalloc(tmp_len, sizeof(char));
+			const size_t tmp_len = strlen(sel_path) + strlen(list[i]) + 2;
+			char *tmp = xnmalloc(tmp_len, sizeof(char));
 			snprintf(tmp, tmp_len, "%s/%s", sel_path, list[i]);
 
 			new_sel += select_file(tmp);
@@ -267,13 +288,13 @@ select_matches(char **list, const char *sel_path, const int matches)
 		if (*workspaces[cur_ws].path == '/'
 		&& !*(workspaces[cur_ws].path + 1)) {
 			/* CWD is root */
-			size_t tmp_len = strlen(list[i]) + 2;
-			tmp = (char *)xnmalloc(tmp_len, sizeof(char));
+			const size_t tmp_len = strlen(list[i]) + 2;
+			tmp = xnmalloc(tmp_len, sizeof(char));
 			snprintf(tmp, tmp_len, "/%s", list[i]);
 		} else {
-			size_t tmp_len = strlen(workspaces[cur_ws].path)
+			const size_t tmp_len = strlen(workspaces[cur_ws].path)
 				+ strlen(list[i]) + 2;
-			tmp = (char *)xnmalloc(tmp_len, sizeof(char));
+			tmp = xnmalloc(tmp_len, sizeof(char));
 			snprintf(tmp, tmp_len, "%s/%s",
 				workspaces[cur_ws].path, list[i]);
 		}
@@ -301,7 +322,7 @@ sel_glob(char *str, const char *sel_path, const mode_t filetype)
 	}
 
 	int ret = glob(pattern, GLOB_BRACE, NULL, &gbuf);
-	if (ret != EXIT_SUCCESS) {
+	if (ret != FUNC_SUCCESS) {
 		globfree(&gbuf);
 		return (-1);
 	}
@@ -312,19 +333,19 @@ sel_glob(char *str, const char *sel_path, const mode_t filetype)
 
 	if (invert == 1) {
 		if (!sel_path) {
-			list = load_matches_invert_cwd(gbuf, filetype, &matches);
+			list = load_matches_invert_cwd(&gbuf, filetype, &matches);
 		} else {
 			ret = scandir(sel_path, &ent, skip_files, xalphasort);
 			if (ret == -1) {
-				xerror("sel: %s: %s\n", sel_path, strerror(errno));
+				xerror("sel: '%s': %s\n", sel_path, strerror(errno));
 				globfree(&gbuf);
 				return (-1);
 			}
 
-			list = load_matches_invert_nocwd(gbuf, ent, filetype, &matches, ret);
+			list = load_matches_invert_nocwd(&gbuf, ent, filetype, &matches, ret);
 		}
 	} else {
-		list = load_matches(gbuf, filetype, &matches);
+		list = load_matches(&gbuf, filetype, &matches);
 	}
 
 	list[matches] = (char *)NULL;
@@ -354,7 +375,7 @@ sel_regex_cwd(regex_t regex, const mode_t filetype, const int invert)
 		if (filetype != 0 && file_info[i].type != filetype)
 			continue;
 
-		char tmp_path[PATH_MAX];
+		char tmp_path[PATH_MAX + 1];
 		if (*workspaces[cur_ws].path == '/'
 		&& !*(workspaces[cur_ws].path + 1)) {
 			snprintf(tmp_path, sizeof(tmp_path), "/%s", file_info[i].name);
@@ -363,7 +384,7 @@ sel_regex_cwd(regex_t regex, const mode_t filetype, const int invert)
 				workspaces[cur_ws].path, file_info[i].name);
 		}
 
-		if (regexec(&regex, file_info[i].name, 0, NULL, 0) == EXIT_SUCCESS) {
+		if (regexec(&regex, file_info[i].name, 0, NULL, 0) == FUNC_SUCCESS) {
 			if (invert == 0)
 				new_sel += select_file(tmp_path);
 		} else if (invert == 1) {
@@ -380,14 +401,14 @@ sel_regex_nocwd(regex_t regex, const char *sel_path, const mode_t filetype,
 {
 	int new_sel = 0;
 	struct dirent **list = (struct dirent **)NULL;
-	int filesn = scandir(sel_path, &list, skip_files, xalphasort);
+	const int filesn = scandir(sel_path, &list, skip_files, xalphasort);
 
 	if (filesn == -1) {
-		xerror("sel: %s: %s\n", sel_path, strerror(errno));
+		xerror("sel: '%s': %s\n", sel_path, strerror(errno));
 		return (-1);
 	}
 
-	mode_t type = convert_filetype_mask(filetype);
+	const mode_t type = convert_filetype_mask(filetype);
 
 	int i = (int)filesn;
 	while (--i >= 0) {
@@ -400,11 +421,11 @@ sel_regex_nocwd(regex_t regex, const char *sel_path, const mode_t filetype,
 			}
 		}
 
-		size_t tmp_len = strlen(sel_path) + strlen(list[i]->d_name) + 2;
-		char *tmp_path = (char *)xnmalloc(tmp_len, sizeof(char));
+		const size_t tmp_len = strlen(sel_path) + strlen(list[i]->d_name) + 2;
+		char *tmp_path = xnmalloc(tmp_len, sizeof(char));
 		snprintf(tmp_path, tmp_len, "%s/%s", sel_path, list[i]->d_name);
 
-		if (regexec(&regex, list[i]->d_name, 0, NULL, 0) == EXIT_SUCCESS) {
+		if (regexec(&regex, list[i]->d_name, 0, NULL, 0) == FUNC_SUCCESS) {
 			if (invert == 0)
 				new_sel += select_file(tmp_path);
 		} else if (invert == 1) {
@@ -439,7 +460,7 @@ sel_regex(char *str, const char *sel_path, const mode_t filetype)
 	int reg_flags = conf.case_sens_list == 1 ? (REG_NOSUB | REG_EXTENDED)
 			: (REG_NOSUB | REG_EXTENDED | REG_ICASE);
 
-	if (regcomp(&regex, pattern, reg_flags) != EXIT_SUCCESS) {
+	if (regcomp(&regex, pattern, reg_flags) != FUNC_SUCCESS) {
 		xerror(_("sel: %s: Invalid regular expression\n"), str);
 		regfree(&regex);
 		return (-1);
@@ -472,12 +493,12 @@ convert_filetype(mode_t *filetype)
 	case 's': *filetype = DT_SOCK; break;
 	case 'p': *filetype = DT_FIFO; break;
 	default:
-		xerror(_("%s: '%c': Unrecognized file type\n"),
-		    PROGRAM_NAME, (char)*filetype);
-		return EXIT_FAILURE;
+		xerror(_("sel: '%c': Unrecognized file type.\n"
+			"Try 'sel --help' for more information.\n"), (char)*filetype);
+		return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static char *
@@ -499,7 +520,8 @@ parse_sel_params(char ***args, int *ifiletype, mode_t *filetype, int *isel_path)
 		if (*(*args)[i] == '~') {
 			char *exp_path = tilde_expand((*args)[i]);
 			if (!exp_path) {
-				xerror("sel: %s: %s\n", (*args)[i], strerror(errno));
+				xerror("sel: '%s': Cannot expand tilde\n", (*args)[i]);
+				*filetype = (mode_t)-1;
 				return (char *)NULL;
 			}
 
@@ -508,8 +530,10 @@ parse_sel_params(char ***args, int *ifiletype, mode_t *filetype, int *isel_path)
 		}
 	}
 
-	if (*filetype != 0 && convert_filetype(filetype) == EXIT_FAILURE)
+	if (*filetype != 0 && convert_filetype(filetype) == FUNC_FAILURE) {
+		*filetype = (mode_t)-1;
 		return (char *)NULL;
+	}
 
 	return sel_path;
 }
@@ -517,11 +541,11 @@ parse_sel_params(char ***args, int *ifiletype, mode_t *filetype, int *isel_path)
 static char *
 construct_sel_path(char *sel_path)
 {
-	char tmpdir[PATH_MAX];
+	char tmpdir[PATH_MAX + 1];
 	xstrsncpy(tmpdir, sel_path, sizeof(tmpdir));
 
 	if (*sel_path == '.' && realpath(sel_path, tmpdir) == NULL) {
-		xerror("sel: %s: %s\n", sel_path, strerror(errno));
+		xerror("sel: '%s': %s\n", sel_path, strerror(errno));
 		return (char *)NULL;
 	}
 
@@ -529,6 +553,7 @@ construct_sel_path(char *sel_path)
 		char *exp_path = tilde_expand(sel_path);
 		if (!exp_path) {
 			xerror("%s\n", _("sel: Error expanding path"));
+			errno = 1;
 			return (char *)NULL;
 		}
 
@@ -538,11 +563,12 @@ construct_sel_path(char *sel_path)
 
 	char *dir = (char *)NULL;
 	if (*tmpdir != '/') {
-		size_t dir_len = strlen(workspaces[cur_ws].path) + strlen(tmpdir) + 2;
-		dir = (char *)xnmalloc(dir_len, sizeof(char));
+		const size_t dir_len = strlen(workspaces[cur_ws].path)
+			+ strnlen(tmpdir, sizeof(tmpdir)) + 2;
+		dir = xnmalloc(dir_len, sizeof(char));
 		snprintf(dir, dir_len, "%s/%s", workspaces[cur_ws].path, tmpdir);
 	} else {
-		dir = savestring(tmpdir, strlen(tmpdir));
+		dir = savestring(tmpdir, strnlen(tmpdir, sizeof(tmpdir)));
 	}
 
 	return dir;
@@ -551,30 +577,25 @@ construct_sel_path(char *sel_path)
 static char *
 check_sel_path(char **sel_path)
 {
-	size_t len = strlen(*sel_path);
+	const size_t len = strlen(*sel_path);
 	if (len > 0 && (*sel_path)[len - 1] == '/')
 		(*sel_path)[len - 1] = '\0';
 
 	if (strchr(*sel_path, '\\')) {
-		char *deq = dequote_str(*sel_path, 0);
+		char *deq = unescape_str(*sel_path, 0);
 		if (deq) {
 			xstrsncpy(*sel_path, deq, strlen(deq) + 1);
 			free(deq);
 		}
 	}
 
+	errno = 0;
 	char *dir = construct_sel_path(*sel_path);
 	if (!dir)
 		return (char *)NULL;
 
-	if (access(dir, X_OK) == -1) {
-		xerror("sel: %s: %s\n", dir, strerror(errno));
-		free(dir);
-		return (char *)NULL;
-	}
-
 	if (xchdir(dir, NO_TITLE) == -1) {
-		xerror("sel: %s: %s\n", dir, strerror(errno));
+		xerror("sel: '%s': %s\n", dir, strerror(errno));
 		free(dir);
 		return (char *)NULL;
 	}
@@ -594,10 +615,12 @@ get_sel_file_size(const size_t i, int *status)
 	if (lstat(sel_elements[i].name, &attr) == -1)
 		return (off_t)-1;
 
-	int base = xargs.si == 1 ? 1000 : 1024;
+	const int base = xargs.si == 1 ? 1000 : 1024;
 	if (S_ISDIR(attr.st_mode)) {
+		fputs(_("Calculating file size... "), stdout); fflush(stdout);
 		sel_elements[i].size = (off_t)(dir_size(sel_elements[i].name,
 			0, status) * base);
+		putchar('\r'); ERASE_TO_RIGHT; fflush(stdout);
 	} else {
 		sel_elements[i].size = (off_t)FILE_SIZE(attr);
 	}
@@ -605,78 +628,33 @@ get_sel_file_size(const size_t i, int *status)
 	return sel_elements[i].size;
 }
 
-static void
-print_total_size(const off_t total, const int status)
-{
-	char *human_size = construct_human_size(total);
-
-	char err[sizeof(xf_c) + 6]; *err = '\0';
-	if (status != 0)
-		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
-
-	printf(_("%s%sTotal size%s: %s%s\n"), df_c, BOLD, df_c, err, human_size);
-}
-
-static void
-print_selected_files(void)
-{
-	if (conf.clear_screen == 1)
-		CLEAR;
-
-	printf(_("%sSelection Box%s\n\n"), BOLD, df_c);
-
-	size_t t = tab_offset, i;
-	off_t total = 0;
-	tab_offset = 0;
-
-	int status = 0;
-	uint8_t epad = DIGINUM(sel_n);
-
-	flags |= IN_SELBOX_SCREEN;
-	for (i = 0; i < sel_n; i++) {
-		printf("%s%*zu%s ", el_c, epad, i + 1, df_c);
-		colors_list(sel_elements[i].name, NO_ELN, NO_PAD, PRINT_NEWLINE);
-
-		fputs(_("Calculating file size... "), stdout); fflush(stdout);
-		int ret = 0;
-		off_t s = get_sel_file_size(i, &ret);
-		if (ret != 0)
-			status = ret;
-		putchar('\r'); ERASE_TO_RIGHT; fflush(stdout);
-
-		if (s != (off_t)-1)
-			total += s;
-	}
-	flags &= ~IN_SELBOX_SCREEN;
-	tab_offset = t;
-
-	putchar('\n');
-	print_total_size(total, status);
-}
-
 static int
 print_sel_results(const int new_sel, const char *sel_path,
 	const char *pattern, const int error)
 {
 	if (new_sel > 0 && xargs.stealth_mode != 1 && sel_file
-	&& save_sel() != EXIT_SUCCESS) {
-		err('e', PRINT_PROMPT, _("sel: Error writing selected files "
+	&& save_sel() != FUNC_SUCCESS) {
+		err('e', PRINT_PROMPT, _("sel: Error writing files "
 			"into the selections file\n"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (sel_path && xchdir(workspaces[cur_ws].path, NO_TITLE) == -1) {
-		xerror("sel: %s: %s\n", workspaces[cur_ws].path, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("sel: '%s': %s\n", workspaces[cur_ws].path, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	if (new_sel <= 0) {
 		if (pattern && error == 0)
 			fputs(_("sel: No matches found\n"), stderr);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	get_sel_files();
+	if (sel_n == 0) {
+		fputs(_("sel: No matches found\n"), stderr);
+		return FUNC_FAILURE;
+	}
 
 	if (conf.autols == 1 && error == 0)
 		reload_dirlist();
@@ -684,7 +662,7 @@ print_sel_results(const int new_sel, const char *sel_path,
 	print_reload_msg(_("%d file(s) selected\n"), new_sel);
 	print_reload_msg(_("%zu total selected file(s)\n"), sel_n);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static char *
@@ -697,21 +675,21 @@ construct_sel_filename(const char *dir, const char *name)
 		if (*workspaces[cur_ws].path == '/'
 		&& !*(workspaces[cur_ws].path + 1)) {
 			flen = strlen(name) + 2;
-			f = (char *)xnmalloc(flen, sizeof(char));
+			f = xnmalloc(flen, sizeof(char));
 			snprintf(f, flen, "/%s", name);
 
 			return f;
 		}
 
 		flen = strlen(workspaces[cur_ws].path) + strlen(name) + 2;
-		f = (char *)xnmalloc(flen, sizeof(char));
+		f = xnmalloc(flen, sizeof(char));
 		snprintf(f, flen, "%s/%s", workspaces[cur_ws].path, name);
 
 		return f;
 	}
 
 	flen = strlen(dir) + strlen(name) + 2;
-	f = (char *)xnmalloc(flen, sizeof(char));
+	f = xnmalloc(flen, sizeof(char));
 	snprintf(f, flen, "%s/%s", dir, name);
 
 	return f;
@@ -723,7 +701,7 @@ select_filename(char *arg, char *dir, int *errors)
 	int new_sel = 0;
 
 	if (strchr(arg, '\\')) {
-		char *deq_str = dequote_str(arg, 0);
+		char *deq_str = unescape_str(arg, 0);
 		if (deq_str) {
 			xstrsncpy(arg, deq_str, strlen(deq_str) + 1);
 			free(deq_str);
@@ -738,10 +716,10 @@ select_filename(char *arg, char *dir, int *errors)
 		char *tmp = construct_sel_filename(dir, name);
 		struct stat attr;
 		if (lstat(tmp, &attr) == -1) {
-			xerror("sel: %s: %s\n", arg, strerror(errno));
+			xerror("sel: '%s': %s\n", arg, strerror(errno));
 			(*errors)++;
 		} else {
-			int r = select_file(tmp);
+			const int r = select_file(tmp);
 			new_sel += r;
 			if (r == 0)
 				(*errors)++;
@@ -752,10 +730,10 @@ select_filename(char *arg, char *dir, int *errors)
 
 	struct stat a;
 	if (lstat(arg, &a) == -1) {
-		xerror("sel: %s: %s\n", name, strerror(errno));
+		xerror("sel: '%s': %s\n", name, strerror(errno));
 		(*errors)++;
 	} else {
-		int r = select_file(name);
+		const int r = select_file(name);
 		new_sel += r;
 		if (r == 0)
 			(*errors)++;
@@ -796,11 +774,11 @@ select_pattern(char *arg, const char *dir, const mode_t filetype, int *err)
 int
 sel_function(char **args)
 {
-	if (!args) return EXIT_FAILURE;
+	if (!args) return FUNC_FAILURE;
 
 	if (!args[1] || IS_HELP(args[1])) {
 		puts(_(SEL_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	mode_t filetype = 0;
@@ -809,8 +787,16 @@ sel_function(char **args)
 	char *dir = (char *)NULL, *pattern = (char *)NULL;
 	char *sel_path = parse_sel_params(&args, &ifiletype, &filetype, &isel_path);
 
-	if (sel_path)
+	if (sel_path) {
 		dir = check_sel_path(&sel_path);
+		if (!dir || !*dir) {
+			free(dir);
+			return errno;
+		}
+	} else if (filetype == (mode_t)-1) { /* parse_sel_params() error */
+		free(dir);
+		return FUNC_FAILURE;
+	}
 
 	for (i = 1; args[i]; i++) {
 		if (i == ifiletype || i == isel_path)
@@ -818,7 +804,7 @@ sel_function(char **args)
 		f++;
 
 		pattern = (char *)NULL;
-		if (check_regex(args[i]) == EXIT_SUCCESS) {
+		if (check_regex(args[i]) == FUNC_SUCCESS) {
 			pattern = args[i];
 			if (*pattern == '!')
 				pattern++;
@@ -838,7 +824,7 @@ sel_function(char **args)
 }
 
 void
-show_sel_files(void)
+list_selected_files(void)
 {
 	if (sel_n == 0) {
 		puts(_("sel: No selected files"));
@@ -848,22 +834,29 @@ show_sel_files(void)
 	if (conf.clear_screen == 1)
 		CLEAR;
 
+	if (conf.pager == 0)
+		HIDE_CURSOR;
+
 	printf(_("%s%sSelection Box%s\n"), df_c, BOLD, df_c);
 
 	int reset_pager = 0;
 
 	putchar('\n');
+
 	struct winsize w;
-	ioctl(STDOUT_FILENO, TIOCGWINSZ, &w);
-	size_t counter = 0;
-	int t_lines = (int)w.ws_row;
+	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) == -1)
+		w.ws_row = 24;
+
+	int t_lines = w.ws_row > 0 ? (int)w.ws_row : 24;
 	t_lines -= 2;
+
+	size_t counter = 0;
 	size_t i;
 	off_t total = 0;
 
-	size_t t = tab_offset;
+	const size_t t = tab_offset;
 	tab_offset = 0;
-	uint8_t epad = DIGINUM(sel_n);
+	const uint8_t epad = DIGINUM(sel_n);
 	int status = 0;
 
 	flags |= IN_SELBOX_SCREEN;
@@ -900,12 +893,11 @@ show_sel_files(void)
 		printf("%s%*zu%s ", el_c, epad, i + 1, df_c);
 		colors_list(sel_elements[i].name, NO_ELN, NO_PAD, PRINT_NEWLINE);
 
-		fputs(_("Calculating file size... "), stdout); fflush(stdout);
 		int ret = 0;
-		off_t s = get_sel_file_size(i, &ret);
+		const off_t s = get_sel_file_size(i, &ret);
 		if (ret != 0)
 			status = ret;
-		putchar('\r'); ERASE_TO_RIGHT; fflush(stdout);
+
 		if (s != (off_t)-1)
 			total += s;
 	}
@@ -917,7 +909,14 @@ show_sel_files(void)
 	if (status != 0)
 		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
 
-	printf(_("\n%s%sTotal size%s: %s%s\n"), df_c, BOLD, df_c, err, human_size);
+	char s[MAX_SHADE_LEN]; *s = '\0';
+	if (conf.colorize == 1)
+		get_color_size(total, s, sizeof(s));
+
+	printf(_("\n%sTotal size: %s%s%s%s\n"), df_c, err, s, human_size, df_c);
+
+	if (conf.pager == 0)
+		UNHIDE_CURSOR;
 
 	if (reset_pager == 1)
 		conf.pager = 1;
@@ -927,27 +926,27 @@ static int
 edit_selfile(void)
 {
 	if (!sel_file || !*sel_file || sel_n == 0)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	struct stat attr;
 	if (stat(sel_file, &attr) == -1)
 		goto ERROR;
 
-	time_t mtime_old = (time_t)attr.st_mtime;
+	const time_t prev_mtime = attr.st_mtime;
 
-	if (open_file(sel_file) != EXIT_SUCCESS) {
-		xerror("%s\n", _("sel: Could not open the selections file"));
-		return EXIT_FAILURE;
+	if (open_file(sel_file) != FUNC_SUCCESS) {
+		xerror("%s\n", _("sel: Cannot open the selections file"));
+		return FUNC_FAILURE;
 	}
 
 	/* Compare new and old modification times: if they match, nothing was modified */
 	if (stat(sel_file, &attr) == -1)
 		goto ERROR;
 
-	if (mtime_old == (time_t)attr.st_mtime)
-		return EXIT_SUCCESS;
+	if (prev_mtime == attr.st_mtime)
+		return FUNC_SUCCESS;
 
-	int ret = get_sel_files();
+	const int ret = get_sel_files();
 	if (conf.autols == 1)
 		reload_dirlist();
 
@@ -955,8 +954,8 @@ edit_selfile(void)
 	return ret;
 
 ERROR:
-	xerror("sel: %s: %s\n", sel_file, strerror(errno));
-	return EXIT_FAILURE;
+	xerror("sel: '%s': %s\n", sel_file, strerror(errno));
+	return FUNC_FAILURE;
 }
 
 static int
@@ -986,7 +985,7 @@ deselect_entries(char **desel_path, const size_t desel_n, int *error,
 			dn--;
 			*error = 1;
 			if (desel_screen == 0) {
-				xerror(_("%s: %s: No such selected file\n"),
+				xerror(_("%s: '%s': No such selected file\n"),
 					PROGRAM_NAME, desel_path[i]);
 			}
 			continue;
@@ -996,9 +995,9 @@ deselect_entries(char **desel_path, const size_t desel_n, int *error,
 		 * deselected element (actually, moving each string after it to
 		 * the previous position). */
 		for (j = desel_index; j < (int)(sel_n - 1); j++) {
-			size_t len = strlen(sel_elements[j + 1].name);
+			const size_t len = strlen(sel_elements[j + 1].name);
 			sel_elements[j].name =
-				(char *)xrealloc(sel_elements[j].name, (len + 1) * sizeof(char));
+				xnrealloc(sel_elements[j].name, len + 1, sizeof(char));
 
 			xstrsncpy(sel_elements[j].name, sel_elements[j + 1].name, len + 1);
 			sel_elements[j].size = sel_elements[j + 1].size;
@@ -1009,7 +1008,7 @@ deselect_entries(char **desel_path, const size_t desel_n, int *error,
 	 * be used anymore, since they contain the same value as the last
 	 * non-deselected element due to the above array rearrangement. */
 	for (i = 1; i <= dn; i++) {
-		int sel_index = (int)sel_n - i;
+		const int sel_index = (int)sel_n - i;
 
 		if (sel_index >= 0 && sel_elements[sel_index].name) {
 			free(sel_elements[sel_index].name);
@@ -1028,9 +1027,9 @@ desel_entries(char **desel_elements, const size_t desel_n, const int desel_scree
 
 	/* Get entries to be deselected */
 	if (desel_screen == 1) { /* Coming from the deselect screen */
-		desel_path = (char **)xnmalloc(desel_n, sizeof(char *));
+		desel_path = xnmalloc(desel_n, sizeof(char *));
 		while (--i >= 0) {
-			int desel_int = atoi(desel_elements[i]);
+			const int desel_int = atoi(desel_elements[i]);
 			if (desel_int == INT_MIN) {
 				desel_path[i] = (char *)NULL;
 				continue;
@@ -1043,7 +1042,7 @@ desel_entries(char **desel_elements, const size_t desel_n, const int desel_scree
 	}
 
 	int err = 0;
-	int dn = deselect_entries(desel_path, desel_n, &err, desel_screen);
+	const int dn = deselect_entries(desel_path, desel_n, &err, desel_screen);
 
 	/* Update the number of selected files according to the number of
 	 * deselected files. */
@@ -1052,10 +1051,8 @@ desel_entries(char **desel_elements, const size_t desel_n, const int desel_scree
 	if ((int)sel_n < 0)
 		sel_n = 0;
 
-	if (sel_n > 0) {
-		sel_elements = (struct sel_t *)xrealloc(sel_elements,
-			sel_n * sizeof(struct sel_t));
-	}
+	if (sel_n > 0)
+		sel_elements = xnrealloc(sel_elements, sel_n, sizeof(struct sel_t));
 
 	/* Deallocate local arrays. */
 	i = (int)desel_n;
@@ -1075,10 +1072,10 @@ desel_entries(char **desel_elements, const size_t desel_n, const int desel_scree
 
 	if (err == 1) {
 		save_sel();
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Deselect all selected files */
@@ -1099,10 +1096,10 @@ deselect_all(void)
 
 /* Deselect files passed as parameters to the desel command
  * Returns zero on success or 1 on error. */
-static inline int
+static int
 deselect_from_args(char **args)
 {
-	char **ds = (char **)xnmalloc(args_n + 1, sizeof(char *));
+	char **ds = xnmalloc(args_n + 1, sizeof(char *));
 	size_t j, k = 0;
 
 	for (j = 1; j <= args_n; j++) {
@@ -1115,12 +1112,14 @@ deselect_from_args(char **args)
 	}
 	ds[k] = (char *)NULL;
 
-	if (desel_entries(ds, args_n, 0) == EXIT_FAILURE)
-		return EXIT_FAILURE;
+	if (desel_entries(ds, args_n, 0) == FUNC_FAILURE)
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
+/* Desel screen: take user input and return an array of input substrings,
+ * updating N to the number of substrings. */
 static char **
 get_desel_input(size_t *n)
 {
@@ -1133,15 +1132,22 @@ get_desel_input(size_t *n)
 	while (!line)
 		line = rl_no_hist(dp);
 
-	char **entries = get_substr(line, ' ');
+	/* get_substr() will try to expand ranges, in which case a range with
+	 * no second field is expanded from the value of the first field to the
+	 * ELN of the last listed file in the CWD (value taken from the global
+	 * variable FILES). But, since we are deselecting files, FILES must be the
+	 * number of selected files (SEL_N), and not that of listed files in
+	 * the CWD. */
+	const filesn_t files_bk = files;
+	files = (filesn_t)sel_n;
+	char **entries = get_substr(line, ' ', 1);
+	files = files_bk;
 	free(line);
 
 	if (!entries)
 		return (char **)NULL;
 
-	size_t i;
-	for (i = 0; entries[i]; i++)
-		(*n)++;
+	for (*n = 0; entries[*n]; (*n)++);
 
 	return entries;
 }
@@ -1155,7 +1161,7 @@ free_desel_elements(const size_t desel_n, char ***desel_elements)
 	free(*desel_elements);
 }
 
-static inline int
+static int
 handle_alpha_entry(const int i, const size_t desel_n, char **desel_elements)
 {
 	if (*desel_elements[i] == 'e' && !desel_elements[i][1]) {
@@ -1167,42 +1173,43 @@ handle_alpha_entry(const int i, const size_t desel_n, char **desel_elements)
 		free_desel_elements(desel_n, &desel_elements);
 		if (conf.autols == 1)
 			reload_dirlist();
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*desel_elements[i] == '*' && !desel_elements[i][1]) {
 		free_desel_elements(desel_n, &desel_elements);
-		int exit_status = deselect_all();
+		const int exit_status = deselect_all();
 		if (conf.autols == 1)
 			reload_dirlist();
 		return exit_status;
 	}
 
-	printf(_("desel: %s: Invalid entry\n"), desel_elements[i]);
+	printf(_("desel: '%s': Invalid entry\n"), desel_elements[i]);
 	free_desel_elements(desel_n, &desel_elements);
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
-static inline int
+static int
 valid_desel_eln(const int i, const size_t desel_n, char **desel_elements)
 {
-	int n = atoi(desel_elements[i]);
+	const int n = atoi(desel_elements[i]);
 
 	if (n <= 0 || (size_t)n > sel_n) {
 		printf(_("desel: %s: Invalid ELN\n"), desel_elements[i]);
 		free_desel_elements(desel_n, &desel_elements);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
-static inline int
+static int
 end_deselect(const int err, char ***args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
+	const size_t argsbk = args_n;
+	size_t desel_files = 0;
 
-	size_t argsbk = args_n, desel_files = 0;
 	if (args_n > 0) {
 		size_t i;
 		for (i = 1; i <= args_n; i++) {
@@ -1214,19 +1221,19 @@ end_deselect(const int err, char ***args)
 	}
 
 	if (!err && save_sel() != 0)
-		exit_status = EXIT_FAILURE;
+		exit_status = FUNC_FAILURE;
 
 	get_sel_files();
 
 	/* There is still some selected file and we are in the desel
-	 * screen: reload this screen */
-	if (sel_n && argsbk == 0 && deselect(*args) != 0)
-		exit_status = EXIT_FAILURE;
+	 * screen: reload this screen. */
+	if (sel_n > 0 && argsbk == 0)
+		return deselect(*args);
 
 	if (err)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	if (conf.autols == 1 && exit_status == EXIT_SUCCESS)
+	if (conf.autols == 1 && exit_status == FUNC_SUCCESS)
 		reload_dirlist();
 
 	if (argsbk > 0) {
@@ -1239,38 +1246,45 @@ end_deselect(const int err, char ***args)
 	return exit_status;
 }
 
+static int
+handle_desel_args(char **args)
+{
+	if (strcmp(args[1], "*") == 0 || strcmp(args[1], "a") == 0
+	|| strcmp(args[1], "all") == 0) {
+		const size_t n = sel_n;
+		const int ret = deselect_all();
+
+		if (conf.autols == 1)
+			reload_dirlist();
+
+		if (ret == FUNC_SUCCESS) {
+			print_reload_msg(_("%zu file(s) deselected\n"), n);
+			print_reload_msg(_("0 total selected file(s)\n"));
+		}
+		return ret;
+
+	} else {
+		const int err = deselect_from_args(args);
+		return end_deselect(err, &args);
+	}
+}
+
 int
 deselect(char **args)
 {
 	if (!args)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (sel_n == 0) {
 		puts(_("desel: No selected files"));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	int err = 0;
+	if (args[1] && *args[1])
+		return handle_desel_args(args);
 
-	if (args[1] && *args[1]) {
-		if (strcmp(args[1], "*") == 0 || strcmp(args[1], "a") == 0
-		|| strcmp(args[1], "all") == 0) {
-			size_t n = sel_n;
-			int ret = deselect_all();
-			if (conf.autols == 1)
-				reload_dirlist();
-			if (ret == EXIT_SUCCESS) {
-				print_reload_msg(_("%zu file(s) deselected\n"), n);
-				print_reload_msg(_("0 total selected file(s)\n"));
-			}
-			return ret;
-		} else {
-			err = deselect_from_args(args);
-			return end_deselect(err, &args);
-		}
-	}
-
-	print_selected_files();
+	/* No arguments: print the deselection screen. */
+	list_selected_files();
 
 	size_t desel_n = 0;
 	char **desel_elements = get_desel_input(&desel_n);
@@ -1280,10 +1294,10 @@ deselect(char **args)
 		if (!is_number(desel_elements[i])) /* Only for 'q', 'e', and '*' */
 			return handle_alpha_entry(i, desel_n, desel_elements);
 
-		if (valid_desel_eln(i, desel_n, desel_elements) == EXIT_FAILURE)
-			return EXIT_FAILURE;
+		if (valid_desel_eln(i, desel_n, desel_elements) == FUNC_FAILURE)
+			return FUNC_FAILURE;
 	}
 
 	desel_entries(desel_elements, desel_n, 1);
-	return end_deselect(err, &args);
+	return end_deselect(0, &args);
 }
