@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -32,10 +32,10 @@
 
 __BEGIN_DECLS
 
-int  deselect(char **);
-int  sel_function(char **);
-int  select_file(char *);
-void show_sel_files(void);
+int  deselect(char **args);
+void list_selected_files(void);
+int  sel_function(char **args);
+int  select_file(char *file);
 int  save_sel(void);
 int  deselect_all(void);
 
