@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=30;46:nd=4;1;31:nf=2;4;31:\
 no=31;47:or=2;4;36:ow=34;42:pi=35:so=1;35:su=37;41:sg=30;43:st=37;44:\
 tw=30;42:uf=2;4;37:"
 
-#define DEF_IFACE_COLORS "bm=1;36:dd=:df=0:dg=:dl=2;37:\
+#define DEF_IFACE_COLORS "bm=1;36:db=:dd=:de=:df=0:dg=:dk=:dl=2;37:\
 dn=0;2;37:do=:dp=:dr=:dw=:dxd=:dxr=:dz=:\
 el=0;36:em=1;31:fc=0;2;37:hb=0;36:hc=2;31:lc=:\
 hd=0;36:he=0;36:hn=0:hp=0;36:hq=0;33:hr=0;31:hs=0;32:hv=0;32:\
@@ -182,8 +182,11 @@ xf=1;31:xs=0;32:"
 #define DEF_HW_C  "\x1b[1;31m" /* Backslash (aka whack) */
 
 /* Colors for the properties and long/detail view functions */
+#define DEF_DB_C  "\x1b[0;33m" /* File allocated blocks */
 #define DEF_DD_C  "\x1b[0;36m" /* Modification date */
+#define DEF_DE_C  "\x1b[0;36m" /* Inode number (long view only) */
 #define DEF_DG_C  "\x1b[0;35m" /* UID, GID */
+#define DEF_DK_C  "\x1b[0;31m" /* Number of links (long view only) */
 #define DEF_DN_C  "\x1b[0;2;37m" /* Dash (no attribute) */
 #define DEF_DO_C  "\x1b[0;36m" /* Perms in octal */
 #define DEF_DP_C  "\x1b[0;35m" /* SUID, SGID, Sticky */
@@ -194,10 +197,10 @@ xf=1;31:xs=0;32:"
 #define DEF_DZ_C  "\x1b[0;32m" /* Size (only for dirs) */
 
 /* Default color shades for date field in file properties */
-#define DEF_DATE_SHADES_8   "1,31-2,36-1,36,36-2"
+#define DEF_DATE_SHADES_8   "1,31-2,36-1,36,36-2,36-2,36-2"
 
 /* Default color shades for size field in file properties */
-#define DEF_SIZE_SHADES_8   "1,31-2,32,33,31"
+#define DEF_SIZE_SHADES_8   "1,31-2,36,32,33,31,35"
 
 #define DEF_DIR_ICO_C "\x1b[0;33m"
 
@@ -207,15 +210,15 @@ xf=1;31:xs=0;32:"
 
 #define DEF_FILE_COLORS_256 "bd=1;38;5;229:ca=30;41:cd=1;38;5;214:di=1;34:\
 ed=2;34:ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=7;36:nd=1;31:nf=2;31:no=4;31;47:\
-or=4;2;36:ow=34;42:pi=38;5;170:sg=30;43:so=1;35:st=37;44:su=37;41:tw=30;42:\
+or=4;2;36:ow=34;42:pi=35:sg=30;43:so=1;35:st=37;44:su=37;41:tw=30;42:\
 uf=4;2;37:"
 
-#define DEF_IFACE_COLORS_256 "bm=1;32:dd=:df=0:dg=38;5;140:dl=2;37:dn=:do=:\
-dp=:dr=:dw=:dxd=:dxr=:dz=:el=36:em=1;31:fc=38;5;246:hb=36:hc=2;37:hd=36:he=36:\
-hn=:hp=36:hq=LY:hr=31:hs=32:hv=32:lc=38;5;43:li=1;32:mi=1;36:nm=1;32:ro=:\
-si=1;34:sb=2;33:sc=2;36:sd=2;37:sf=4;2;36:sh=2;35:sp=2;31:sx=2;32:sz=4;2;35:\
-ti=1;36:ts=4;35:tt=1;2;36:tx=0:wc=1;36:wm=38;5;228:ws1=34:ws2=31:ws3=38;5;228:\
-ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
+#define DEF_IFACE_COLORS_256 "bm=1;32:db=:dd=:de=:df=0:dg=38;5;140:\
+dk=:dl=2;37:dn=:do=:dp=:dr=:dw=:dxd=:dxr=:dz=:el=36:em=1;31:fc=38;5;246:hb=36:\
+hc=2;37:hd=36:he=36:hn=:hp=36:hq=LY:hr=31:hs=32:hv=32:lc=38;5;43:li=1;32:\
+mi=1;36:nm=1;32:ro=:si=1;34:sb=2;33:sc=2;36:sd=2;37:sf=4;2;36:sh=2;35:sp=2;31:\
+sx=2;32:sz=4;2;35:ti=1;36:ts=4;35:tt=1;2;36:tx=0:wc=1;36:wm=38;5;228:ws1=34:\
+ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 
 #define DEF_EXT_COLORS_256 "*.tar=1;31:*.tgz=1;31:*.arc=1;31:*.arj=1;31:\
 *.taz=1;31:*.lha=1;31:*.lz4=1;31:*.lzh=1;31:*.lzma=1;31:*.tlz=1;31:*.txz=1;31:\
@@ -229,10 +232,10 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 *.xbm=35:*.xpm=35:*.png=35:*.svg=35:*.svgz=35:*.pcx=35:*.pbm=35:*.pgm=35:\
 *.ppm=35:*.tga=35:*.tif=35:*.tiff=35:*.mng=35:\
 *.mov=1;35:*.mpg=1;35:*.mpeg=1;35:*.m2v=1;35:*.mkv=1;35:*.webm=1;35:\
-*.webp=1;35:*.ogm=1;35:*.mp4=1;35:*.m4v=1;35:*.mp4v=1;35:*.vob=1;35:*.qt=1;35:\
+*.ogm=1;35:*.mp4=1;35:*.m4v=1;35:*.mp4v=1;35:*.vob=1;35:*.qt=1;35:\
 *.nuv=1;35:*.wmv=1;35:*.asf=1;35:*.rm=1;35:*.rmvb=1;35:*.flc=1;35:*.avi=1;35:\
 *.fli=1;35:*.flv=1;35:*.gl=1;35:*.dl=1;35:*.xcf=1;35:*.xwd=1;35:*.yuv=1;35:\
-*.cgm=1;35:*.emf=1;35:*.ogv=1;35:*.ogx=1;35:\
+*.cgm=1;35:*.emf=1;35:*.ogv=1;35:*.ogx=1;35:*.webp=1;35:\
 *.aac=38;5;214:*.au=38;5;214:*.m4a=38;5;214:*.mid=38;5;214:*.midi=38;5;214:\
 *.mp3=38;5;214:*.mka=38;5;214:*.ogg=38;5;214:*.opus=38;5;214:*.spx=38;5;214:\
 *.wma=38;5;214:*.wv=38;5;214:*.wav=38;5;214:*.flac=38;5;214:*.aif=38;5;214:\
@@ -273,7 +276,7 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #endif /* __sun */
 #define DEF_OR_C256  "\x1b[2;4;36m" /* Orphaned/broken symlink */
 #define DEF_OW_C256  "\x1b[34;42m"  /* Other-writable */
-#define DEF_PI_C256  "\x1b[38;5;170m"   /* FIFO/pipe */
+#define DEF_PI_C256  "\x1b[35m"     /* FIFO/pipe */
 #define DEF_SG_C256  "\x1b[30;43m"  /* SGID file */
 #define DEF_SO_C256  "\x1b[1;35m"   /* Socket */
 #define DEF_ST_C256  "\x1b[37;44m"  /* Sticky bit set */
@@ -342,8 +345,11 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_HW_C256  "\x1b[1;31m" /* Backslash (aka whack) */
 
 /* Colors for the properties and long/detail view functions */
+#define DEF_DB_C256  "\x1b[0;38;5;214m" /* File allocated blocks */
 #define DEF_DD_C256  "\x1b[0;36m"       /* Modification date */
+#define DEF_DE_C256  "\x1b[0;38;5;79m"  /* Inode number (long view only) */
 #define DEF_DG_C256  "\x1b[0;38;5;140m" /* UID, GID */
+#define DEF_DK_C256  "\x1b[0;38;5;197m" /* Number of links (long view only) */
 #define DEF_DN_C256  "\x1b[0;2;37m"     /* Dash (no attribute) */
 #define DEF_DO_C256  "\x1b[0;38;5;79m"  /* Perms in octal */
 #define DEF_DP_C256  "\x1b[0;38;5;140m" /* SUID, SGID, Sticky */
@@ -361,6 +367,11 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 
 #define DEF_DIR_ICO_C256 "\x1b[0;33m"
 
+/* Character used to print unkonwn/wrong values */
+#define UNKNOWN_CHR     '?'
+/* Same as UNKNOWN_CHR, but as a string */
+#define UNKNOWN_STR     "?"
+#define UNKNOWN_STR_LEN 1
 
 /* Characters used to classify files when running colorless (and classify
  * is enabled) */
@@ -374,7 +385,7 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define LINK_CHR    '@'
 #define SOCK_CHR    '='
 #define WHT_CHR     '%' /* BSD whiteout */
-#define UNKNOWN_CHR '?'
+#define UNK_CHR     UNKNOWN_CHR
 
 /* Characters used to represent file types in the permissions string
  * (long view and p/pp command) */
@@ -385,26 +396,31 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define CHARDEV_PCHR 'c'
 #define FIFO_PCHR    'p'
 #define LNK_PCHR     'l'
-#define UNK_PCHR     '?'
 #define WHT_PCHR     'w' /* Whiteout (BSD) */
 #define ARCH1_PCHR   'a' /* Archive state 1 (NetBSD) */
 #define ARCH2_PCHR   'A' /* Archive state 2 (NetBSD) */
 #define DOOR_PCHR    'D' /* Door (Solaris) */
 #define PORT_PCHR    'P' /* Event port (Solaris) */
+#define UNK_PCHR     UNKNOWN_CHR
 
 #define SELFILE_CHR  '*'
 #define TRIMFILE_CHR '~'
 
 /* Character used to mark files with extended attributes (long view) */
 #define XATTR_CHAR '@'
+#define XATTR_STR  "@"
 
-/* Character used to mark dir sizes for which du(1) reported an error */
+/* Character used to mark dir sizes for which du(1) reported an error. */
 #define DU_ERR_CHAR '!'
 
 /* Character used to replace invalid characters (either a control char or an
  * invalid UTF-8 char) in file names. */
 #define INVALID_CHR '^'
 
+/* String used for the messages pointer (ex: "-> 2 file(s) removed")
+ * Note: The color used for this string is 'mi' (in the color scheme file). */
+#define MSG_PTR_STR "->"
+
 #define MNT_UDEVIL 	0
 #define MNT_UDISKS2 1
 
@@ -442,6 +458,7 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_EXT_CMD_OK 1
 #define DEF_FILES_COUNTER 1
 #define DEF_FOLLOW_SYMLINKS 1
+#define DEF_FOLLOW_SYMLINKS_LONG 0
 #define DEF_FULL_DIR_SIZE 0
 #define DEF_FUZZY_MATCH 0
 #define DEF_FUZZY_MATCH_ALGO 2 /* 1 or 2. 2 is Unicode aware, but slower than 1 */
@@ -473,13 +490,18 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_NOELN 0
 #define DEF_ONLY_DIRS 0
 #define DEF_PAGER 0
+/* Possible values: PAGER_AUTO, PAGER_LONG, and PAGER_SHORT */
+#define DEF_PAGER_VIEW PAGER_AUTO
 #define DEF_PRINTSEL 0
 #define DEF_PRINT_REMOVED_FILES 1
 #define DEF_PRIVATE_WS_SETTINGS 0
-/* "xfpims" =
- * xattrs/caps/ACLs, files counter, perms, owner/grp ids, mod time, size (human) */
-#define DEF_PROP_FIELDS "xfpims"
+/* "xfpIsm" =
+ * xattrs/caps/ACLs, files counter, perms, owner/grp names, size (human), mtime */
+#define DEF_PROP_FIELDS "xfpIsm"
+#define DEF_PROP_FIELDS_GAP 1 /* Spaces between columns in long view */
 #define DEF_PURGE_JUMPDB 0
+#define DEF_READ_AUTOCMD_FILES 0
+#define DEF_READ_DOTHIDDEN 0
 #define DEF_READONLY 0
 #define DEF_REFRESH_ON_EMPTY_LINE 1
 #define DEF_REFRESH_ON_RESIZE 1
@@ -492,6 +514,7 @@ ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_SHARE_SELBOX 0
 #define DEF_SHOW_HIDDEN 0
 #define DEF_SHOW_BACKUP_FILES 1
+#define DEF_SKIP_NON_ALNUM_PREFIX 1
 #define DEF_SI 0 /* If 1, compute sizes in powers of 1000 instead of 1024 */
 /* Available sort options:
  * SNONE, SNAME, STSIZE, SATIME, SBTIME, SCTIME, SMTIME
