@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -43,19 +43,18 @@ typedef char *rl_cpvfunc_t;
 #endif /* __OpenBSD__ */
 
 #include "aux.h"
+#include "builtins.h"
 #include "checks.h"
 #include "colors.h"
 #include "fuzzy_match.h"
-#include "jump.h"
-#include "messages.h"
-#include "navigation.h" /* fastback() */
-#include "readline.h"
-#include "builtins.h"
-#include "prompt.h"
-
 #ifndef _NO_HIGHLIGHT
 # include "highlight.h"
 #endif /* !_NO_HIGHLIGHT */
+#include "jump.h"
+#include "messages.h"
+#include "navigation.h" /* fastback() */
+#include "prompt.h"
+#include "readline.h"
 
 #define NO_MATCH      0
 #define PARTIAL_MATCH 1
@@ -101,6 +100,7 @@ check_int_cmd_desc(const char *s, const size_t l)
 		if (*s == 'd') return D_DESC;
 		if (*s == 'f') return F_DESC;
 		if (*s == 'j') return J_DESC;
+		if (*s == 'k') return K_DESC;
 		if (*s == 'l') return L_DESC;
 		if (*s == 'm') return M_DESC;
 		if (*s == 'n') return N_DESC;
@@ -156,6 +156,7 @@ check_int_cmd_desc(const char *s, const size_t l)
 
 		else if (*s == 'k') {
 			if (*(s + 1) == 'b') return KB_DESC;
+			if (*(s + 1) == 'k') return KK_DESC;
 		}
 
 		else if (*s == 'l') {
@@ -383,7 +384,7 @@ recover_from_wrong_cmd(void)
 	|| (words_num > 1 && point_is_first_word == 0))) {
 		char *p = (strrchr(rl_line_buffer, ' '));
 		if (p && p != rl_line_buffer && *(p - 1) != '\\' && *(p + 1) != ' ')
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 	}
 
 	fputs(NC, stdout);
@@ -401,7 +402,7 @@ recover_from_wrong_cmd(void)
 #endif /* !_NO_HIGHLIGHT */
 	wrong_cmd = 0;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* This function is only used before running a keybind command. We don't
@@ -487,7 +488,7 @@ calculate_suggestion_lines(int *baej, const size_t suggestion_len)
 	 * the current line */
 	size_t slines = 1;
 
-	if (cucs > term_cols) {
+	if (cucs > term_cols && term_cols > 0) {
 		slines = cucs / (size_t)term_cols;
 		const int cucs_rem = (int)cucs % term_cols;
 		if (cucs_rem > 0)
@@ -536,7 +537,7 @@ check_conditions(const size_t offset, const size_t wlen, int *baej,
 	size_t *slines)
 {
 	if (offset > wlen)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Do not print suggestions bigger than what the current terminal
 	 * window size can hold. If length is zero (invalid wide char), or if
@@ -546,14 +547,14 @@ check_conditions(const size_t offset, const size_t wlen, int *baej,
 	const size_t suggestion_len = wlen - offset;
 	if (suggestion_len == 0 || suggestion_len == ARG_MAX
 	|| (int)suggestion_len > (term_cols * term_lines) - curcol)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	*slines = calculate_suggestion_lines(baej, suggestion_len - 1);
 
 	if (*slines > (size_t)term_lines || (xargs.vt100 == 1 && *slines > 1))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static inline void
@@ -629,7 +630,7 @@ print_suggestion(const char *str, size_t offset, char *color)
 
 	const size_t str_len = wc_xstrlen(str);
 	size_t slines = 0;
-	if (check_conditions(offset, str_len, &baej, &slines) == EXIT_FAILURE) {
+	if (check_conditions(offset, str_len, &baej, &slines) == FUNC_FAILURE) {
 		UNHIDE_CURSOR;
 		return;
 	} else {
@@ -694,7 +695,7 @@ get_reg_file_color(const char *filename, const struct stat *attr,
 		return fi_c;
 
 	color_len += 4;
-	char *ext_color = (char *)xnmalloc(color_len, sizeof(char));
+	char *ext_color = xnmalloc(color_len, sizeof(char));
 	snprintf(ext_color, color_len, "\x1b[%sm", extcolor);
 
 	*free_color = 1;
@@ -1381,7 +1382,7 @@ check_jcmd(char *line)
 		clear_suggestion(CS_FREEBUF);
 
 	/* Split line into an array of substrings */
-	char **substr = line ? get_substr(line, ' ') : (char **)NULL;
+	char **substr = line ? get_substr(line, ' ', 0) : (char **)NULL;
 	if (!substr)
 		return NO_MATCH;
 
@@ -1471,6 +1472,9 @@ check_variables(const char *str, const size_t len)
 			continue;
 
 		char *ret = strchr(environ[i], '=');
+		if (!ret)
+			continue;
+
 		*ret = '\0';
 		suggestion.type = VAR_SUG;
 		char t[NAME_MAX + 1];
@@ -1633,7 +1637,8 @@ check_sort_methods(const char *str, const size_t len)
 	}
 
 	int a = atoi(str);
-	if (a < 0 || a > SORT_TYPES) {
+	if (a < 0 || a > SORT_TYPES
+	|| (conf.light_mode == 1 && !ST_IN_LIGHT_MODE(a))) {
 		if (suggestion.printed)
 			clear_suggestion(CS_FREEBUF);
 		return NO_MATCH;
@@ -1653,7 +1658,7 @@ check_prompts(char *word, const size_t len)
 	size_t l = len;
 
 	if (strchr(word, '\\')) {
-		q = dequote_str(word, 0);
+		q = unescape_str(word, 0);
 		w = q ? q : word;
 		l = w == q ? strlen(w) : len;
 	}
@@ -1685,11 +1690,11 @@ get_last_word(const char *last_space)
 	const char *tmp = (last_space && *(last_space + 1)) ? last_space + 1
 			: (rl_line_buffer ? rl_line_buffer : (char *)NULL);
 	if (tmp) {
-		size_t len = strlen(tmp);
-		last_word = (char *)xrealloc(last_word, (len + 1) * sizeof(char));
+		const size_t len = strlen(tmp);
+		last_word = xnrealloc(last_word, len + 1, sizeof(char));
 		xstrsncpy(last_word, tmp, len + 1);
 	} else {
-		last_word = (char *)xrealloc(last_word, 1 * sizeof(char));
+		last_word = xnrealloc(last_word, 1, sizeof(char));
 		*last_word = '\0';
 	}
 
@@ -1718,7 +1723,7 @@ check_workspaces(char *word, size_t wlen)
 	size_t l = wlen;
 
 	if (strchr(word, '\\')) {
-		q = dequote_str(word, 0);
+		q = unescape_str(word, 0);
 		w = q ? q : word;
 		l = w == q ? strlen(w) : wlen;
 	}
@@ -1780,7 +1785,7 @@ check_profiles(char *word, const size_t len)
 	size_t l = len;
 
 	if (strchr(word, '\\')) {
-		q = dequote_str(word, 0);
+		q = unescape_str(word, 0);
 		w = q ? q : word;
 		l = w == q ? strlen(w) : len;
 	}
@@ -1816,7 +1821,7 @@ check_remotes(char *word, const size_t len)
 	size_t l = len;
 
 	if (strchr(word, '\\')) {
-		q = dequote_str(word, 0);
+		q = unescape_str(word, 0);
 		w = q ? q : word;
 		l = w == q ? strlen(w) : len;
 	}
@@ -1851,7 +1856,7 @@ check_color_schemes(char *word, const size_t len)
 	size_t l = len;
 
 	if (strchr(word, '\\')) {
-		q = dequote_str(word, 0);
+		q = unescape_str(word, 0);
 		w = q ? q : word;
 		l = w == q ? strlen(w) : len;
 	}
@@ -1887,7 +1892,7 @@ check_bookmark_names(char *word, const size_t len)
 	size_t l = len - prefix;
 
 	if (strchr(word + prefix, '\\')) {
-		q = dequote_str(word + prefix, 0);
+		q = unescape_str(word + prefix, 0);
 		w = q ? q : word + prefix;
 		l = w == q ? strlen(w) : len;
 	}
@@ -1936,7 +1941,7 @@ check_backdir(void)
 		*q = '\0';
 
 	char *lb = rl_line_buffer + 3;
-	char *ds = strchr(lb, '\\') ? dequote_str(lb, 0) : lb;
+	char *ds = strchr(lb, '\\') ? unescape_str(lb, 0) : lb;
 
 	/* Find the query string in the list of parent directories */
 	char *p = conf.case_sens_path_comp == 1 ? strstr(bk_cwd, ds)
@@ -2009,7 +2014,7 @@ rl_suggestions(const unsigned char c)
 		/* No suggestion at all if comment */
 		if (suggestion.printed)
 			clear_suggestion(CS_FREEBUF);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	int printed = 0, zero_offset = 0;
@@ -2021,7 +2026,7 @@ rl_suggestions(const unsigned char c)
 		suggestion_buf = (char *)NULL;
 		if (wrong_cmd)
 			recover_from_wrong_cmd();
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	suggestion.full_line_len = (size_t)rl_end + 1;
@@ -2072,6 +2077,7 @@ rl_suggestions(const unsigned char c)
 		goto SUCCESS;
 	}
 
+	/* First word is an assignment */
 	if (c == '=' && words_num == 1 && wrong_cmd == 1) {
 		recover_from_wrong_cmd();
 		goto SUCCESS;
@@ -2097,7 +2103,7 @@ rl_suggestions(const unsigned char c)
 		 * ######################################*/
 
 	/* Fastback */
-	if (word && *word == '.' && word[1] == '.' && word[2] == '.'
+	if (word && *word == '.' && word[1] == '.' && (!word[2] || word[2] == '.')
 	&& (printed = check_fastback(word)) != NO_MATCH)
 		goto SUCCESS;
 
@@ -2523,8 +2529,11 @@ rl_suggestions(const unsigned char c)
 	}
 
 	/* 3.f) Cmds in PATH and CliFM internals cmds, but only for the first word. */
-	if (words_num > 1)
+	if (words_num > 1) {
+		if (suggestion_buf)
+			clear_suggestion(CS_FREEBUF);
 		goto NO_SUGGESTION;
+	}
 
 CHECK_FIRST_WORD:
 
@@ -2585,7 +2594,7 @@ CHECK_FIRST_WORD:
 	} else {
 	/* There's no suggestion nor any command name matching the first entered
 	 * word. So, we assume we have an invalid command name. Switch to the
-	 * warning prompt to warn the user */
+	 * warning prompt to warn the user. */
 		if (*word != '/' || strchr(word + 1, '/'))
 			print_warning_prompt(*word, c);
 	}
@@ -2647,7 +2656,7 @@ SUCCESS:
 	free(last_word);
 	last_word = (char *)NULL;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 
 FAIL:
 	if (suggestion.printed == 1)
@@ -2657,7 +2666,7 @@ FAIL:
 	free(last_word);
 	last_word = (char *)NULL;
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 #else
 void *_skip_me_suggestions;
