@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 
 /* The following functions are taken from Bash (1.14.7), licensed GPL-1.0-or-later,
  * and modified if needed:
- * PUTX
  * stat_char
  * get_y_or_n
  * print_filename
@@ -54,47 +53,35 @@ typedef char *rl_cpvfunc_t;
 #include <errno.h>
 
 #include <termios.h> /* Get cursor position functions */
-#include <limits.h> /* INT_MIN */
 
-#include "exec.h"
 #include "aux.h"
-#include "misc.h"
 #include "checks.h"
 #include "colors.h"
+#ifndef _NO_HIGHLIGHT
+# include "highlight.h"
+#endif /* !_NO_HIGHLIGHT */
+#include "misc.h"
 #include "navigation.h"
 #include "readline.h"
 #include "selection.h"
 #include "sort.h"
-
-#ifndef _NO_HIGHLIGHT
-# include "highlight.h"
-#endif /* !_NO_HIGHLIGHT */
-
+#include "spawn.h"
+#include "strings.h" /* quote_str() */
 #ifndef _NO_SUGGESTIONS
 # include "suggestions.h"
 #endif /* !_NO_SUGGESTIONS */
 
-#include "strings.h" /* quote_str() */
-
 #define CPR     "\x1b[6n" /* Cursor position report */
 #define CPR_LEN (sizeof(CPR) - 1)
 
 #define SHOW_PREVIEWS(c) ((c) == TCMP_PATH || (c) == TCMP_SEL \
 || (c) == TCMP_RANGES || (c) == TCMP_DESEL || (c) == TCMP_JUMP \
 || (c) == TCMP_TAGS_F || (c) == TCMP_GLOB || (c) == TCMP_FILE_TYPES_FILES \
-|| (c) == TCMP_BM_PATHS)
-
-#define PUTX(c) \
-	if (CTRL_CHAR(c)) { \
-          putc('^', rl_outstream); \
-          putc(UNCTRL(c), rl_outstream); \
-	} else if (c == RUBOUT) { \
-		putc('^', rl_outstream); \
-		putc('?', rl_outstream); \
-	} else \
-		putc(c, rl_outstream)
+|| (c) == TCMP_BM_PATHS || (c) == TCMP_UNTRASH || (c) == TCMP_TRASHDEL)
 
 #ifndef _NO_FZF
+static char finder_in_file[PATH_MAX + 1];
+static char finder_out_file[PATH_MAX + 1];
 
 /* We need to know the longest entry (if previewing files) to correctly
  * calculate the width of the preview window. */
@@ -104,6 +91,8 @@ static size_t longest_prev_entry;
  * (both vertical and horizontal), needed by TAB completion in fzf mode
  * with previews enabled */
 
+static struct termios orig_termios;
+
 /* Set the terminal into raw mode. Return 0 on success and -1 on error */
 static int
 enable_raw_mode(const int fd)
@@ -139,15 +128,15 @@ enable_raw_mode(const int fd)
 
 FAIL:
 	errno = ENOTTY;
-	return -1;
+	return (-1);
 }
 
 static int
 disable_raw_mode(const int fd)
 {
-	if (tcsetattr(fd, TCSAFLUSH, &orig_termios) != -1)
-		return EXIT_SUCCESS;
-	return EXIT_FAILURE;
+	if (tcsetattr(fd, TCSAFLUSH, &orig_termios) == -1)
+		return FUNC_FAILURE;
+	return FUNC_SUCCESS;
 }
 
 /* Use the "ESC [6n" escape sequence to query the cursor position (both
@@ -159,11 +148,11 @@ get_cursor_position(int *c, int *l)
 	char buf[32];
 	unsigned int i = 0;
 
-	if (enable_raw_mode(STDIN_FILENO) == -1) return EXIT_FAILURE;
+	if (enable_raw_mode(STDIN_FILENO) == -1) return FUNC_FAILURE;
 
 	/* 1. Ask the terminal about cursor position */
 	if (write(STDOUT_FILENO, CPR, CPR_LEN) != CPR_LEN)
-		{ disable_raw_mode(STDIN_FILENO); return EXIT_FAILURE; }
+		{ disable_raw_mode(STDIN_FILENO); return FUNC_FAILURE; }
 
 	/* 2. Read the response: "ESC [ rows ; cols R" */
 	int read_err = 0;
@@ -177,21 +166,21 @@ get_cursor_position(int *c, int *l)
 	buf[i] = '\0';
 
 	if (disable_raw_mode(STDIN_FILENO) == -1 || read_err == 1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* 3. Parse the response */
 	if (*buf != KEY_ESC || *(buf + 1) != '[' || !*(buf + 2))
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	char *p = strchr(buf + 2, ';');
-	if (!p || !*(p + 1)) return EXIT_FAILURE;
+	if (!p || !*(p + 1)) return FUNC_FAILURE;
 
 	*p = '\0';
 	*l = atoi(buf + 2); *c = atoi(p + 1);
 	if (*l == INT_MIN || *c == INT_MIN)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 #endif /* !_NO_FZF */
 
@@ -259,7 +248,7 @@ get_y_or_n(void)
 		int c = fgetc(stdin);
 		if (c == 'y' || c == 'Y' || c == ' ')
 			return (1);
-		 if (c == 'n' || c == 'N' || c == RUBOUT || c == EOF) {
+		if (c == 'n' || c == 'N' || c == RUBOUT || c == EOF) {
 			putchar('\n');
 			return (0);
 		}
@@ -282,9 +271,8 @@ print_filename(char *to_print, char *full_pathname)
 	|| t == TCMP_TRASHDEL)) {
 		colors_list(to_print, NO_ELN, NO_PAD, NO_NEWLINE);
 	} else {
-		for (s = to_print + tab_offset; *s; s++) {
-			PUTX(*s);
-		}
+		for (s = to_print + tab_offset; *s; s++)
+			putchar(*s);
 	}
 
 	if (rl_filename_completion_desired && conf.colorize == 0) {
@@ -308,7 +296,7 @@ print_filename(char *to_print, char *full_pathname)
 
 			size_t slen = strlen(s);
 			size_t tlen = strlen(to_print);
-			char *new_full_pathname = (char *)xnmalloc(slen + tlen + 2, sizeof(char));
+			char *new_full_pathname = xnmalloc(slen + tlen + 2, sizeof(char));
 			snprintf(new_full_pathname, slen + tlen + 2, "%s/%s", s, to_print);
 
 			extension_char = stat_char(new_full_pathname);
@@ -436,6 +424,10 @@ get_entry_color(char *entry, const char *norm_prefix)
 	if (conf.colorize == 0)
 		return (char *)NULL;
 
+	char vt_file[PATH_MAX + 1]; *vt_file = '\0';
+	if (virtual_dir == 1 && is_file_in_cwd(entry))
+		xreadlink(XAT_FDCWD, entry, vt_file, sizeof(vt_file));
+
 	struct stat attr;
 
 	/* Normalize URI file scheme */
@@ -443,9 +435,9 @@ get_entry_color(char *entry, const char *norm_prefix)
 	if (len > FILE_URI_PREFIX_LEN && IS_FILE_URI(entry))
 		entry += FILE_URI_PREFIX_LEN;
 
-	if (norm_prefix) {
+	if (norm_prefix && !*vt_file) {
 		char *s = strrchr(entry, '/');
-		char tmp[PATH_MAX];
+		char tmp[PATH_MAX + 1];
 		snprintf(tmp, sizeof(tmp), "%s/%s", norm_prefix,
 			(s && *(++s)) ? s : entry);
 		if (lstat(tmp, &attr) != -1)
@@ -458,8 +450,9 @@ get_entry_color(char *entry, const char *norm_prefix)
 	if ( (*entry == '/' || (*entry == '.' && entry[1] == '/') )
 	&& (t == TCMP_PATH || t == TCMP_SEL || t == TCMP_DESEL
 	|| t == TCMP_BM_PATHS || t == TCMP_DIRHIST || t == TCMP_JUMP) ) {
-		if (lstat(entry, &attr) != -1)
-			return fzftab_color(entry, &attr);
+		char *f = *vt_file ? vt_file : entry;
+		if (lstat(f, &attr) != -1)
+			return fzftab_color(f, &attr);
 		return uf_c;
 	}
 
@@ -477,17 +470,24 @@ get_entry_color(char *entry, const char *norm_prefix)
 	}
 
 	if (t == TCMP_PATH || t == TCMP_RANGES) {
-		char tmp[PATH_MAX];
-		snprintf(tmp, sizeof(tmp), "%s/%s", workspaces[cur_ws].path, entry);
-		if (lstat(tmp, &attr) != -1)
-			return fzftab_color(tmp, &attr);
+		if (*vt_file) {
+			if (lstat(vt_file, &attr) != -1)
+				return fzftab_color(vt_file, &attr);
+		} else {
+			char tmp[PATH_MAX + 1];
+			snprintf(tmp, sizeof(tmp), "%s/%s", workspaces[cur_ws].path, entry);
+			if (lstat(tmp, &attr) != -1)
+				return fzftab_color(tmp, &attr);
+		}
+
 		return uf_c;
 	}
 
 	if (t == TCMP_UNTRASH || t == TCMP_TRASHDEL || t == TCMP_GLOB
 	|| t == TCMP_TAGS_F || t == TCMP_FILE_TYPES_FILES) {
-		if (lstat(entry, &attr) != -1)
-			return fzftab_color(entry, &attr);
+		char *f = (t == TCMP_GLOB && *vt_file) ? vt_file : entry;
+		if (lstat(f, &attr) != -1)
+			return fzftab_color(f, &attr);
 		return uf_c;
 	}
 
@@ -522,10 +522,10 @@ get_last_input_word(void)
 	return lastword;
 }
 
-/* Store the deescaped string STR into the buffer BUF (only up to MAX bytes).
+/* Store the unescaped string STR into the buffer BUF (only up to MAX bytes).
  * Returns the number of copied bytes. */
 static size_t
-deescape_word(char *str, char *buf, const size_t max)
+unescape_word(char *str, char *buf, const size_t max)
 {
 	size_t i = 0;
 	char *p = str;
@@ -564,10 +564,8 @@ append_ending_char(const enum comp_type ct)
 	size_t deq_str_len = 0;
 
 	if (strchr(lastword, '\\'))
-		deq_str_len = deescape_word(lastword, deq_str, sizeof(deq_str) - 1);
+		deq_str_len = unescape_word(lastword, deq_str, sizeof(deq_str) - 1);
 
-	char _path[PATH_MAX + NAME_MAX];
-	*_path = '\0';
 	char *tmp = *deq_str ? deq_str : lastword;
 
 	size_t len = tmp == deq_str ? deq_str_len : strlen(tmp);
@@ -701,7 +699,8 @@ get_last_word(char *str, const int original_query)
 static void
 set_fzf_env_vars(const int height)
 {
-	int col = 0, line = 0;
+	int col = 0;
+	int line = 0;
 
 	if (!(flags & PREVIEWER) && term_caps.req_cur_pos == 1) {
 		get_cursor_position(&col, &line);
@@ -711,7 +710,9 @@ set_fzf_env_vars(const int height)
 
 	/* Let's correct image coordinates on the screen based on the preview
 	 * window style. */
-	int x = term_cols, y = line;
+	int x = term_cols;
+	int y = line;
+
 	switch (fzf_preview_border_type) {
 	case FZF_BORDER_BOTTOM: /* fallthrough */
 	case FZF_BORDER_NONE:   /* fallthrough */
@@ -766,12 +767,32 @@ get_preview_win_width(const int offset)
 	return (size_t)-1;
 }
 
+/* Change to the trash directory so that we can generate file previews.
+ * Only for the 'u' and 't del' commands. */
+static int
+cd_trashdir(const int prev)
+{
+#ifndef _NO_TRASH
+	return (prev == 1 && (cur_comp_type == TCMP_UNTRASH
+	|| cur_comp_type == TCMP_TRASHDEL) && trash_files_dir
+	&& *trash_files_dir && trash_n > 0 && workspaces
+	&& cur_ws >= 0 && workspaces[cur_ws].path
+	&& xchdir(trash_files_dir, NO_TITLE) == 0);
+#else
+	UNUSED(prev);
+	return 0;
+#endif /* !_NO_TRASH */
+}
+
 static int
 run_finder(const size_t height, const int offset, const char *lw,
 	const int multi)
 {
 	int prev = (conf.fzf_preview > 0 && SHOW_PREVIEWS(cur_comp_type) == 1)
 		? FZF_INTERNAL_PREVIEWER : 0;
+
+	int restore_cwd = cd_trashdir(prev);
+
 	int prev_hidden = conf.fzf_preview == 2 ? 1 : 0;
 	if (conf.fzf_preview == FZF_EXTERNAL_PREVIEWER)
 		prev = FZF_EXTERNAL_PREVIEWER;
@@ -812,7 +833,7 @@ run_finder(const size_t height, const int offset, const char *lw,
 		/* All fixed parameters are compatible with at least fzf 0.16.11 (Aug 1, 2017) */
 		char prev_opts[18 + 32];
 		*prev_opts = '\0';
-		char prev_str[] = "--preview \"clifm --preview {}\"";
+		const char prev_str[] = "--preview \"clifm --preview {}\"";
 
 		if (prev > 0) { /* Either internal of external previewer */
 			set_fzf_env_vars((int)height);
@@ -861,7 +882,12 @@ ctrl-d:deselect-all,ctrl-t:toggle-all" : "",
 
 	int dr = (flags & DELAYED_REFRESH) ? 1 : 0;
 	flags &= ~DELAYED_REFRESH;
+	const mode_t old_mask = umask(0077);
 	int ret = launch_execl(cmd);
+	umask(old_mask);
+
+	if (restore_cwd == 1)
+		xchdir(workspaces[cur_ws].path, NO_TITLE);
 
 	/* Restore the user's shell to its original value. */
 	user.shell = shell_bk;
@@ -900,7 +926,7 @@ get_tagged_file_target(char *filename)
 	char dir[PATH_MAX];
 	char *p = (char *)NULL;
 	if (strchr(filename, '\\'))
-		p = dequote_str(filename, 0);
+		p = unescape_str(filename, 0);
 
 	snprintf(dir, sizeof(dir), "%s/%s/%s", tags_dir, cur_tag, p ? p : filename);
 	free(p);
@@ -941,7 +967,7 @@ get_glob_file_target(char *str, const char *initial_path)
 		return str;
 
 	size_t len = strlen(initial_path) + strlen(str) + 1;
-	char *p = (char *)xnmalloc(len, sizeof(char));
+	char *p = xnmalloc(len, sizeof(char));
 	snprintf(p, len, "%s%s", initial_path, str);
 
 	return p;
@@ -959,15 +985,17 @@ get_finder_output(const int multi, char *base)
 		return print_no_finder_file();
 	}
 
-	char *buf = (char *)xnmalloc(1, sizeof(char)), *line = (char *)NULL;
+	char *buf = xnmalloc(1, sizeof(char)), *line = (char *)NULL;
 	*buf = '\0';
 	size_t bsize = 0, line_size = 0;
 	ssize_t line_len = 0;
 	char *initial_path = (cur_comp_type == TCMP_GLOB) ? base : (char *)NULL;
 
 	while ((line_len = getline(&line, &line_size, fp)) > 0) {
-		if (line[line_len - 1] == '\n')
-			line[--line_len] = '\0';
+		if (line[line_len - 1] == '\n') {
+			line_len--;
+			line[line_len] = '\0';
+		}
 
 		if (cur_comp_type == TCMP_FILE_TYPES_OPTS && *line && *(line + 1)) {
 			*(line + 1) = '\0';
@@ -996,11 +1024,11 @@ get_finder_output(const int multi, char *base)
 				s = get_tagged_file_target(line);
 			} else if (cur_comp_type == TCMP_BM_PREFIX) {
 				size_t len = strlen(line) + 3;
-				s = (char *)xnmalloc(len, sizeof(char));
+				s = xnmalloc(len, sizeof(char));
 				snprintf(s, len, "b:%s", line);
 			} else if (cur_comp_type == TCMP_TAGS_T) {
 				size_t len = strlen(line) + 3;
-				s = (char *)xnmalloc(len, sizeof(char));
+				s = xnmalloc(len, sizeof(char));
 				snprintf(s, len, "t:%s", line);
 			}
 			q = escape_str(s);
@@ -1017,7 +1045,7 @@ get_finder_output(const int multi, char *base)
 
 		size_t qlen = (r != line) ? strlen(r) : (size_t)line_len;
 		bsize += qlen + 3;
-		buf = (char *)xrealloc(buf, bsize * sizeof(char));
+		buf = xnrealloc(buf, bsize, sizeof(char));
 		xstrncat(buf, strlen(buf), r, bsize);
 
 		if (multi == 1) {
@@ -1097,7 +1125,9 @@ store_completions(char **matches)
 		no_file_comp = 1; /* We're not completing file names. */
 
 	char *norm_prefix = (char *)NULL;
-	if (ct == TCMP_PATH && ((*matches[0] == '.' && matches[0][1] == '/')
+	/* "./_", "../_", and "_/.._" */
+	if (ct == TCMP_PATH && ((*matches[0] == '.' && (matches[0][1] == '/'
+	|| (matches[0][1] == '.' && matches[0][2] == '/')))
 	|| strstr(matches[0], "/..")))
 		norm_prefix = normalize_prefix(matches[0]);
 
@@ -1376,10 +1406,10 @@ static int
 clean_rl_buffer(const char *text)
 {
 	if (!text || !*text)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (rl_point != rl_end)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	/* If the previous char is not space, then a common prefix was appended:
 	 * remove it. */
@@ -1410,7 +1440,7 @@ clean_rl_buffer(const char *text)
 
 	rl_insert_text(text);
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Calculate the offset (left padding) of the finder's window based on
@@ -1456,10 +1486,13 @@ get_finder_offset(const char *query, const char *text, char **matches,
 	 * (readline.c)). */
 
 	if (text && conf.fuzzy_match == 1 && ct != TCMP_TAGS_F
-	&& ct != TCMP_DIRHIST)
+	&& ct != TCMP_DIRHIST) {
 		/* TEXT is not NULL whenever a common prefix was added, replacing
 		 * the original query string. */
-		finder_offset -= (int)(wc_xstrlen(matches[0]) - wc_xstrlen(text));
+		const size_t bslashes = count_chars(text, '\\');
+		finder_offset -=
+			(int)(wc_xstrlen(matches[0]) - wc_xstrlen(text) + bslashes);
+	}
 
 	if (!lw) {
 		finder_offset++;
@@ -1620,7 +1653,7 @@ do_some_cleanup(char **buf, char **matches, const char *query,
 	else if (ct == TCMP_OPENWITH && strchr(*buf, ' ')) {
 		/* We have multiple words ("CMD ARG..."): quote the string. */
 		size_t len = strlen(*buf) + 3;
-		char *tmp = (char *)xnmalloc(len, sizeof(char));
+		char *tmp = xnmalloc(len, sizeof(char));
 		snprintf(tmp, len, "\"%s\"", *buf);
 		free(*buf);
 		*buf = tmp;
@@ -1663,7 +1696,7 @@ do_some_cleanup(char **buf, char **matches, const char *query,
 	else if (ct == TCMP_USERS) {
 		size_t l = strlen(*buf);
 		char *p = savestring(*buf, l);
-		*buf = (char *)xrealloc(*buf, (l + 2) * sizeof(char));
+		*buf = xnrealloc(*buf, (l + 2), sizeof(char));
 		snprintf(*buf, l + 2, "~%s", p);
 		free(p);
 	}
@@ -1698,7 +1731,7 @@ do_completion(char *buf, const size_t prefix_len, const int multi)
 	&& cur_comp_type != TCMP_HIST && !multi) {
 		p = escape_str(buf);
 		if (!p)
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 	} else {
 		p = savestring(buf, blen);
 	}
@@ -1706,7 +1739,7 @@ do_completion(char *buf, const size_t prefix_len, const int multi)
 	write_completion(p, prefix_len, multi);
 	free(p);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Calculate currently used lines to go back to the correct cursor
@@ -1716,7 +1749,7 @@ move_cursor_up(const int total_line_len)
 {
 	int lines = 1;
 
-	if (total_line_len > term_cols) {
+	if (total_line_len > term_cols && term_cols > 0) {
 		lines = total_line_len / term_cols;
 		int rem = (int)total_line_len % term_cols;
 		if (rem > 0)
@@ -1747,12 +1780,12 @@ set_finder_paths(void)
 	const int sm = (xargs.stealth_mode == 1);
 	const char *p = sm ? P_tmpdir : tmp_dir;
 
-	char *rand_ext = gen_rand_str(sm ? 16 : 10);
+	char *rand_ext = gen_rand_str(sm ? RAND_SUFFIX_LEN + 6 : RAND_SUFFIX_LEN);
 	snprintf(finder_in_file, sizeof(finder_in_file), "%s/.temp%s",
 		p, rand_ext ? rand_ext : "a3_2yu!d43");
 	free(rand_ext);
 
-	rand_ext = gen_rand_str(sm ? 20 : 14);
+	rand_ext = gen_rand_str(sm ? RAND_SUFFIX_LEN + 10 : RAND_SUFFIX_LEN + 4);
 	snprintf(finder_out_file, sizeof(finder_out_file), "%s/.temp%s",
 		p, rand_ext ? rand_ext : "0rNkds7++@");
 	free(rand_ext);
@@ -1784,7 +1817,7 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 	/* Store possible completions in FINDER_IN_FILE to pass them to the finder. */
 	size_t num_matches = store_completions(matches);
 	if (num_matches == (size_t)-1)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Set a pointer to the last word in the query string. We use this to
 	 * highlight the matching prefix in the list of matches. */
@@ -1818,7 +1851,7 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 		q = (char *)NULL;
 	}
 
-	char *deq = q ? (strchr(q, '\\') ? dequote_str(q, 0) : q) : (char *)NULL;
+	char *deq = q ? (strchr(q, '\\') ? unescape_str(q, 0) : q) : (char *)NULL;
 
 	/* Run the finder application and store the ouput into FINDER_OUT_FILE. */
 	int ret = run_finder(height, finder_offset, deq, multi);
@@ -1832,7 +1865,7 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 		move_cursor_up(total_line_len);
 
 	/* No results (the user pressed ESC or the Left arrow key). */
-	if (ret != EXIT_SUCCESS) {
+	if (ret != FUNC_SUCCESS) {
 		unlink(finder_out_file);
 		return clean_rl_buffer(text);
 	}
@@ -1840,7 +1873,7 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 	/* Retrieve finder's output from FINDER_OUT_FILE. */
 	char *buf = get_finder_output(multi, matches[0]);
 	if (!buf)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	/* Calculate the length of the matching prefix to insert into the
 	 * line buffer only the non-matched part of the string returned by the
@@ -1850,9 +1883,9 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 	do_some_cleanup(&buf, matches, query, &prefix_len);
 
 	/* Let's insert the selected match(es): BUF. */
-	if (buf && *buf && do_completion(buf, prefix_len, multi) == EXIT_FAILURE) {
+	if (buf && *buf && do_completion(buf, prefix_len, multi) == FUNC_FAILURE) {
 		free(buf);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	free(buf);
@@ -1866,7 +1899,7 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 	}
 #endif /* !_NO_SUGGESTIONS */
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 #endif /* !_NO_FZF */
 
@@ -1893,7 +1926,7 @@ int
 tab_complete(const int what_to_do)
 {
 	if (rl_notab == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	if (*rl_line_buffer == '#' || cur_color == hc_c) {
 		/* No completion at all if comment */
@@ -1901,7 +1934,7 @@ tab_complete(const int what_to_do)
 		if (suggestion.printed)
 			clear_suggestion(CS_FREEBUF);
 #endif /* _NO_SUGGESTIONS */
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	rl_compentry_func_t *our_func = rl_completion_entry_function;
@@ -2011,7 +2044,7 @@ AFTER_USUAL_COMPLETION:
 	if (!matches || !matches[0]) {
 		rl_ring_bell();
 		free(text);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 #ifndef _NO_FZF
@@ -2057,7 +2090,7 @@ AFTER_USUAL_COMPLETION:
 
 		/* We have marked all the dead slots with (char *)&dead_slot
 		 * Copy all the non-dead entries into a new array. */
-		temp_array = (char **)xnmalloc(3 + newlen, sizeof (char *));
+		temp_array = xnmalloc(3 + newlen, sizeof (char *));
 		for (i = j = 1; matches[i]; i++) {
 			if (matches[i] != (char *)&dead_slot) {
 				temp_array[j] = matches[i];
@@ -2378,7 +2411,7 @@ AFTER_USUAL_COMPLETION:
 
 DISPLAY_MATCHES:
 #ifndef _NO_FZF
-		if (fzftab != 1) {
+		if (fzftab != 1)
 #endif /* !_NO_FZF */
 		{
 			max = 0;
@@ -2418,16 +2451,12 @@ DISPLAY_MATCHES:
 			if (limit != 1 && (limit * max == term_cols))
 				limit--;
 
-			if (limit == 0)
+			if (limit <= 0)
 			  limit = 1;
 
 			/* How many iterations of the printing loop? */
 			count = (len + (limit - 1)) / limit;
-
 		}
-#ifndef _NO_FZF
-		}
-#endif /* !_NO_FZF */
 
 		putchar('\n');
 #ifndef _NO_HIGHLIGHT
@@ -2464,7 +2493,7 @@ DISPLAY_MATCHES:
 				dd = normalize_path(matches[0], strlen(matches[0]));
 				if (dd) {
 					size_t ddlen = strlen(dd) + 2;
-					char *ddd = (char *)xnmalloc(ddlen, sizeof(char *));
+					char *ddd = xnmalloc(ddlen, sizeof(char *));
 					snprintf(ddd, ddlen, "%s/", dd);
 					free(dd);
 					dd = ddd;
@@ -2503,6 +2532,8 @@ CALC_OFFSET:
 				goto RESTART;
 			goto RESET_PATH;
 		}
+#else
+		if (1) {} /* This just silences a warning */
 #endif /* !_NO_FZF */
 
 		/* Standard TAB completion */
@@ -2665,7 +2696,7 @@ RESTART:
 	default:
 		xerror("\r\nreadline: %c: Bad value for what_to_do "
 			"in tab_complete\n", what_to_do);
-		exit(EXIT_FAILURE);
+		exit(FUNC_FAILURE);
 	}
 
 	for (i = 0; matches[i]; i++)
@@ -2673,5 +2704,5 @@ RESTART:
 	free(matches);
 	free(text);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
