@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  * 
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,12 +25,10 @@
 #ifndef TRASH_H
 #define TRASH_H
 
-#include <time.h>
-
 __BEGIN_DECLS
 
-int trash_function(char **);
-int untrash_function(char **);
+int trash_function(char **args);
+int untrash_function(char **args);
 
 __END_DECLS
 
